#!/bin/ksh
set -e

# Lorenz model
echo
echo "-------------------"
echo " Running L95 tests"
echo "-------------------"
echo

cd test
mkdir -p Data

../../src/test_util.x --log_level=warning

rm -f l95.*.out
echo "Running L95 truth"
../l95_forecast.x truth.xml > l95.truth.out
echo "Running L95 background forecast"
../l95_forecast.x forecast.xml > l95.forecast.out

echo "Generate L95 3D-Var observations from the truth"
../l95_makeobs.x makeobs3d.xml > l95.makeobs3d.out
echo "Generate L95 4D-Var observations from the truth"
../l95_makeobs.x makeobs4d.xml > l95.makeobs4d.out

../l95_test.x test.xml

echo "Running L95 incremental 3D-Var"
../l95_4dvar.x 3dvar.xml > l95.3dvar.out
echo "Running L95 incremental 4D-Var"
../l95_4dvar.x 4dvar.xml > l95.4dvar.out
echo "Running L95 weak constraint 4D-Var - forcing formulation"
../l95_4dvar.x 4dforcing.xml > l95.4dforcing.out
echo "Running L95 weak constraint 4D-Var - saddlepoint formulation"
../l95_4dvar.x 4dsaddlepoint.xml > l95.4dsaddlepoint.out
echo "Running L95 4D-Var - RPCG"
../l95_4dvar.x 4dvar.rpcg.xml > l95.rpcg.out

echo "Generating L95 ensemble perturbations"
../l95_genpert.x genenspert.xml > l95.genpert.out
echo "Running L95 4D-Var - hybrid formulation"
../l95_4dvar.x 4dhybrid.xml > l95.4dhybrid.out

echo "Generate L95 observations from the truth"
../l95_makeobs.x makeobs4d12h.xml > l95.makeobs4d12h.out
echo "Running L95 background forecast for 4D-Ens-Var"
../l95_forecast.x bgensfc.xml > l95.bgensfc.out
echo "Running L95 4D-Ens-Var"
../l95_4dvar.x 4densvar.xml > l95.4densvar.out

echo
echo "-------------------"
echo " Comparing results"
echo "-------------------"
echo

egrep 'Nonlinear J|reduction in residual norm' *.out > exp.summary.txt
diff -s ../control.summary.txt exp.summary.txt
rm -f exp.summary.txt

