/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "ErrorCovarianceL95.h"

#include "logger/Logger.h"
#include "util/Config.h"
#include "util/asser1.h"

#include "lorenz95/FieldL95.h"
#include "lorenz95/IncrementL95.h"
#include "lorenz95/L95Fortran.h"
#include "lorenz95/StateL95.h"

// -----------------------------------------------------------------------------
namespace lorenz95 {
// -----------------------------------------------------------------------------
ErrorCovarianceL95::ErrorCovarianceL95(const oops::Geometry & geom,
                                       const util::Config & config) {
  time_ = util::DateTime(config.getData("date"));
  double sigmab = config.getReal("standard_deviation");
  double lscale = config.getReal("length_scale");
  const int resol = geom.config().getInt("resol");
  l95bkg_setup_f90(&ftnConfig_, sigmab, lscale, resol);

  const util::Config noconf(config, util::ConfigEmpty());
  novars_.reset(new oops::Variables(noconf));
}
// -----------------------------------------------------------------------------
ErrorCovarianceL95::~ErrorCovarianceL95() {
  l95bkg_delete_f90(&ftnConfig_);
}
// -----------------------------------------------------------------------------
void ErrorCovarianceL95::linearize(const StateL95 &, const oops::Geometry & resol,
                                   const util::Config & tlm) {
  resol_.reset(new oops::Geometry(resol));
  tlm_.reset(new util::Config(tlm));
  LOG(Trace) << "ErrorCovarianceL95 linearized.";
}
// -----------------------------------------------------------------------------
void ErrorCovarianceL95::mult(const IncrementL95 & dxin,
                              IncrementL95 & dxout) const {
  const FieldF90 * dxi = dxin.getFields();
  FieldF90 * dxo = dxout.getFields();
  l95bkg_mult_f90(&dxi, &dxo, &ftnConfig_);
}
// -----------------------------------------------------------------------------
void ErrorCovarianceL95::inv_mult(const IncrementL95 & dxin,
                                  IncrementL95 & dxout) const {
  const FieldF90 * dxi = dxin.getFields();
  FieldF90 * dxo = dxout.getFields();
  l95bkg_inv_mult_f90(&dxi, &dxo, &ftnConfig_);
}
// -----------------------------------------------------------------------------
void ErrorCovarianceL95::randomize(IncrementL95 & dx) const {
  FieldF90 * fdx = dx.getFields();
  l95bkg_randomize_f90(&ftnConfig_, &fdx);
}
// -----------------------------------------------------------------------------

}  // namespace lorenz95
