/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef L95_SRC_ERRORCOVARIANCEL95_H_
#define L95_SRC_ERRORCOVARIANCEL95_H_

#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include "oops/base/Geometry.h"
#include "oops/base/Variables.h"
#include "util/Config.h"
#include "util/DateTime.h"

#include "lorenz95/L95Fortran.h"

// Forward declarations
namespace lorenz95 {
  class IncrementL95;
  class StateL95;

/// Background error covariance matrix for Lorenz 95 model.
/*!
 *  Background error covariance matrix for Lorenz 95 model
 *  is Gaussian.
 */

// -----------------------------------------------------------------------------
class ErrorCovarianceL95 : private boost::noncopyable {
 public:
  ErrorCovarianceL95(const oops::Geometry &, const util::Config &);
  ~ErrorCovarianceL95();

  void linearize(const StateL95 &, const oops::Geometry &, const util::Config &);

  void mult(const IncrementL95 &, IncrementL95 &) const;
  void inv_mult(const IncrementL95 &, IncrementL95 &) const;

  void randomize(IncrementL95 &) const;

  const util::DateTime & validTime() const {return time_;}
  const oops::Geometry & resolution() const {return *resol_;}
  const oops::Variables & ctrlvars() const {return *novars_;}

 private:
  F90bmat * ftnConfig_;
  boost::scoped_ptr<const util::Config> tlm_;
  boost::scoped_ptr<const oops::Geometry> resol_;
  boost::scoped_ptr<const oops::Variables> novars_;
  util::DateTime time_;
};
// -----------------------------------------------------------------------------
}  // namespace lorenz95

#endif  // L95_SRC_ERRORCOVARIANCEL95_H_
