/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef L95_SRC_FIELDL95_H_
#define L95_SRC_FIELDL95_H_

#include <string>

#include "util/DateTime.h"
#include "util/Duration.h"

// Forward declarations
namespace oops {
  class Geometry;
}

namespace util {
  class Config;
}

namespace lorenz95 {
  class LocsL95;
  class GomL95;

/// Empty class to give a type to fortran type l95fields::l95field
class FieldF90 {};

// -----------------------------------------------------------------------------
/// Class to represent fields for the L95 model
class FieldL95 {
 public:
// Constructors and basic operators
  FieldL95(const oops::Geometry &, const util::DateTime &);
  FieldL95(const FieldL95 &);
  virtual ~FieldL95();
  FieldL95 & operator=(const FieldL95 &);

// Interpolate full field
  void changeResolution(const FieldL95 &);

// Utilities
  void read(const util::Config &);
  void write(const util::Config &) const;
  const util::DateTime validTime() const {return time_;}
  void prtNorms(const std::string &) const;

 protected:
  void zero();

// Interpolate to given location
  void interp(const LocsL95 &, GomL95 &) const;
  void interpAD(const LocsL95 &, const GomL95 &);

// Update fields
  void add(const FieldL95 &);
  void diff(const FieldL95 &, const FieldL95 &);
  void axpy(const double &, const FieldL95 &, const bool);

// Update time
  void timeUpdate(const util::Duration & dt) {time_ += dt;}

// Utilities
  FieldF90 * toFortran() {return flds_;}
  const FieldF90 * toFortran() const {return flds_;}

 private:
  int resol_;
  FieldF90 * flds_;
  util::DateTime time_;
};
// -----------------------------------------------------------------------------
}  // namespace lorenz95

#endif  // L95_SRC_FIELDL95_H_
