/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "lorenz95/GomL95.h"

#include <cstdlib>
#include <vector>

#include "logger/Logger.h"
#include "util/asser1.h"

namespace lorenz95 {

// -----------------------------------------------------------------------------
GomL95::GomL95(const std::vector<int> & mask)
  : size_(mask.size()), iobs_(mask), locval_(size_), current_(0)
{
  ASSERT(size_>0);
  LOG(Trace)  << "GomL95 size=" << size_;
}
// -----------------------------------------------------------------------------
GomL95::~GomL95() {}
// -----------------------------------------------------------------------------
void GomL95::zero() {
  for (int jj = 0; jj < size_; ++jj) locval_[jj] = 0.0;
}
// -----------------------------------------------------------------------------
void GomL95::random() {
//  static unsigned int * ii;
//  *ii = 12345;
  for (int jj = 0; jj < size_; ++jj)
//    locval_[jj] = 2.0 * (rand_r(ii)/static_cast<double>(RAND_MAX)-0.5);
    locval_[jj] = 2.0 * (rand()/static_cast<double>(RAND_MAX)-0.5);
}
// -----------------------------------------------------------------------------
double GomL95::dot_product_with(const GomL95 & gom) const {
  double zz = 0.0;
  for (int jj = 0; jj < size_; ++jj) zz += locval_[jj] * gom.locval_[jj];
  return zz;
}
// -----------------------------------------------------------------------------

}  // namespace lorenz95
