/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef L95_SRC_GOML95_H_
#define L95_SRC_GOML95_H_

#include<vector>

namespace lorenz95 {

/// GomL95 class to handle locations for L95 model.

class GomL95 {
 public:
  explicit GomL95(const std::vector<int> &);
  ~GomL95();

  double dot_product_with(const GomL95 &) const;
  void zero();
  void random();

  const double & operator()(const int ii) const {return locval_[ii];}
  double & operator()(const int ii) {return locval_[ii];}
  int getobs(const int il) const {return iobs_[il];}
  const int & current() const {return current_;}
  int nobs() const {return size_;}

  const double * toFortran() const {return &locval_[0];}
  double * toFortran() {return &locval_[0];}

 private:
  int size_;
  std::vector<int> iobs_;
  std::vector<double> locval_;
  int current_;
};

}  // namespace lorenz95

#endif  // L95_SRC_GOML95_H_
