/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef L95_SRC_INCREMENTL95_H_
#define L95_SRC_INCREMENTL95_H_

#include <string>

#include "oops/base/GeneralizedDeparture.h"
#include "util/dot_product.h"

#include "lorenz95/FieldL95.h"
#include "lorenz95/L95Fortran.h"
#include "lorenz95/ModelL95.h"

namespace oops {
  class Geometry;
  class Variables;
}

namespace util {
  class Config;
  class DateTime;
  class Duration;
}

namespace lorenz95 {
  class ErrorCovarianceL95;
  class GomL95;
  class LocsL95;
  class StateL95;
  class TrajectoryL95;
  class ModelError;

/// Increment Class: Difference between two states
/*!
 *  Some fields that are present in a State may not be present in
 *  an Increment. The Increment contains everything that is needed by
 *  the tangent-linear and adjoint models, and has methods to propagate
 *  itself using these models.
 */

// -----------------------------------------------------------------------------

class IncrementL95: public FieldL95, public oops::GeneralizedDeparture {
 public:

/// Constructor, destructor
  IncrementL95(const oops::Geometry &, const oops::Variables &, const util::DateTime &);
  IncrementL95(const IncrementL95 &);
  virtual ~IncrementL95();

/// Basic operators
  void diff(const StateL95 &, const StateL95 &);
  IncrementL95 & operator =(const IncrementL95 &);
  IncrementL95 & operator+=(const IncrementL95 &);
  IncrementL95 & operator-=(const IncrementL95 &);
  IncrementL95 & operator*=(const double &);
  void zero(const util::Duration & shift = util::Duration(0) );
  void random();
  void axpy(const double &, const IncrementL95 &, const bool check = true);
  double dot_product_with(const IncrementL95 &) const;
  void schur_product_with(const IncrementL95 &);

/// Interpolate to observation location
  void interpolateTL(const LocsL95 &, GomL95 &) const;
  void interpolateAD(const LocsL95 &, const GomL95 &);

/// Access to data... Could we do without that?
  FieldF90 * getFields() {return FieldL95::toFortran();}
  const FieldF90 * getFields() const {return FieldL95::toFortran();}

 protected:
  void initTL(const ModelL95 &, const TrajectoryL95 &);
  void initAD(const ModelL95 &, const TrajectoryL95 &);
  void stepTL(const ModelL95 &, const TrajectoryL95 &, const ModelError &);
  void stepAD(const ModelL95 &, const TrajectoryL95 &, ModelError &);

  void accumul(const double & zz, const StateL95 & xx);
};

// -----------------------------------------------------------------------------
}  // namespace lorenz95

#endif  // L95_SRC_INCREMENTL95_H_
