/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef L95_SRC_L95FORTRAN_H_
#define L95_SRC_L95FORTRAN_H_

// Forward declarations
namespace util {
  class Config;
}

namespace lorenz95 {

class TrajF90;
class FieldF90;

/// Empty class to give a type to fortran type L95BKGERR::l95_b_config
class F90bmat {};
class F90locmat {};
class ModelConfig {};

/// Interface to Fortran Lorenz 95 model
/*!
 * The core of the Lorenz 95 model is coded in Fortran.
 * Here we define the interfaces to the Fortran code.
 */

extern "C" {
// Model
  void l95_setup_f90(const util::Config * const *, const util::Config * const *,
                     ModelConfig **);
  void l95_delete_f90(ModelConfig ** c);
  void l95_propagate_f90(const ModelConfig * const *, FieldF90 **);
  void l95_propagate_tl_f90(const ModelConfig * const *, FieldF90 **, const TrajF90 **);
  void l95_propagate_ad_f90(const ModelConfig * const *, FieldF90 **, const TrajF90 **);

// Trajectory
  void l95_create_traj_f90(const ModelConfig * const *, FieldF90 **, TrajF90 **);
  void l95_delete_traj_f90(TrajF90 **);

// Background error
  void l95bkg_setup_f90(F90bmat **, const double &, const double &,
                        const int &);
  void l95bkg_delete_f90(F90bmat ** b);
  void l95bkg_mult_f90(const FieldF90 **, FieldF90 **, const F90bmat * const *);
  void l95bkg_inv_mult_f90(const FieldF90 **, FieldF90 ** f,
                           const F90bmat * const *);
  void l95bkg_randomize_f90(const F90bmat * const *, FieldF90 **);

// Localization matrix
  void l95loc_setup_f90(F90locmat **, const double &, const int &);
  void l95loc_delete_f90(F90locmat ** b);
  void l95loc_mult_f90(FieldF90 **, const F90locmat * const *);

// Fields manipulations
  void l95field_create_f90(FieldF90 **, const int &);
  void l95field_diff_f90(FieldF90 **, const FieldF90 **, const FieldF90 **);
  void l95field_delete_f90(FieldF90 ** f);
  void l95field_zero_f90(FieldF90 ** f);
  void l95field_copy_f90(FieldF90 **, const FieldF90 **);
  void l95field_add_f90(FieldF90 **, const FieldF90 **);
  void l95field_diff_incr_f90(FieldF90 **, const FieldF90 **, const FieldF90**);
  void l95field_sub_f90(FieldF90 **, const FieldF90 **);
  void l95field_mul_f90(FieldF90 **, const double &);
  void l95field_axpy_f90(FieldF90 **, const double &, const FieldF90 **);
  void l95field_schur_f90(FieldF90 **, const FieldF90 **);
  void l95field_prod_f90(const FieldF90 **, const FieldF90 **, double &);
  void l95field_resol_f90(FieldF90 **, const FieldF90 **);
  void l95field_read_f90(const int &, const char *, FieldF90 **, char *);
  void l95field_write_f90(const int &, const char *, const FieldF90 **,
                          const char *);
  void l95field_norm_f90(const FieldF90 **, double &, double &, double &);
  void l95field_interpol_tl_f90(const FieldF90 **, const int &, const double *,
                                const int &, double *, const int &);
  void l95field_interpol_ad_f90(FieldF90 **, const int &, const double *,
                                const int &, const double *, const int &);
  void l95field_random_f90(FieldF90 ** f);
}

}  // namespace lorenz95

#endif  // L95_SRC_L95FORTRAN_H_
