/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "LocalizationMatrixL95.h"

#include "logger/Logger.h"
#include "util/Config.h"
#include "util/asser1.h"

#include "FieldL95.h"
#include "IncrementL95.h"
#include "L95Fortran.h"

// -----------------------------------------------------------------------------
namespace lorenz95 {
// -----------------------------------------------------------------------------
LocalizationMatrixL95::LocalizationMatrixL95(const oops::Geometry & geom, const util::Config & config) {
  double lscale = config.getReal("length_scale");
  const int resol = geom.config().getInt("resol");

  l95loc_setup_f90(&ftnConfig_, lscale, resol);
}
// -----------------------------------------------------------------------------
LocalizationMatrixL95::~LocalizationMatrixL95() {
  l95loc_delete_f90(&ftnConfig_);
}

// -----------------------------------------------------------------------------
void LocalizationMatrixL95::multiply(IncrementL95 & dx) const {
  FieldF90 * dxf = dx.getFields();
  l95loc_mult_f90(&dxf, &ftnConfig_);
}

}  // namespace lorenz95
