/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef L95_SRC_MODELL95_H_
#define L95_SRC_MODELL95_H_

#include <boost/scoped_ptr.hpp>

#include "oops/base/Geometry.h"
#include "oops/base/Variables.h"
#include "util/Config.h"
#include "util/Duration.h"
#include "util/asser1.h"

#include "lorenz95/L95Fortran.h"

namespace lorenz95 {

/// Holds pointer to Fortran L95 model configuration.
/*!
 * This class is needed so an object can be created and managed by the
 * C++ layer, and the fortran data deallocated when the C++ object is
 * destructed.
 */

// -----------------------------------------------------------------------------

class ModelL95 {
 public:
  ModelL95(const oops::Geometry & resol, const util::Config & model)
    : ftnModel_(0), resol_(resol)
  {
    const util::Config * modConf = &model;
    const util::Config * resConf = &resol.config();
    l95_setup_f90(&modConf, &resConf, &ftnModel_);
    tstep_ = util::Duration(modConf->getData("tstep"));

    const util::Config noconf(model, util::ConfigEmpty());
    novars_.reset(new oops::Variables(noconf));
  }

  ~ModelL95() {l95_delete_f90(&ftnModel_);}

  ModelConfig * toFortran() {return ftnModel_;}
  const ModelConfig * toFortran() const {return ftnModel_;}

  const util::Duration & timestep() const {return tstep_;}
  const oops::Geometry & resolution() const {return resol_;}
  const oops::Variables & variables() const {return *novars_;}
  const oops::Variables & variablesTL() const {return *novars_;}

 private:
  ModelConfig * ftnModel_;
  util::Duration tstep_;
  const oops::Geometry resol_;
  boost::scoped_ptr<const oops::Variables> novars_;
};

// -----------------------------------------------------------------------------

}  // namespace l95

#endif  // L95_SRC_MODELL95_H_
