/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef L95_SRC_OBSTABLE_H_
#define L95_SRC_OBSTABLE_H_

#include <fstream>
#include <map>
#include <string>
#include <vector>

#include <boost/noncopyable.hpp>

#include "util/DateTime.h"

#include "lorenz95/ObsVec1D.h"

// Forward declarations
namespace util {
  class Config;
}

namespace lorenz95 {
  class LocsL95;

using std::ifstream;
using std::ofstream;
using util::DateTime;

/// A Simple Observation Data Handler
/*!
 *  ObsTable defines a simple observation handler
 *  that mimicks the interfaces required from ODB.
 */

// -----------------------------------------------------------------------------
class ObsTable : private boost::noncopyable {
 public:
  explicit ObsTable(const util::Config &);
  ~ObsTable();

  void putdb(const std::string &, const ObsVec1D &);
  ObsVec1D * getdb(const std::string &) const;

  void createObs(const std::vector<DateTime> &, const std::vector<double> &);

  LocsL95 * locations(const DateTime &, const DateTime &) const;
  std::vector<int> timeSelect(const DateTime &, const DateTime &) const;
  int nobs() const {return times_.size();}

 private:
  void otOpen(const std::string &);
  void otWrite(const std::string &) const;

  std::vector<DateTime> times_;
  std::vector<double> locations_;
  std::map<std::string, ObsVec1D> data_;

  std::string nameIn_;
  std::string nameOut_;
};
// -----------------------------------------------------------------------------
}  // namespace lorenz95

#endif  // L95_SRC_OBSTABLE_H_
