/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef L95_SRC_OBSVEC1D_H_
#define L95_SRC_OBSVEC1D_H_

#include <vector>

#include "util/asser1.h"

namespace lorenz95 {

// -----------------------------------------------------------------------------
/// Vector in observation space
/*!
 *  ObsVec1D is implemented as an STL vector
 */

class ObsVec1D {
 public:
  explicit ObsVec1D(const int no): data_(no) {}
  explicit ObsVec1D(std::vector<double> vec): data_(vec) {}
  ObsVec1D(const ObsVec1D &, const bool copy = true);
  ~ObsVec1D() {}

  ObsVec1D & operator= (const ObsVec1D &);
  ObsVec1D & operator*= (const double &);
  ObsVec1D & operator+= (const ObsVec1D &);
  ObsVec1D & operator-= (const ObsVec1D &);
  ObsVec1D & operator*= (const ObsVec1D &);
  ObsVec1D & operator/= (const ObsVec1D &);

  void zero();
  void axpy(const double &, const ObsVec1D &);
  void invert();
  void random();
  double dot_product_with(const ObsVec1D &) const;

  void minMaxAvg(double &, double &, double &) const;
  int nobs() const {return data_.size();}

  double & operator() (const int ii) {return data_[ii];}
  const double & operator() (const int ii) const {return data_[ii];}

 private:
  std::vector<double> data_;
};
//-----------------------------------------------------------------------------
}  // namespace
#endif  // L95_SRC_OBSVEC1D_H_
