/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef L95_SRC_OBSERVATIONL95_H_
#define L95_SRC_OBSERVATIONL95_H_

#include <string>
#include <vector>

#include <boost/shared_ptr.hpp>

#include "lorenz95/NoObsTraj.h"
#include "lorenz95/ObsTable.h"

// Forward declarations
namespace util {
  class Config;
  class DateTime;
}

using util::DateTime;

namespace lorenz95 {
  class GomL95;
  class LocsL95;
  class ObsBias;
  class ObsVec1D;

/// Observation for Lorenz 95 model.
/*!
 *  ObservationL95 defines ObsOperator for Lorenz 95 model.
 */

// -----------------------------------------------------------------------------

class ObservationL95 : private boost::noncopyable {
 public:
  static std::string obsname_;
  static ObservationL95 * create(const util::Config & conf)
    {return new ObservationL95(conf);}

  ~ObservationL95();

// Obs Operators
  void obsEquiv(const GomL95 &, ObsVec1D &, const ObsBias &) const;
  void obsEquiv(const GomL95 &, ObsVec1D &, const ObsBias &,
                NoObsTraj &) const;
  void obsEquivTL(const GomL95 &, ObsVec1D &, const ObsBias &,
                  const NoObsTraj &) const;
  void obsEquivAD(GomL95 &, const ObsVec1D &, ObsBias &,
                  const NoObsTraj &) const;

// Other
  LocsL95 * locations(const DateTime &, const DateTime &) const;
  GomL95 * getGOM(const DateTime &, const DateTime &) const;
  NoObsTraj * getTraj() const {return new NoObsTraj();}
  ObsVec1D * distribute(const util::Config &, const DateTime &, const DateTime &);
  ObsVec1D * read(const std::string &) const;
  void save(const std::string &, const ObsVec1D &) const;
  const std::string & obsname() const {return obsname_;}

 private:
  explicit ObservationL95(const util::Config &);
  boost::shared_ptr<ObsTable> obsdb_;
};

// -----------------------------------------------------------------------------

}  // namespace lorenz95

#endif  // L95_SRC_OBSERVATIONL95_H_
