/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "StateL95.h"

#include <string>

#include "logger/Logger.h"
#include "oops/base/Geometry.h"
#include "oops/base/Variables.h"
#include "util/Config.h"
#include "util/DateTime.h"
#include "util/Duration.h"
#include "util/asser1.h"

#include "lorenz95/FieldL95.h"
#include "lorenz95/GomL95.h"
#include "lorenz95/IncrementL95.h"
#include "lorenz95/L95Fortran.h"
#include "lorenz95/LocsL95.h"
#include "lorenz95/ModelError.h"
#include "lorenz95/ModelL95.h"
#include "lorenz95/TrajectoryL95.h"

namespace lorenz95 {

// -----------------------------------------------------------------------------
/// Constructor, destructor
// -----------------------------------------------------------------------------
StateL95::StateL95(const oops::Geometry & resol, const oops::Variables & vars,
                   const util::DateTime & vt)
  : FieldL95(resol, vt)
{
  ASSERT(vars.config().empty());
  LOG(Trace) << "StateL95::StateL95 created";
}
// -----------------------------------------------------------------------------
StateL95::StateL95(const oops::Geometry & resol, const util::Config & file)
  : FieldL95(resol, util::DateTime())
{
  FieldL95::read(file);
  LOG(Trace) << "StateL95::StateL95 created and read in.";
}
// -----------------------------------------------------------------------------
StateL95::StateL95(const StateL95 & xx): FieldL95(xx) {
  LOG(Trace) << "StateL95::StateL95 copy-created.";
}
// -----------------------------------------------------------------------------
StateL95::~StateL95() {
  LOG(Trace) << "StateL95::StateL95 destructed.";
}
// -----------------------------------------------------------------------------
/// Basic operators
// -----------------------------------------------------------------------------
StateL95 & StateL95::operator=(const StateL95 & rhs) {
  FieldL95::operator=(rhs);
  return *this;
}
// -----------------------------------------------------------------------------
/// Interpolate to observation location
// -----------------------------------------------------------------------------
void StateL95::interpolate(const LocsL95 & locs, GomL95 & cols) const {
  FieldL95::interp(locs, cols);
}
// -----------------------------------------------------------------------------
/// Interactions with Increments
// -----------------------------------------------------------------------------
StateL95 & StateL95::operator+=(const IncrementL95 & dx) {
  FieldL95::add(dx);
  return *this;
}
// -----------------------------------------------------------------------------
/// Run forecast TL and AD
// -----------------------------------------------------------------------------
void StateL95::init(const ModelL95 &) {}
// -----------------------------------------------------------------------------
void StateL95::step(const ModelL95 & model, const ModelError &) {
  const ModelConfig * ftnModel = model.toFortran();
  FieldF90 * flds = FieldL95::toFortran();
  l95_propagate_f90(&ftnModel, &flds);
  FieldL95::timeUpdate(model.timestep());
}
// -----------------------------------------------------------------------------
TrajF90 * StateL95::stepTraj(const ModelL95 & model) {
  const ModelConfig * ftnModel = model.toFortran();
  FieldF90 * flds = FieldL95::toFortran();
  TrajF90 * ftraj = 0;
  l95_create_traj_f90(&ftnModel, &flds, &ftraj);
  FieldL95::timeUpdate(model.timestep());
  return ftraj;
}
// -----------------------------------------------------------------------------

}  // namespace lorenz95
