/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef L95_SRC_STATEL95_H_
#define L95_SRC_STATEL95_H_

#include "lorenz95/FieldL95.h"

namespace oops {
  class Geometry;
  class Variables;
}

namespace util {
  class Config;
}

namespace lorenz95 {
  class GomL95;
  class IncrementL95;
  class LocsL95;
  class ModelError;
  class ModelL95;
  class TrajF90;

/// L95 model state
/*!
 * A State contains everything that is needed to propagate the state
 * forward in time.
 */

// -----------------------------------------------------------------------------
class StateL95: public FieldL95 {
 public:

/// Constructor, destructor
  StateL95(const oops::Geometry &, const oops::Variables &, const util::DateTime &);
  StateL95(const oops::Geometry &, const util::Config &);
  StateL95(const StateL95 &);
  virtual ~StateL95();
  StateL95 & operator=(const StateL95 &);

/// Interpolate to observation location
  void interpolate(const LocsL95 &, GomL95 &) const;

/// Add Increment
  StateL95 & operator+=(const IncrementL95 &);

/// Set trajectory. Should it be protected and Trajectory be a sub-class? YT
  TrajF90 * stepTraj(const ModelL95 &);

 protected:
  void init(const ModelL95 &);
  void step(const ModelL95 &, const ModelError &);

  void zero() {FieldL95::zero();}
  void accumul(const double & zz, const StateL95 & xx) {FieldL95::axpy(zz, xx, false);}
};
// -----------------------------------------------------------------------------

}  // namespace lorenz95

#endif  // L95_SRC_STATEL95_H_
