!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Fortran module to handle L95 Localization Matrix
module L95LOCALIZATION

use l95fields
use iso_c_binding
use c_logger_mod
use kinds

implicit none

type l95_loc_config
  real(kind=kind_real), pointer :: matcirc(:)
end type l95_loc_config
private

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine c_l95loc_setup(c_config,rlen,c_resol) bind(c,name='l95loc_setup_f90')

implicit none
type(c_ptr), intent(inout) :: c_config
real(c_double), intent(in) :: rlen
integer(c_int), intent(in) :: c_resol

integer :: i,j,nresol
integer :: ifax(13)
character(len=160) :: record
real(kind=kind_real), allocatable :: trigs(:), work(:), struct_fn(:)
real(kind=kind_real) :: z
type(l95_loc_config), pointer :: config

nresol = c_resol

allocate(config)

!--- Calculate spectral weights for correlations:
!--- First we construct the structure function in grid space and FFT it

allocate(struct_fn(nresol+2))
do j=1,nresol
  z = (sqrt(0.5_kind_real)/rlen)*real(min(j-1,1+nresol-j),kind_real)
  struct_fn(j) = exp(-z*z)
enddo

allocate (trigs(3*nresol/2+1))
allocate(work(nresol+2))
call set99 (trigs,ifax,nresol)
call fft991 (struct_fn,work,trigs,ifax,1,nresol+2,nresol,1,-1)
deallocate(work)
deallocate(trigs)

if (mod(nresol,2)/=0) then
  write(record,*) "c_l95loc_setup: number of gridpoints=",nresol
  call c_logger(logger_error,trim(record))
  call abor1_ftn("c_l95loc_setup: odd number of grid points")
endif

allocate(config%matcirc(0:nresol/2))

do i=0,nresol/2
  config%matcirc(i) = max(0.0_kind_real, real(nresol,kind_real)*struct_fn(1+2*i))
enddo

deallocate(struct_fn)

!--- return the config structure to c++

c_config = c_loc(config)

return
end subroutine c_l95loc_setup

! ------------------------------------------------------------------------------

subroutine c_l95loc_delete(c_config) bind(c,name='l95loc_delete_f90')

type(c_ptr), intent(inout) :: c_config

type(l95_loc_config), pointer :: config

call c_f_pointer(c_config, config)

deallocate(config%matcirc)
deallocate(config)

end subroutine c_l95loc_delete

! ------------------------------------------------------------------------------

subroutine c_l95loc_mult(c_fld,c_config) &
         & bind(c,name='l95loc_mult_f90')

implicit none
type(c_ptr), intent(inout) :: c_fld
type(c_ptr), intent(in)    :: c_config

integer :: kk, iri, m
integer :: ifax(13)
type(l95field), pointer :: fld
type(l95_loc_config), pointer :: config
real(kind=kind_real), allocatable :: trigs(:), xext(:), work(:)

call c_f_pointer(c_fld,fld)
call c_f_pointer(c_config, config)

kk = fld%resol
allocate(trigs(3*kk/2+1))
allocate(xext(kk+2))
allocate(work(kk+2))

call set99 (trigs,ifax,kk)

xext(1:kk) = fld%x(:)
xext(kk+1:kk+2) = 0.0_kind_real

call fft991 (xext,work,trigs,ifax,1,kk+2,kk,1,-1)

do m=0,kk/2
  do iri=1,2
    xext(2*m+iri) = xext(2*m+iri) * config%matcirc(m)
  enddo
enddo

call fft991 (xext,work,trigs,ifax,1,kk+2,kk,1,+1)
fld%x(:) = xext(1:kk)

deallocate(work)
deallocate(xext)
deallocate(trigs)

return
end subroutine c_l95loc_mult


end module L95LOCALIZATION
