/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef QG_MODEL_GOMQG_H_
#define QG_MODEL_GOMQG_H_

#include <string>

#include "model/QgFortran.h"
#include "util/DateTime.h"
#include "util/asser1.h"

namespace qg {

/// GomQG class to handle local model values for QG model.

class GomQG {
 public:
  GomQG() {qg_gom_create_f90(&f90gom_);}
  ~GomQG() {qg_gom_delete_f90(&f90gom_);}

  void zero() {qg_gom_zero_f90(&f90gom_);}
  void random() {qg_gom_random_f90(&f90gom_);}
  double dot_product_with(const GomQG & other) const {
    double zz;
    const F90goms * fgom = other.toFortran();
    qg_gom_dotprod_f90(&f90gom_, &fgom, zz);
    return zz;
  }

  F90goms * toFortran() {return f90gom_;}
  const F90goms * toFortran() const {return f90gom_;}
 private:
  F90goms * f90gom_;
};

}  // namespace qg

#endif  // QG_MODEL_GOMQG_H_
