/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "model/ObsHelpQG.h"

#include <map>
#include <string>

#include "logger/Logger.h"
#include "model/ObsVecQG.h"
#include "model/QgFortran.h"
#include "util/Config.h"

namespace qg {
// -----------------------------------------------------------------------------
std::map < std::string, int > ObsHelpQG::theObsFileCount_;
// -----------------------------------------------------------------------------

ObsHelpQG * ObsHelpQG::getHelper(const util::Config & config) {
  static std::map < std::string, ObsHelpQG * > theObsFileRegister_;
  typedef std::map< std::string, ObsHelpQG * >::iterator otiter;

  std::string ofin;
  if (config.elementExists("ObsFileIn")) {
    const util::Config ofinConf(config, "ObsFileIn");
    ofin = config.getData("ObsFileIn");
  }
  std::string ofout = config.getData("ObsFileOut");
  LOG(Trace) << "Obs files are: " << ofin << " and " << ofout;
  std::string ref = ofin + ofout;

  ObsHelpQG * helper;
  otiter it = theObsFileRegister_.find(ref);
  if (it != theObsFileRegister_.end()) {
    // File already open
    LOG(Trace) << "ObsHelpQG::getHelper: " << ofin << " already opened.";
    helper = it->second;
    theObsFileCount_[ref]+=1;
    LOG(Trace) << "ObsHelpQG same, count=" << theObsFileCount_[ref];
  } else {
    // Open new file
    LOG(Trace) << "ObsHelpQG::getHelper: " << "Opening " << ofin << " .";
    helper = new ObsHelpQG(ref, config);
    theObsFileRegister_[ref]=helper;
  }

  return helper;
}

// -----------------------------------------------------------------------------

void ObsHelpQG::clearHelper(ObsHelpQG * helper) {
  theObsFileCount_[helper->ref_]-=1;
  LOG(Trace) << "ObsHelpQG cleared, count=" << theObsFileCount_[helper->ref_];
  if (theObsFileCount_[helper->ref_] == 0) {
    qg_obsdb_delete_f90(&helper->f90help_);
    delete helper;
  }
}

// -----------------------------------------------------------------------------

ObsHelpQG::ObsHelpQG(const std::string & ref, const util::Config & config)
  : ref_(ref)
{
  const util::Config * configc = &config;
  qg_obsdb_setup_f90(&f90help_, &configc);
  theObsFileCount_[ref_]=1;
  LOG(Trace) << "ObsHelpQG created, count=" << theObsFileCount_[ref_];
}

// -----------------------------------------------------------------------------

ObsHelpQG::ObsHelpQG(const ObsHelpQG & other)
  : ref_(other.ref_), f90help_(other.f90help_)
{
  theObsFileCount_[ref_]+=1;
  LOG(Trace) << "ObsHelpQG copied, count=" << theObsFileCount_[ref_];
}

// -----------------------------------------------------------------------------

ObsHelpQG::~ObsHelpQG() {
  LOG(Trace) << "ObsHelpQG destructed";
}

// -----------------------------------------------------------------------------

void ObsHelpQG::putdb(const std::string & req, const std::string & col,
                      const ObsVecQG & vec) {
  const F90ovec * fvec = vec.toFortran();
  qg_obsdb_put_f90(&f90help_, req.size(), req.c_str(), col.size(), col.c_str(),
                   &fvec);
}

// -----------------------------------------------------------------------------

ObsVecQG * ObsHelpQG::getdb(const std::string & req, const std::string & col) const {
  LOG(Trace) << "ObsHelpQG:getdb req=" << req << ", col=" << col;
  F90ovec * fvec;
  qg_obsdb_get_f90(&f90help_, req.size(), req.c_str(), col.size(), col.c_str(),
                   &fvec);
  return new ObsVecQG(fvec);
}

// -----------------------------------------------------------------------------
}  // namespace qg
