/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef QG_MODEL_OBSSTREAMQG_H_
#define QG_MODEL_OBSSTREAMQG_H_

#include <string>
#include <vector>

#include "model/ObsHelpQG.h"
#include "model/ObsTrajQG.h"
#include "model/QgObservation.h"

// Forward declarations
namespace util {
  class Config;
  class DateTime;
}

namespace oops {
  class NullObsAux;
}

namespace qg {
  class GomQG;
  class LocQG;
  class ObsVecQG;

// -----------------------------------------------------------------------------
/// Streamfunction observation for QG model.
/*!
 *  ObsStreamQG for QG model inherits from ObsEquivalent.
 */

class ObsStreamQG : public QgObservation {
 public:
  explicit ObsStreamQG(const util::Config &);
  ~ObsStreamQG();

// Obs Operators
  void obsEquiv(const GomQG &, ObsVecQG &, const oops::NullObsAux &) const;
  void obsEquiv(const GomQG &, ObsVecQG &, const oops::NullObsAux &,
                ObsTrajQG &) const;

  void obsEquivTL(const GomQG &, ObsVecQG &, const oops::NullObsAux &,
                  const ObsTrajQG &) const;

  void obsEquivAD(GomQG &, const ObsVecQG &, oops::NullObsAux &,
                  const ObsTrajQG &) const;

// Other
  LocQG * locations(const util::DateTime &, const util::DateTime &) const;
  GomQG * getGOM(const util::DateTime &, const util::DateTime &) const;
  ObsTrajQG * getTraj() const {return new ObsTrajQG();}
  ObsVecQG * distribute(const util::Config &,
                        const util::DateTime &, const util::DateTime &);
  ObsVecQG * read(const std::string &) const;
  void save(const std::string &, const ObsVecQG &) const;
  const std::string obsname() {return obsname_;}

 private:
  const std::string obsname_;
  ObsHelpQG * obsdb_;
  F90strm * f90stream_;
};
// -----------------------------------------------------------------------------

}  // namespace qg
#endif  // QG_MODEL_OBSSTREAMQG_H_
