/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "model/ObsVecQG.h"
#include "model/QgFortran.h"

namespace qg {
// -----------------------------------------------------------------------------
ObsVecQG::ObsVecQG(const ObsVecQG & other, const bool) {
  qg_obsvec_clone_f90(&other.f90ovec_, &f90ovec_);
}
// -----------------------------------------------------------------------------
ObsVecQG::~ObsVecQG() {
  qg_obsvec_delete_f90(&f90ovec_);
}
// -----------------------------------------------------------------------------
ObsVecQG & ObsVecQG::operator= (const ObsVecQG & rhs) {
  const F90ovec * f90rhs = rhs.f90ovec_;
  qg_obsvec_assign_f90(&f90ovec_, &f90rhs);
  return *this;
}
// -----------------------------------------------------------------------------
ObsVecQG & ObsVecQG::operator*= (const double & zz) {
  qg_obsvec_mul_scal_f90(&f90ovec_, zz);
  return *this;
}
// -----------------------------------------------------------------------------
ObsVecQG & ObsVecQG::operator+= (const ObsVecQG & rhs) {
  const F90ovec * f90rhs = rhs.f90ovec_;
  qg_obsvec_add_f90(&f90ovec_, &f90rhs);
  return *this;
}
// -----------------------------------------------------------------------------
ObsVecQG & ObsVecQG::operator-= (const ObsVecQG & rhs) {
  const F90ovec * f90rhs = rhs.f90ovec_;
  qg_obsvec_sub_f90(&f90ovec_, &f90rhs);
  return *this;
}
// -----------------------------------------------------------------------------
ObsVecQG & ObsVecQG::operator*= (const ObsVecQG & rhs) {
  const F90ovec * f90rhs = rhs.f90ovec_;
  qg_obsvec_mul_f90(&f90ovec_, &f90rhs);
  return *this;
}
// -----------------------------------------------------------------------------
ObsVecQG & ObsVecQG::operator/= (const ObsVecQG & rhs) {
  const F90ovec * f90rhs = rhs.f90ovec_;
  qg_obsvec_div_f90(&f90ovec_, &f90rhs);
  return *this;
}
// -----------------------------------------------------------------------------
void ObsVecQG::zero() {
  qg_obsvec_zero_f90(&f90ovec_);
}
// -----------------------------------------------------------------------------
void ObsVecQG::axpy(const double & zz, const ObsVecQG & rhs) {
  const F90ovec * f90rhs = rhs.f90ovec_;
  qg_obsvec_axpy_f90(&f90ovec_, zz, &f90rhs);
}
// -----------------------------------------------------------------------------
void ObsVecQG::invert() {
  qg_obsvec_invert_f90(&f90ovec_);
}
// -----------------------------------------------------------------------------
void ObsVecQG::random() {
  qg_obsvec_random_f90(&f90ovec_);
}
// -----------------------------------------------------------------------------
double ObsVecQG::dot_product_with(const ObsVecQG & other) const {
  const F90ovec * f90other = other.f90ovec_;
  double zz;
  qg_obsvec_dotprod_f90(&f90ovec_, &f90other, zz);
  return zz;
}
// -----------------------------------------------------------------------------
void ObsVecQG::minMaxAvg(double & zmin, double & zmax, double & zavg) const {
  qg_obsvec_minmaxavg_f90(&f90ovec_, zmin, zmax, zavg);
}
// -----------------------------------------------------------------------------
int ObsVecQG::nobs() const {
  int nobs;
  qg_obsvec_nobs_f90(&f90ovec_, nobs);
  return nobs;
}
// -----------------------------------------------------------------------------
}  // namespace qg
