/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef QG_MODEL_QGERRORCOVARIANCE_H_
#define QG_MODEL_QGERRORCOVARIANCE_H_

#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include "model/QgFortran.h"
#include "model/QgModel.h"
#include "oops/base/Geometry.h"
#include "oops/base/Variables.h"
#include "util/Config.h"
#include "util/DateTime.h"

// Forward declarations
namespace qg {
  class QgIncrement;
  class QgState;

// -----------------------------------------------------------------------------
/// Background error covariance matrix for QG model.

class QgErrorCovariance : private boost::noncopyable {
 public:
  QgErrorCovariance(const oops::Geometry &, const util::Config &);
  ~QgErrorCovariance();

  void linearize(const QgState &, const oops::Geometry &, const util::Config &);

  void mult(const QgIncrement &, QgIncrement &) const;
  void inv_mult(const QgIncrement &, QgIncrement &) const;

  void randomize(QgIncrement &) const;

  const util::DateTime & validTime() const {return time_;}
  const oops::Geometry & resolution() const {return *geom_;}
  const oops::Variables & ctrlvars() const {return *ctrlvars_;}

 private:
  F90bmat * ftnConfig_;
  boost::scoped_ptr<const oops::Geometry> geom_;
  boost::scoped_ptr<const oops::Variables> ctrlvars_;
  boost::shared_ptr<QgModel>  model_;
  util::DateTime time_;
};
// -----------------------------------------------------------------------------

}  // namespace qg
#endif  // QG_MODEL_QGERRORCOVARIANCE_H_
