/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef QG_MODEL_QGFIELDS_H_
#define QG_MODEL_QGFIELDS_H_

#include <string>

#include <boost/shared_ptr.hpp>

#include "model/QgGeometry.h"
#include "model/VarQG.h"
#include "util/DateTime.h"
#include "util/Duration.h"

// Forward declarations
namespace util {
  class Config;
}

namespace qg {
  class LocQG;
  class GomQG;

// -----------------------------------------------------------------------------
/// Class to represent a FieldSet for the QG model
class QgFields {
 public:
// Constructors and basic operators
  QgFields(const QgGeometry &, const VarQG &, const util::DateTime &);
  QgFields(const QgFields &);
  ~QgFields();

  QgFields & operator=(const QgFields &);
  QgFields & operator+=(const QgFields &);
  QgFields & operator-=(const QgFields &);
  QgFields & operator*=(const double &);
  void zero(const util::Duration & shift = util::Duration(0) );
  void axpy(const double &, const QgFields &);
  double dot_product_with(const QgFields &) const;
  void schur_product_with(const QgFields &);
  void random();

// Interpolate to given location
  void interpolate(const LocQG &, GomQG &) const;
  void interpolateTL(const LocQG &, GomQG &) const;
  void interpolateAD(const LocQG &, const GomQG &);

// Interpolate full fields
  void changeResolution(const QgFields &);
  void add(const QgFields &);
  void diff(const QgFields &, const QgFields &);

// Utilities
  void read(const util::Config &);
  void write(const util::Config &) const;
  void prtNorms(const std::string &) const;

  const util::DateTime time() const {return time_;}
  void timeUpdate(const util::Duration dt) {time_ += dt;}

  F90flds * toFortran() {return flds_;}
  const F90flds * toFortran() const {return flds_;}

 private:
  F90flds * flds_;
  boost::shared_ptr<const QgGeometry> geom_;
  boost::shared_ptr<const VarQG> vars_;
  util::DateTime time_;
};
// -----------------------------------------------------------------------------

}  // namespace qg
#endif  // QG_MODEL_QGFIELDS_H_
