/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef QG_MODEL_QGFORTRAN_H_
#define QG_MODEL_QGFORTRAN_H_

// Forward declarations

namespace util {
  class Config;
  class DateTime;
  class Duration;
}

namespace qg {

/// Empty class to give a type to fortran type qg_geom_mod::qg_geom
class F90geom {};
/// Empty class to give a type to fortran type qg_configs::qg_config
class F90conf {};
/// Empty class to give a type to fortran type qg_vars_mod::qg_vars
class F90vars {};
/// Empty class to give a type to fortran type qg_locs_mod::qg_locs
class F90locs {};
/// Empty class to give a type to fortran type qg_fields::qg_field
class F90flds {};
/// Empty class to give a type to fortran type qg_trajectories::qg_trajectory
class F90traj {};
/// Empty class to give a type to fortran type qg_3d_covar_configs::qg_3d_covar_config
class F90bmat {};
/// Empty class to give a type to fortran type qg_goms_mod::qg_goms
class F90goms {};
/// Empty class to give a type to fortran type qg_obs_vectors::obs_vect
class F90ovec {};
/// Empty class to give a type to fortran type qg_obs_data::obs_data
class F90help {};
/// Empty class to give a type to fortran type qg_stream_mod::qg_strm
class F90strm {};
/// Empty class to give a type to fortran type qg_wind_mod::qg_wind (wind components)
class F90wind {};
/// Empty class to give a type to fortran type qg_wind_mod::qg_wind (wind speed)
class F90wspd {};

/// Interface to Fortran QG model
/*!
 * The core of the QG model is coded in Fortran.
 * Here we define the interfaces to the Fortran code.
 */

extern "C" {
// -----------------------------------------------------------------------------
//  Geometry
// -----------------------------------------------------------------------------
  void qg_geo_setup_f90(F90geom **, const util::Config **);
  void qg_geo_clone_f90(const F90geom * const *, F90geom **);
  void qg_geo_delete_f90(F90geom ** g);

// -----------------------------------------------------------------------------
//  Model
// -----------------------------------------------------------------------------
  void qg_setup_f90(const util::Config **, const F90geom **, F90conf **);
  void qg_delete_f90(F90conf ** c);

  void qg_prepare_integration_f90(const F90conf * const *, F90flds **);
  void qg_prepare_integration_tl_f90(const F90conf * const *, F90flds **);
  void qg_prepare_integration_ad_f90(const F90conf * const *, F90flds **);

  void qg_propagate_f90(const F90conf * const *, F90flds **);
  void qg_prop_traj_f90(const F90conf * const *, F90flds **, F90traj **);
  void qg_propagate_tl_f90(const F90conf * const *, F90flds **,
                           const F90traj **);
  void qg_propagate_ad_f90(const F90conf * const *, F90flds **,
                           const F90traj **);

  void qg_wipe_traj_f90(F90traj ** t);

// -----------------------------------------------------------------------------
//  Fields
// -----------------------------------------------------------------------------
  void qg_field_create_f90(F90flds **, const F90geom **, const F90vars **);
  void qg_field_delete_f90(F90flds ** f);

  void qg_field_copy_f90(F90flds **, const F90flds * const *);
  void qg_field_zero_f90(F90flds ** f);
  void qg_field_self_add_f90(F90flds **, const F90flds * const *);
  void qg_field_self_schur_f90(F90flds **, const F90flds * const *);
  void qg_field_self_sub_f90(F90flds **, const F90flds * const *);
  void qg_field_self_mul_f90(F90flds **, const double &);
  void qg_field_axpy_f90(F90flds **, const double &, F90flds * const *);
  void qg_field_dot_prod_f90(const F90flds * const *, const F90flds * const *, double &);

  void qg_field_add_incr_f90(F90flds **, const F90flds * const *);
  void qg_field_diff_incr_f90(F90flds **, const F90flds * const *,
                              const F90flds * const *);

  void qg_field_change_resol_f90(F90flds **, const F90flds * const *);

  void qg_field_read_file_f90(F90flds **, const util::Config **,
                              util::DateTime **);
  void qg_field_write_file_f90(const F90flds * const *,
                               const util::Config **,
                               const util::DateTime **);

  void qg_field_interp_tl_f90(const F90flds **, const F90locs **, F90goms **);
  void qg_field_interp_ad_f90(F90flds **, const F90locs **, const F90goms **);

  void qg_field_gpnorm_f90(const F90flds **, const int &, const char *);
  void qg_field_random_f90(F90flds ** f);

// -----------------------------------------------------------------------------
//  Background error
// -----------------------------------------------------------------------------
  void qg_b_setup_f90(F90bmat **, const util::Config **, const F90geom **);
  void qg_b_delete_f90(F90bmat ** b);

//  void qg_b_linearize_f90(F90bmat **, const util::Config **);

  void qg_b_mult_f90(const F90bmat * const *, const F90flds **, F90flds **);
  void qg_b_inv_mult_f90(const F90bmat * const *, const F90flds **, F90flds **);

  void qg_b_randomize_f90(const F90bmat * const *, F90flds **);

// -----------------------------------------------------------------------------
//  Variables
// -----------------------------------------------------------------------------
  void qg_var_create_f90(F90vars **, const util::Config * const *);
  void qg_var_clone_f90(const F90vars * const *, F90vars **);
  void qg_var_delete_f90(F90vars ** v);

// -----------------------------------------------------------------------------
//  Locations
// -----------------------------------------------------------------------------
  void qg_loc_delete_f90(F90locs ** l);
  void qg_loc_nobs_f90(const F90locs * const *, int &);

// -----------------------------------------------------------------------------
//  Local Values (GOM)
// -----------------------------------------------------------------------------
  void qg_gom_create_f90(F90goms ** g);
  void qg_gom_delete_f90(F90goms ** g);
  void qg_gom_zero_f90(F90goms ** g);
  void qg_gom_random_f90(F90goms ** g);
  void qg_gom_dotprod_f90(const F90goms * const *, const F90goms **, double &);

// -----------------------------------------------------------------------------
//  Streamfunction observations
// -----------------------------------------------------------------------------
  void qg_stream_setup_f90(F90strm **, const util::Config **, F90help **);
  void qg_stream_delete_f90(F90strm ** s);

  void qg_stream_equiv_f90(const F90goms **, F90ovec **);
  void qg_stream_equiv_tl_f90(const F90goms **, F90ovec **);
  void qg_stream_equiv_ad_f90(F90goms **, const F90ovec **);

  void qg_stream_locations_f90(const F90strm * const *, const util::DateTime **,
                               const util::DateTime **, F90locs **);
  void qg_stream_getgom_f90(const F90strm * const *,
                            const util::DateTime **, const util::DateTime **,
                            F90goms **);
  void qg_stream_generate_f90(const F90strm * const *, const util::Config **,
                              const util::DateTime * const *,
                              const util::Duration * const *, const int &,
                              F90ovec **);

// -----------------------------------------------------------------------------
//  Wind observations
// -----------------------------------------------------------------------------
  void qg_wind_setup_f90(F90wind **, const util::Config **, F90help **);
  void qg_wind_delete_f90(F90wind ** w);

  void qg_wind_equiv_f90(const F90goms **, F90ovec **);
  void qg_wind_equiv_tl_f90(const F90goms **, F90ovec **);
  void qg_wind_equiv_ad_f90(F90goms **, const F90ovec **);

  void qg_wind_locations_f90(const F90wind * const *,
                             const util::DateTime **, const util::DateTime **,
                             F90locs **);
  void qg_wind_getgom_f90(const F90wind * const *,
                          const util::DateTime **, const util::DateTime **,
                          F90goms **);
  void qg_wind_generate_f90(const F90wind * const *, const util::Config **,
                            const util::DateTime * const *,
                            const util::Duration * const *, const int &,
                            F90ovec **);

// -----------------------------------------------------------------------------
//  Wind speed observations
// -----------------------------------------------------------------------------
  void qg_wspeed_setup_f90(F90wspd **, const util::Config **, F90help **);
  void qg_wspeed_delete_f90(F90wspd ** s);

  void qg_wspeed_eqv_f90(const F90goms **, F90ovec **);
  void qg_wspeed_tra_f90(const F90goms **, F90ovec **, F90goms **);
  void qg_wspeed_equiv_tl_f90(const F90goms **, F90ovec **, const F90goms **);
  void qg_wspeed_equiv_ad_f90(F90goms **, const F90ovec **, const F90goms **);

  void qg_wspeed_locations_f90(const F90wspd * const *, const util::DateTime **,
                               const util::DateTime **, F90locs **);
  void qg_wspeed_getgom_f90(const F90wspd * const *,
                            const util::DateTime **, const util::DateTime **,
                            F90goms **);
  void qg_wspeed_gettra_f90(const F90wspd * const *, F90goms **);
  void qg_wspeed_generate_f90(const F90wspd * const *, const util::Config **,
                              const util::DateTime * const *,
                              const util::Duration * const *, const int &,
                              F90ovec **);

// -----------------------------------------------------------------------------
//  Observation Vectors
// -----------------------------------------------------------------------------
  void qg_obsvec_clone_f90(const F90ovec * const *, F90ovec ** o);
  void qg_obsvec_delete_f90(F90ovec ** o);

  void qg_obsvec_assign_f90(F90ovec **, const F90ovec * const *);
  void qg_obsvec_zero_f90(F90ovec ** f);
  void qg_obsvec_mul_scal_f90(F90ovec **, const double &);
  void qg_obsvec_add_f90(F90ovec **, const F90ovec * const *);
  void qg_obsvec_sub_f90(F90ovec **, const F90ovec * const *);
  void qg_obsvec_mul_f90(F90ovec **, const F90ovec * const *);
  void qg_obsvec_div_f90(F90ovec **, const F90ovec * const *);
  void qg_obsvec_axpy_f90(F90ovec **, const double &, const F90ovec * const *);
  void qg_obsvec_invert_f90(F90ovec ** v);
  void qg_obsvec_random_f90(F90ovec ** v);
  void qg_obsvec_dotprod_f90(const F90ovec * const *, const F90ovec * const *,
                             double &);
  void qg_obsvec_minmaxavg_f90(const F90ovec * const *, double &, double &,
                               double &);
  void qg_obsvec_nobs_f90(const F90ovec * const *, int &);

// -----------------------------------------------------------------------------
//  Observation Handler
// -----------------------------------------------------------------------------
  void qg_obsdb_setup_f90(F90help **, const util::Config **);
  void qg_obsdb_delete_f90(F90help ** h);
  void qg_obsdb_get_f90(const F90help * const *, const int &, const char *,
                        const int &, const char *, F90ovec **);
  void qg_obsdb_put_f90(F90help **, const int &, const char *,
                        const int &, const char *, const F90ovec **);

  // -----------------------------------------------------------------------------
  //  Localization matrix
  // -----------------------------------------------------------------------------
  void qg_localization_setup_f90(F90bmat **, const util::Config **, const F90geom **);
  void qg_localization_delete_f90(F90bmat ** b);
  void qg_localization_mult_f90(const F90bmat * const *, F90flds **);

}

// -----------------------------------------------------------------------------

}  // namespace qg
#endif  // QG_MODEL_QGFORTRAN_H_
