/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef QG_MODEL_QGMODEL_H_
#define QG_MODEL_QGMODEL_H_

#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include "oops/base/Geometry.h"
#include "oops/base/Variables.h"
#include "model/QgFortran.h"
#include "util/Duration.h"

// Forward declarations
namespace oops {
  class NullModelAux;
}

namespace util {
  class Config;
}

namespace qg {
  class QgFields;
  class QgTrajectory;

// -----------------------------------------------------------------------------
/// QG model definition.
/*!
 *  QG nonlinear and linear model definition and
 *  configuration parameters.
 */

class QgModel: private boost::noncopyable {
 public:
  QgModel(const oops::Geometry &, const util::Config &);
  ~QgModel();

/// Prepare model integration
  void init(QgFields &) const;
  void initTL(QgFields &, const QgTrajectory &) const;
  void initAD(QgFields &, const QgTrajectory &) const;

/// Model integration
  void step(QgFields &, const oops::NullModelAux &) const;
  void stepTL(QgFields &, const oops::NullModelAux &, const QgTrajectory &) const;
  void stepAD(QgFields &, oops::NullModelAux &, const QgTrajectory &) const;
  F90traj * stepTraj(QgFields &) const;

/// Other utilities
  const util::Duration & timestep() const {return tstep_;}
  const oops::Geometry & resolution() const {return geom_;}

  const oops::Variables & variables() const {return *vars_;}
  const oops::Variables & variablesTL() const {return *varsTL_;}

 private:
  F90conf * qgConfig_;
  util::Duration tstep_;
  const oops::Geometry geom_;
  boost::scoped_ptr<const oops::Variables> vars_;
  boost::scoped_ptr<const oops::Variables> varsTL_;
};
// -----------------------------------------------------------------------------

}  // namespace qg
#endif  // QG_MODEL_QGMODEL_H_
