/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "model/QgObservation.h"

#include <map>
#include <string>

#include "logger/Logger.h"
#include "util/abor1_cpp.h"
#include "util/Config.h"

namespace qg {

// -----------------------------------------------------------------------------

// We have to use a pointer here because we don't know the order in
// which objects are created and this causes problems with xlc.

std::map < std::string, ObsFactory * > * ObsFactory::makers_ = 0;

// -----------------------------------------------------------------------------

ObsFactory::ObsFactory(const std::string & name) {
  if (!makers_) makers_=new std::map < std::string, ObsFactory * >();

  if (makers_->find(name) != makers_->end()) {
    LOG(Error) << name << " already registered in QG observation factory.";
    ABORT("Element already registered in ObsFactory.");
  }
  (*makers_)[name] = this;
}

// -----------------------------------------------------------------------------

QgObservation * ObsFactory::create(const util::Config & conf) {
  if (!makers_) makers_=new std::map < std::string, ObsFactory * >();

  std::string id = conf.getData("ObsType");
  LOG(Trace) << "QG ObsFactory ObsType =" << id;
  std::map<std::string, ObsFactory*>::iterator j = makers_->find(id);
  if (j == makers_->end()) {
    LOG(Error) << id << " does not exist in QG observation factory.";
    ABORT("Element does not exist in ObsFactory.");
  }
  return (*j).second->make(conf);
}

// -----------------------------------------------------------------------------
}  // namespace qg
