/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef QG_MODEL_QGOBSERVATION_H_
#define QG_MODEL_QGOBSERVATION_H_

#include <map>
#include <string>

#include <boost/noncopyable.hpp>

// Forward declarations
namespace oops {
  class NullObsAux;
}

namespace util {
  class Config;
  class DateTime;
}

namespace qg {
  class GomQG;
  class LocQG;
  class ObsVecQG;
  class ObsTrajQG;

/// Observations for QG model.
/*!
 *  Base class for QG observations.
 */

// -----------------------------------------------------------------------------

class QgObservation;

/// ObsFactory
class ObsFactory {
 public:
  static QgObservation * create(const util::Config &);
  virtual ~ObsFactory() { makers_->clear(); }
 protected:
  explicit ObsFactory(const std::string &);
 private:
  virtual QgObservation * make(const util::Config &) =0;
  static std::map < std::string, ObsFactory * > * makers_;
};

template<class T>
class ObsMaker : public ObsFactory {
  virtual QgObservation * make(const util::Config & c) {return new T(c);}
 public:
  explicit ObsMaker(const std::string & name) : ObsFactory(name) {}
};

// -----------------------------------------------------------------------------

class QgObservation : private boost::noncopyable {
 public:
  static QgObservation * create(const util::Config & conf)
    {return ObsFactory::create(conf);}

  virtual ~QgObservation() {}

// Obs Operators
  virtual void obsEquiv(const GomQG &, ObsVecQG &, const oops::NullObsAux &) const =0;
  virtual void obsEquiv(const GomQG &, ObsVecQG &, const oops::NullObsAux &,
                        ObsTrajQG &) const =0;

  virtual void obsEquivTL(const GomQG &, ObsVecQG &, const oops::NullObsAux &,
                          const ObsTrajQG &) const =0;

  virtual void obsEquivAD(GomQG &, const ObsVecQG &, oops::NullObsAux &,
                          const ObsTrajQG &) const =0;

// Other
  virtual LocQG * locations(const util::DateTime &, const util::DateTime &) const =0;
  virtual GomQG * getGOM(const util::DateTime &, const util::DateTime &) const =0;
  virtual ObsTrajQG * getTraj() const =0;
  virtual ObsVecQG * distribute(const util::Config &,
                                const util::DateTime &, const util::DateTime &) =0;
  virtual ObsVecQG * read(const std::string &) const =0;
  virtual void save(const std::string &, const ObsVecQG &) const =0;
  virtual const std::string obsname() =0;
};

// -----------------------------------------------------------------------------

}  // namespace qg
#endif  // QG_MODEL_QGOBSERVATION_H_
