/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef QG_MODEL_QGSTATE_H_
#define QG_MODEL_QGSTATE_H_

#include <string>

#include <boost/scoped_ptr.hpp>

#include "model/QgFields.h"
#include "util/DateTime.h"

namespace oops {
  class Geometry;
  class NullModelAux;
  class Variables;
}

namespace util {
  class Config;
}

namespace qg {
  class GomQG;
  class LocQG;
  class QgIncrement;
  class QgModel;
  class F90traj;

/// QG model state
/*!
 * A State contains everything that is needed to propagate the state
 * forward in time.
 */

// -----------------------------------------------------------------------------
class QgState {
 public:

/// Constructor, destructor
  QgState(const oops::Geometry &, const oops::Variables &, const util::DateTime &);
  QgState(const oops::Geometry &, const util::Config &);
  QgState(const QgState &);
  virtual ~QgState();
  QgState & operator=(const QgState &);

/// Interpolate to observation location
  void interpolate(const LocQG &, GomQG &) const;

/// Interpolate full fields
  void changeResolution(const QgState &);

/// Interactions with Increment
  QgState & operator+=(const QgIncrement &);
  void diff(const QgState &, QgFields &) const;
  void xpay(const double &, QgFields &) const;

/// I/O and diagnostics
  void read(const util::Config &);
  void write(const util::Config &) const;
  void prtNorms(const std::string &) const;
  const util::DateTime validTime() const {return fields_->time();}

/// Set trajectory
  F90traj * saveTrajectory(const QgModel &);

 protected:
  void init(const QgModel &);
  void step(const QgModel &, const oops::NullModelAux &);

  void zero();
  void accumul(const double &, const QgState &);

 private:
  boost::scoped_ptr<QgFields> fields_;
};
// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // QG_MODEL_QGSTATE_H_
