!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Fortran module handling observations for the QG model

module c_qg_obs_data

use qg_obs_data
use qg_obs_vectors
use iso_c_binding
use c_config_mod
use string_f_c_mod

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine qg_obs_setup(c_self, c_conf) bind(c,name='qg_obsdb_setup_f90')
use c_logger_mod, only : c_logger, logger_info
implicit none
type(c_ptr), intent(inout) :: c_self
type(c_ptr), intent(in)    :: c_conf !< configuration

type(obs_data), pointer :: self
character(len=100) :: fin, fout
character(len=124) :: record

if (c_config_element_exists(c_conf,"ObsFileIn")) then
  fin  = c_config_get_data(c_conf,max_string,"ObsFileIn")
else
  fin  = ""
endif
write(record,*)'qg_obs_setup: file in =',fin
call c_logger(logger_info,trim(record))

fout = c_config_get_data(c_conf,max_string,"ObsFileOut")
write(record,*)'qg_obs_setup: file out=',fout
call c_logger(logger_info,trim(record))

allocate(self)
call obs_setup(trim(fin), trim(fout), self)
c_self = c_loc(self)

call flush()
end subroutine qg_obs_setup

! ------------------------------------------------------------------------------

subroutine qg_obs_delete(c_self) bind(c,name='qg_obsdb_delete_f90')
implicit none
type(c_ptr), intent(inout) :: c_self
type(obs_data), pointer :: self

call c_f_pointer(c_self,self)
call obs_delete(self)
deallocate(self)

end subroutine qg_obs_delete

! ------------------------------------------------------------------------------

subroutine qg_obs_get(c_self, lreq, c_req, lcol, c_col, c_ovec) bind(c,name='qg_obsdb_get_f90')
implicit none
type(c_ptr), intent(in) :: c_self
integer(c_int), intent(in) :: lreq, lcol
character(kind=c_char,len=1), intent(in) :: c_req(lreq+1), c_col(lcol+1)
type(c_ptr), intent(inout) :: c_ovec

type(obs_data), pointer :: self
type(obs_vect), pointer :: ovec
character(len=lreq) :: req
character(len=lcol) :: col

call c_f_pointer(c_self, self)
!call c_f_pointer(c_ovec, ovec)
call c_f_string(c_req, req)
call c_f_string(c_col, col)

allocate(ovec)
call obs_get(self, trim(req), trim(col), ovec)
c_ovec = c_loc(ovec)

end subroutine qg_obs_get

! ------------------------------------------------------------------------------

subroutine qg_obs_put(c_self, lreq, c_req, lcol, c_col, c_ovec) bind(c,name='qg_obsdb_put_f90')
implicit none
type(c_ptr), intent(inout) :: c_self
integer(c_int), intent(in) :: lreq, lcol
character(kind=c_char,len=1), intent(in) :: c_req(lreq+1), c_col(lcol+1)
type(c_ptr), intent(in) :: c_ovec

type(obs_data), pointer :: self
type(obs_vect), pointer :: ovec
character(len=lreq) :: req
character(len=lcol) :: col

call c_f_pointer(c_self, self)
call c_f_pointer(c_ovec, ovec)
call c_f_string(c_req, req)
call c_f_string(c_col, col)

call obs_put(self, trim(req), trim(col), ovec)

end subroutine qg_obs_put

! ------------------------------------------------------------------------------
end module c_qg_obs_data
