!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Prepare for an integration of the QG model.

!> At the start of a timestep of the QG model, the state must contain
!! streamfunction, potential vorticity and wind components. The control
!! variable for the analysis, however, contains only streamfunction.
!! This routine calculates potential vorticity and wind from the
!! streamfunction, and is called before an integration of the QG model.

subroutine c_qg_prepare_integration(c_config, c_state) &
         & bind(c,name='qg_prepare_integration_f90')

use iso_c_binding
use qg_fields
use qg_configs
use qg_constants, only: bet

implicit none
type(c_ptr), intent(in)    :: c_config  !< Configuration structure
type(c_ptr), intent(inout) :: c_state   !< Model fields

type(qg_config), pointer :: config
type(qg_field), pointer  :: flds

! ------------------------------------------------------------------------------

call c_f_pointer(c_state,flds)
call c_f_pointer(c_config,config)

! -- calculate potential vorticity and wind components

call calc_pv(flds%nx,flds%ny,flds%q,flds%x,flds%x_north,flds%x_south, &
           & config%f1,config%f2,config%deltax,config%deltay,bet,config%rs)
call zonal_wind(flds%u,flds%x,flds%x_north,flds%x_south,flds%nx,flds%ny, &
              & config%deltay)
call meridional_wind(flds%v,flds%x,flds%nx,flds%ny,config%deltax)

end subroutine c_qg_prepare_integration
