!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Perform a timestep of the QG model - Tangent Linear

!> This routine is called from C++ to propagate the increment

subroutine c_qg_propagate_tl(c_config, c_incr, c_traj) &
           bind(c,name='qg_propagate_tl_f90')

use iso_c_binding
use qg_fields
use qg_trajectories
use qg_configs
use qg_constants, only: bet
use kinds

implicit none
type(c_ptr), intent(in)    :: c_config  !< Config structure
type(c_ptr), intent(inout) :: c_incr    !< Model fields
type(c_ptr), intent(in)    :: c_traj    !< Trajectory structure

type(qg_config),     pointer :: config
type(qg_field),      pointer :: flds
type(qg_trajectory), pointer :: traj

real(kind_real), allocatable :: qnew(:,:,:), x_traj(:,:,:)
real(kind_real), allocatable :: q_traj(:,:,:), u_traj(:,:,:), v_traj(:,:,:)
real(kind_real), allocatable :: qn_traj(:,:), qs_traj(:,:)
real(kind_real) :: xn_traj(2), xs_traj(2)

! ------------------------------------------------------------------------------

call c_f_pointer(c_config,config)
call c_f_pointer(c_incr,flds)
call c_f_pointer(c_traj,traj)

!--- workspace and trajectory
allocate(qnew(flds%nx,flds%ny,2))
allocate(x_traj(flds%nx,flds%ny,2))
allocate(qn_traj(flds%nx,2))
allocate(qs_traj(flds%nx,2))
allocate(q_traj(flds%nx,flds%ny,2))
allocate(u_traj(flds%nx,flds%ny,2))
allocate(v_traj(flds%nx,flds%ny,2))
call get_traj(traj,flds%nx,flds%ny,x_traj,xn_traj,xs_traj,qn_traj,qs_traj)

!--- generate trajectory values for potential vorticity and wind

call calc_pv(flds%nx,flds%ny,q_traj,x_traj,xn_traj,xs_traj, &
    &        config%f1,config%f2,config%deltax,config%deltay,bet,config%rs)
call zonal_wind (u_traj,x_traj,xn_traj,xs_traj,flds%nx,flds%ny, config%deltay)
call meridional_wind (v_traj,x_traj,flds%nx,flds%ny,config%deltax)

!--- advect the potential vorticity

qnew(:,:,:)=0.0_kind_real
call advect_pv_tl(qnew,flds%q,q_traj,qn_traj,qs_traj, &
    &             flds%u,u_traj,flds%v,v_traj,flds%nx,flds%ny,&
    &             config%deltax,config%deltay,config%dt)

!--- invert the potential vorticity to determine streamfunction

call invert_pv_tl(flds%x,qnew,flds%nx,flds%ny, &
                & config%deltax,config%deltay,config%f1,config%f2)

! -- calculate potential vorticity and wind components

flds%q(:,:,:) = qnew(:,:,:)
call zonal_wind_tl(flds%u,flds%x,flds%nx,flds%ny,config%deltay)
call meridional_wind_tl(flds%v,flds%x,flds%nx,flds%ny,config%deltax)

!--- clean-up
deallocate(qnew)
deallocate(x_traj)
deallocate(qn_traj)
deallocate(qs_traj)
deallocate(q_traj)
deallocate(u_traj)
deallocate(v_traj)
! ------------------------------------------------------------------------------
return
end subroutine c_qg_propagate_tl
