!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Invert potential vorticity - Adjoint

subroutine invert_pv_ad (x,pv,nx,ny,deltax,deltay,F1,F2)

use kinds

implicit none
integer, intent(in) :: nx           !< Zonal grid dimension
integer, intent(in) :: ny           !< Meridional grid dimension
real(kind=kind_real), intent(inout) :: x(nx,ny,2)   !< Streamfunction adjoint variable
real(kind=kind_real), intent(inout)  :: pv(nx,ny,2) !< PV adjoint variable
real(kind=kind_real), intent(in) :: deltax          !< Zonal grid spacing (non-dimensional)
real(kind=kind_real), intent(in) :: deltay          !< Meridional grid spacing (non-dimensional)
real(kind=kind_real), intent(in) :: F1              !< Coefficient in PV operator
real(kind=kind_real), intent(in) :: F2              !< Coefficient in PV operator

real(kind=kind_real) :: pv_nobc(nx,ny,2)

!--- Solve the elliptic system. Here, we use the symmetry of the operator,
!--- rather than coding the adjoint of the solver.

x(:,:,1) = -F1*x(:,:,1)
x(:,:,2) = -F2*x(:,:,2)

call solve_elliptic_system (pv_nobc,x,nx,ny,deltax,deltay,F1,F2)

x(:,:,:) = 0.0_kind_real
pv(:,:,1) = pv(:,:,1) + pv_nobc(:,:,1)*(-1.0_kind_real/F1)
pv(:,:,2) = pv(:,:,2) + pv_nobc(:,:,2)*(-1.0_kind_real/F2)

end subroutine invert_pv_ad
