!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Calculate meridional wind component from the streamfunction

!> Nothing fancy.
!! It's just a standard centred finite difference.

subroutine meridional_wind (v,x,nx,ny,deltax)

!--- calculate meridional wind component

use kinds

implicit none
integer, intent(in) :: nx       !< Zonal grid dimension
integer, intent(in) :: ny       !< Meridional grid dimension
real(kind=kind_real), intent(out) :: v(nx,ny,2) !< Meridional wind
real(kind=kind_real), intent(in)  :: x(nx,ny,2) !< Streamfunction
real(kind=kind_real), intent(in) :: deltax      !< Zonal grid spacing (non-dimensional)

v(1:nx-1,:,:) = (0.5_kind_real/deltax)*x(2:nx,:,:)
v(nx    ,:,:) = (0.5_kind_real/deltax)*x(1   ,:,:)
v(2:nx  ,:,:) = v(2:nx,:,:) - (0.5_kind_real/deltax)*x(1:nx-1,:,:)
v(1     ,:,:) = v(1   ,:,:) - (0.5_kind_real/deltax)*x(nx    ,:,:)

end subroutine meridional_wind
