!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Interfaces to be called from C++ for Fortran handling of QG model fields

! ------------------------------------------------------------------------------

subroutine qg_field_create_c(c_self, c_geom, c_vars) bind(c,name='qg_field_create_f90')
use iso_c_binding
use qg_fields
use qg_geom_mod
use qg_vars_mod
implicit none
type(c_ptr), intent(inout) :: c_self
type(c_ptr), intent(in)    :: c_geom !< Geometry
type(c_ptr), intent(in)    :: c_vars !< List of variables

type(qg_field), pointer :: self
type(qg_geom),  pointer :: geom
type(qg_vars),  pointer :: vars

call c_f_pointer(c_geom,geom)
call c_f_pointer(c_vars,vars)
allocate(self)

call create(self, geom, vars)

c_self = c_loc(self)

return
end subroutine qg_field_create_c

! ------------------------------------------------------------------------------

subroutine qg_field_delete_c(c_self) bind(c,name='qg_field_delete_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_self
type(qg_field), pointer :: self

call c_f_pointer(c_self,self)

call delete(self)

deallocate(self)
c_self = c_null_ptr

return
end subroutine qg_field_delete_c

! ------------------------------------------------------------------------------

subroutine qg_field_zeros_c(c_self) bind(c,name='qg_field_zeros_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_self
type(qg_field), pointer :: self

call c_f_pointer(c_self,self)
call zeros(self)

return
end subroutine qg_field_zeros_c

! ------------------------------------------------------------------------------

subroutine qg_field_zero_c(c_self) bind(c,name='qg_field_zero_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_self
type(qg_field), pointer :: self

call c_f_pointer(c_self,self)
call zeros(self)

return
end subroutine qg_field_zero_c

! ------------------------------------------------------------------------------

subroutine qg_field_copy_c(c_self,c_rhs) bind(c,name='qg_field_copy_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_self
type(c_ptr), intent(in)    :: c_rhs

type(qg_field), pointer :: self
type(qg_field), pointer :: rhs
call c_f_pointer(c_self,self)
call c_f_pointer(c_rhs ,rhs)

call copy(self, rhs)

return
end subroutine qg_field_copy_c

! ------------------------------------------------------------------------------

subroutine qg_field_self_add_c(c_self,c_rhs) bind(c,name='qg_field_self_add_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_self
type(c_ptr), intent(in)    :: c_rhs

type(qg_field), pointer :: self
type(qg_field), pointer :: rhs
call c_f_pointer(c_self,self)
call c_f_pointer(c_rhs ,rhs)

call self_add(self,rhs)

return
end subroutine qg_field_self_add_c

! ------------------------------------------------------------------------------

subroutine qg_field_self_schur_c(c_self,c_rhs) bind(c,name='qg_field_self_schur_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_self
type(c_ptr), intent(in)    :: c_rhs

type(qg_field), pointer :: self
type(qg_field), pointer :: rhs
call c_f_pointer(c_self,self)
call c_f_pointer(c_rhs ,rhs)

call self_schur(self,rhs)

return
end subroutine qg_field_self_schur_c

! ------------------------------------------------------------------------------

subroutine qg_field_self_sub_c(c_self,c_rhs) bind(c,name='qg_field_self_sub_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_self
type(c_ptr), intent(in)    :: c_rhs

type(qg_field), pointer :: self
type(qg_field), pointer :: rhs
call c_f_pointer(c_self,self)
call c_f_pointer(c_rhs ,rhs)

call self_sub(self,rhs)

return
end subroutine qg_field_self_sub_c

! ------------------------------------------------------------------------------

subroutine qg_field_self_mul_c(c_self,c_zz) bind(c,name='qg_field_self_mul_f90')
use iso_c_binding
use qg_fields
use kinds
implicit none
type(c_ptr), intent(inout) :: c_self
real(c_double), intent(in) :: c_zz
type(qg_field), pointer :: self
real(kind=kind_real) :: zz

call c_f_pointer(c_self,self)
zz = c_zz

call self_mul(self,zz)

return
end subroutine qg_field_self_mul_c

! ------------------------------------------------------------------------------

subroutine qg_field_axpy_c(c_self,c_zz,c_rhs) bind(c,name='qg_field_axpy_f90')
use iso_c_binding
use qg_fields
use kinds
implicit none
type(c_ptr), intent(inout) :: c_self
real(c_double), intent(in) :: c_zz
type(c_ptr), intent(in)    :: c_rhs

type(qg_field), pointer :: self
type(qg_field), pointer :: rhs
real(kind=kind_real) :: zz

call c_f_pointer(c_self,self)
call c_f_pointer(c_rhs ,rhs)
zz = c_zz

call axpy(self,zz,rhs)

return
end subroutine qg_field_axpy_c

! ------------------------------------------------------------------------------

subroutine qg_field_dot_prod_c(c_fld1,c_fld2,c_prod) bind(c,name='qg_field_dot_prod_f90')
use iso_c_binding
use qg_fields
use kinds
implicit none
type(c_ptr), intent(in)     :: c_fld1, c_fld2
real(c_double), intent(out) :: c_prod
real(kind=kind_real) :: zz
type(qg_field), pointer :: fld1, fld2

call c_f_pointer(c_fld1,fld1)
call c_f_pointer(c_fld2,fld2)

call dot_prod(fld1,fld2,zz)

c_prod = zz

return
end subroutine qg_field_dot_prod_c

! ------------------------------------------------------------------------------

subroutine qg_field_add_incr_c(c_self,c_rhs) bind(c,name='qg_field_add_incr_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_self
type(c_ptr), intent(in)    :: c_rhs
type(qg_field), pointer :: self
type(qg_field), pointer :: rhs

call c_f_pointer(c_self,self)
call c_f_pointer(c_rhs ,rhs)

call add_incr(self,rhs)

return
end subroutine qg_field_add_incr_c

! ------------------------------------------------------------------------------

subroutine qg_field_diff_incr_c(c_lhs,c_x1,c_x2) bind(c,name='qg_field_diff_incr_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_lhs
type(c_ptr), intent(in)    :: c_x1
type(c_ptr), intent(in)    :: c_x2
type(qg_field), pointer :: lhs
type(qg_field), pointer :: x1
type(qg_field), pointer :: x2

call c_f_pointer(c_lhs,lhs)
call c_f_pointer(c_x1 ,x1)
call c_f_pointer(c_x2 ,x2)

call diff_incr(lhs,x1,x2)

return
end subroutine qg_field_diff_incr_c

! ------------------------------------------------------------------------------

subroutine qg_field_random_c(c_fld) bind(c,name='qg_field_random_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_fld
type(qg_field), pointer :: fld
call c_f_pointer(c_fld,fld)

call random(fld)

return
end subroutine qg_field_random_c
! ------------------------------------------------------------------------------

subroutine qg_field_change_resol_c(c_fld,c_rhs) bind(c,name='qg_field_change_resol_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(inout) :: c_fld
type(c_ptr), intent(in)    :: c_rhs
type(qg_field), pointer :: fld, rhs

call c_f_pointer(c_fld,fld)
call c_f_pointer(c_rhs,rhs)

call change_resol(fld,rhs)

return
end subroutine qg_field_change_resol_c

! ------------------------------------------------------------------------------

subroutine qg_field_read_file_c(c_fld, c_config, c_dt) bind(c,name='qg_field_read_file_f90')
use iso_c_binding
use qg_fields
use datetime_mod

implicit none
type(c_ptr), intent(inout) :: c_fld    !< Fields
type(c_ptr), intent(in)    :: c_config !< Configuration
type(c_ptr), intent(inout) :: c_dt     !< DateTime

type(qg_field), pointer :: fld
type(datetime) :: fdate

call c_f_pointer(c_fld,fld)
call read_file(fld, c_config, fdate)
call f_c_datetime(fdate, c_dt)

return
end subroutine qg_field_read_file_c

! ------------------------------------------------------------------------------

subroutine qg_field_write_file_c(c_fld, c_config, c_dt) bind(c,name='qg_field_write_file_f90')
use iso_c_binding
use qg_fields
use datetime_mod

implicit none
type(c_ptr), intent(in)    :: c_fld    !< Fields
type(c_ptr), intent(in)    :: c_config !< Configuration
type(c_ptr), intent(inout) :: c_dt     !< DateTime

type(qg_field), pointer :: fld
type(datetime) :: fdate

call c_f_pointer(c_fld,fld)
call c_f_datetime(c_dt, fdate)

call write_file(fld, c_config, fdate)

return
end subroutine qg_field_write_file_c

! ------------------------------------------------------------------------------

subroutine qg_field_gpnorm_c(c_fld,cglen,cgrep) bind(c,name='qg_field_gpnorm_f90')
use iso_c_binding
use qg_fields
implicit none
type(c_ptr), intent(in) :: c_fld
integer(c_int), intent(in) :: cglen
character(kind=c_char, len=1), intent(in) :: cgrep(cglen)
type(qg_field), pointer :: fld
integer, parameter :: max_string_length=100 ! Yuk!
character(len=max_string_length) :: clgrep  !< Horrible
integer :: jj

if (cglen>max_string_length) call abor1_ftn("max_string_length too small")
clgrep=''
do jj=1,cglen
  clgrep(jj:jj) = cgrep(jj)
enddo
call c_f_pointer(c_fld,fld)
call gpnorm(fld,trim(clgrep))

return
end subroutine qg_field_gpnorm_c

! ------------------------------------------------------------------------------

subroutine qg_field_interp_tl_c(c_fld,c_pos,c_gom) bind(c,name='qg_field_interp_tl_f90')
use iso_c_binding
use qg_fields
use qg_locs_mod
use qg_goms_mod
implicit none
type(c_ptr), intent(in)    :: c_fld
type(c_ptr), intent(in)    :: c_pos
type(c_ptr), intent(inout) :: c_gom
type(qg_field), pointer :: fld
type(qg_locs),  pointer :: locs
type(qg_goms),  pointer :: gom

call c_f_pointer(c_fld,fld)
call c_f_pointer(c_pos,locs)
call c_f_pointer(c_gom,gom)

call interp_tl(fld, locs, gom)

return
end subroutine qg_field_interp_tl_c

! ------------------------------------------------------------------------------

subroutine qg_field_interp_ad_c(c_fld,c_pos,c_gom) bind(c,name='qg_field_interp_ad_f90')
use iso_c_binding
use qg_fields
use qg_locs_mod
use qg_goms_mod
implicit none
type(c_ptr), intent(inout) :: c_fld
type(c_ptr), intent(in)    :: c_pos
type(c_ptr), intent(in)    :: c_gom
type(qg_field), pointer :: fld
type(qg_locs),  pointer :: locs
type(qg_goms),  pointer :: gom

call c_f_pointer(c_fld,fld)
call c_f_pointer(c_pos,locs)
call c_f_pointer(c_gom,gom)

call interp_ad(fld, locs, gom)

return
end subroutine qg_field_interp_ad_c

! ------------------------------------------------------------------------------
