!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Fortran module handling geometry for the QG model

module qg_geom_mod

use iso_c_binding
use c_config_mod

implicit none
private
public :: qg_geom

! ------------------------------------------------------------------------------

!> Fortran derived type to hold geometry data for the QG model
type :: qg_geom
  integer :: nx
  integer :: ny
end type qg_geom

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine c_qg_geo_setup(c_self, c_config) bind(c,name='qg_geo_setup_f90')
implicit none
type(c_ptr), intent(inout) :: c_self
type(c_ptr), intent(in)    :: c_config

type(qg_geom), pointer :: self
allocate(self)
self%nx = c_config_get_int(c_config,"nx")
self%ny = c_config_get_int(c_config,"ny")
c_self = c_loc(self)

end subroutine c_qg_geo_setup

! ------------------------------------------------------------------------------

subroutine c_qg_geo_clone(c_self, c_other) bind(c,name='qg_geo_clone_f90')
implicit none
type(c_ptr), intent(in)    :: c_self
type(c_ptr), intent(inout) :: c_other

type(qg_geom), pointer :: self, other

call c_f_pointer(c_self,self)
allocate(other)
other%nx = self%nx
other%ny = self%ny
c_other = c_loc(other)

end subroutine c_qg_geo_clone

! ------------------------------------------------------------------------------

subroutine c_qg_geo_delete(c_self) bind(c,name='qg_geo_delete_f90')

implicit none
type(c_ptr), intent(inout) :: c_self

type(qg_geom), pointer :: self
call c_f_pointer(c_self,self)
deallocate(self)

end subroutine c_qg_geo_delete

! ------------------------------------------------------------------------------

end module qg_geom_mod
