!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Fortran module handling interpolated (to obs locations) model variables

module qg_goms_mod

use iso_c_binding
use qg_vars_mod
use random_vectors_mod
use kinds

implicit none
private
public :: qg_goms, gom_setup

! ------------------------------------------------------------------------------

!> Fortran derived type to hold interpolated fields required by the obs operators
type :: qg_goms
  integer :: nobs
  integer :: nvar
  integer :: used
  integer, allocatable :: indx(:)
  real(kind=kind_real), allocatable :: values(:,:)
  character(len=1), allocatable :: variables(:)
  logical :: lalloc
end type qg_goms

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine c_qg_gom_create(c_self) bind(c,name='qg_gom_create_f90')

implicit none
type(c_ptr), intent(out) :: c_self

type(qg_goms), pointer :: self
allocate(self)
self%lalloc = .false.
c_self = c_loc(self)

end subroutine c_qg_gom_create

! ------------------------------------------------------------------------------

subroutine gom_setup(self, vars, kobs)
implicit none
type(qg_goms), intent(inout) :: self
type(qg_vars), intent(in) :: vars
integer, intent(in) :: kobs(:)

self%nobs=size(kobs)
self%nvar=vars%nv
self%used=0

allocate(self%indx(self%nobs))
self%indx(:)=kobs(:)

allocate(self%variables(self%nvar))
self%variables(:)=vars%fldnames(:)

allocate(self%values(self%nvar,self%nobs))

self%lalloc = .true.

end subroutine gom_setup

! ------------------------------------------------------------------------------

subroutine c_qg_gom_delete(c_self) bind(c,name='qg_gom_delete_f90')

implicit none
type(c_ptr), intent(inout) :: c_self

type(qg_goms), pointer :: self
call c_f_pointer(c_self,self)
if (self%lalloc) then
  deallocate(self%values)
  deallocate(self%indx)
  deallocate(self%variables)
endif
deallocate(self)

end subroutine c_qg_gom_delete

! ------------------------------------------------------------------------------

subroutine c_qg_gom_zero(c_self) bind(c,name='qg_gom_zero_f90')
implicit none
type(c_ptr), intent(inout) :: c_self
type(qg_goms), pointer :: self
call c_f_pointer(c_self,self)
self%values(:,:)=0.0_kind_real
end subroutine c_qg_gom_zero

! ------------------------------------------------------------------------------

subroutine c_qg_gom_random(c_self) bind(c,name='qg_gom_random_f90')
implicit none
type(c_ptr), intent(inout) :: c_self
type(qg_goms), pointer :: self
call c_f_pointer(c_self,self)
call random_vector(self%values(:,:))
end subroutine c_qg_gom_random

! ------------------------------------------------------------------------------

subroutine c_qg_gom_dotprod(c_self, c_other, prod) bind(c,name='qg_gom_dotprod_f90')
implicit none
type(c_ptr), intent(in) :: c_self, c_other
real(c_double), intent(out) :: prod
type(qg_goms), pointer :: self, other
integer :: jo, jv

call c_f_pointer(c_self,self)
call c_f_pointer(c_other,other)
prod=0.0_kind_real
do jo=1,self%nobs
  do jv=1,self%nvar
    prod=prod+self%values(jv,jo)*other%values(jv,jo)
  enddo
enddo

end subroutine c_qg_gom_dotprod

! ------------------------------------------------------------------------------

end module qg_goms_mod
