!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!
module QGLOCALIZATION
use iso_c_binding
use qg_fields
use kinds
use qg_constants
use qg_geom_mod
use c_config_mod
use c_logger_mod
use qg_3d_covar_configs

! ------------------------------------------------------------------------------
contains
subroutine qg_localization_mult(c_config, c_xincr) bind(c,name='qg_localization_mult_f90')

implicit none
type(c_ptr), intent(in) :: c_config
type(c_ptr), intent(in) :: c_xincr

type(qg_3d_covar_config), pointer :: config   !< Config structure
type(qg_field), pointer :: xincr
real(kind=kind_real), allocatable :: xctl(:,:,:) ! Control vector

! ------------------------------------------------------------------------------
call c_f_pointer(c_config, config)
call c_f_pointer(c_xincr    , xincr)

allocate(xctl(config%nx, config%ny, 2))

xctl(:,:,:)=0.0_kind_real
call qg_3d_covar_sqrt_mult_ad(config%nx,config%ny,xincr,xctl,config)
call zeros(xincr)
call qg_3d_covar_sqrt_mult(config%nx,config%ny,xincr,xctl,config)

deallocate(xctl)
end subroutine qg_localization_mult

subroutine qg_localization_setup(c_config, c_model, c_geom) bind(c,name='qg_localization_setup_f90')


implicit none
type(c_ptr), intent(inout) :: c_config
type(c_ptr), intent(in)   :: c_model  !< The configuration
type(c_ptr), intent(in) :: c_geom     !< Geometry

! ------------------------------------------------------------------------------
type(qg_3d_covar_config), pointer :: config !< covar structure
type(qg_geom), pointer :: geom     !< Geometry

allocate(config)
call c_f_pointer(c_geom, geom)
call qg_3d_covar_setup(c_model, geom, config)
c_config = c_loc(config)

return
end subroutine qg_localization_setup

! ------------------------------------------------------------------------------
! ------------------------------------------------------------------------------

subroutine qg_localization_delete(c_config) bind(c,name='qg_localization_delete_f90')

implicit none
type(c_ptr), intent(inout) :: c_config !< The model covariance structure

! ------------------------------------------------------------------------------

call qg_3d_covar_delete(c_config)

! ------------------------------------------------------------------------------

end subroutine qg_localization_delete

end module QGLOCALIZATION
