!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Fortran module handling observation locations

module qg_locs_mod

use iso_c_binding
use qg_obs_vectors
use kinds

implicit none
private
public :: qg_locs, qg_loc_setup

! ------------------------------------------------------------------------------

!> Fortran derived type to hold observation locations
type :: qg_locs
  integer :: nloc
  real(kind=kind_real), allocatable :: xyz(:,:)
end type qg_locs

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine qg_loc_setup(self, lvec)
implicit none
type(qg_locs), intent(inout) :: self
type(obs_vect), intent(in) :: lvec
integer :: jc, jo

self%nloc=lvec%nobs
allocate(self%xyz(3,self%nloc))
do jo=1,self%nloc
  do jc=1,3
    self%xyz(jc,jo)=lvec%values(jc,jo)
  enddo
enddo

end subroutine qg_loc_setup

! ------------------------------------------------------------------------------

subroutine c_qg_loc_delete(c_self) bind(c,name='qg_loc_delete_f90')

implicit none
type(c_ptr), intent(inout) :: c_self

type(qg_locs), pointer :: self
call c_f_pointer(c_self,self)
deallocate(self%xyz)
deallocate(self)

end subroutine c_qg_loc_delete

! ------------------------------------------------------------------------------

subroutine c_qg_loc_nobs(c_self, kobs) bind(c,name='qg_loc_nobs_f90')

implicit none
type(c_ptr), intent(in) :: c_self
integer(c_int), intent(out) :: kobs

type(qg_locs), pointer :: self
call c_f_pointer(c_self,self)
kobs = self%nloc

end subroutine c_qg_loc_nobs

! ------------------------------------------------------------------------------

end module qg_locs_mod
