!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Fortran module to handle variables for the QG model
module qg_vars_mod

use iso_c_binding
use c_config_mod
use c_logger_mod

implicit none
private
public :: qg_vars, qg_vars_setup

! ------------------------------------------------------------------------------

!> Fortran derived type to represent QG model variables
type :: qg_vars
  integer :: nv
  character(len=1), allocatable :: fldnames(:) !< Variable identifiers
  logical :: lbc
end type qg_vars

! ------------------------------------------------------------------------------
contains
! ------------------------------------------------------------------------------

subroutine qg_vars_setup(self, cvars)
implicit none
type(qg_vars), intent(inout) :: self
character(len=1), intent(in) :: cvars(:)
integer :: jj

self%nv = size(cvars)
self%lbc = .false.

do jj=1,self%nv
  if (cvars(jj)/="x" .and. cvars(jj)/="q" .and. cvars(jj)/="u" .and. cvars(jj)/="v") &
     & call abor1_ftn ("qg_vars_setup: unknown field")
enddo
allocate(self%fldnames(self%nv))
self%fldnames(:)=cvars(:)

end subroutine qg_vars_setup

! ------------------------------------------------------------------------------

subroutine c_qg_vars_create(c_self, c_config) bind(c,name='qg_var_create_f90')
implicit none
type(c_ptr), intent(inout) :: c_self
type(c_ptr), intent(in)    :: c_config

type(qg_vars), pointer :: self
character(len=2) :: svar

allocate(self)

svar = c_config_get_data(c_config,len(svar),"variables")
select case (svar)
case ("nl")
  self%nv = 4
  self%lbc = .true.
  allocate(self%fldnames(4))
  self%fldnames(:) = (/"x","q","u","v"/)
case ("tl")
  self%nv = 4
  self%lbc = .false.
  allocate(self%fldnames(4))
  self%fldnames(:) = (/"x","q","u","v"/)
case ("cv")
  self%nv = 1
  self%lbc = .true.
  allocate(self%fldnames(1))
  self%fldnames(1) = "x"
case ("ci")
  self%nv = 1
  self%lbc = .false.
  allocate(self%fldnames(1))
  self%fldnames(1) = "x"
case ("x")
  self%nv = 1
  self%lbc = .false.
  allocate(self%fldnames(1))
  self%fldnames(1) = "x"
case default
  call abor1_ftn("c_qg_vars_create: undefined variables")
end select

c_self = c_loc(self)

return
end subroutine c_qg_vars_create

! ------------------------------------------------------------------------------

subroutine c_qg_vars_clone(c_self, c_other) bind(c,name='qg_var_clone_f90')
implicit none
type(c_ptr), intent(in)    :: c_self
type(c_ptr), intent(inout) :: c_other

type(qg_vars), pointer :: self, other
call c_f_pointer(c_self,self)

allocate(other)
other%nv = self%nv
other%lbc = self%lbc

allocate(other%fldnames(other%nv))
other%fldnames(:)=self%fldnames(:)

c_other = c_loc(other)

return
end subroutine c_qg_vars_clone

! ------------------------------------------------------------------------------

subroutine c_qg_vars_delete(c_self) bind(c,name='qg_var_delete_f90')

implicit none
type(c_ptr), intent(inout) :: c_self

type(qg_vars), pointer :: self
call c_f_pointer(c_self,self)
deallocate(self%fldnames)
deallocate(self)

return
end subroutine c_qg_vars_delete

! ------------------------------------------------------------------------------

end module qg_vars_mod
