#!/bin/ksh
set -e

# QG Model
echo
echo "------------------"
echo " Running QG tests"
echo "------------------"
echo

#unit tests
../src/test_util.x --log_level=warning

cd test
mkdir -p Data

rm -f qg.*.out
echo "Running QG truth"
../qg_forecast.x truth.xml > qg.truth.out
echo "Running QG background forecast"
../qg_forecast.x forecast.xml > qg.forecast.out

echo "Generating QG 3D-Var observations from the truth"
../qg_makeobs.x makeobs3d.xml > qg.makeobs3d.out
echo "Generating QG 4D-Var observations from the truth"
../qg_makeobs.x makeobs4d.xml > qg.makeobs4d.out

../qg_test.x test.xml

echo "Running QG incremental 3D-Var"
../qg_4dvar.x 3dvar.xml > qg.3dvar.out
echo "Running QG incremental 4D-Var"
../qg_4dvar.x 4dvar.xml > qg.4dvar.out
echo "Running QG weak constraint 4D-Var - forcing formulation"
../qg_4dvar.x 4dforcing.xml > qg.4dforcing.out
echo "Running QG weak constraint 4D-Var - saddlepoint formulation"
../qg_4dvar.x 4dsaddlepoint.xml > qg.4dsaddlepoint.out

echo "Generating QG observations from the truth"
../qg_makeobs.x makeobs4d12h.xml > qg.makeobs4d12h.out
echo "Running QG 4D-Ens-Var"
../qg_4dvar.x 4densvar.xml > qg.4densvar.out

echo "Running QG forecast with DFI"
../qg_dfi.x dfi.xml > qg.dfi.out

echo
echo "-------------------"
echo " Comparing results"
echo "-------------------"
echo

egrep 'Nonlinear J|reduction in residual norm' *.out > exp.summary.txt
diff -s ../control.summary.txt exp.summary.txt
rm -f exp.summary.txt

