/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "logger/Logger.h"

#include <stdio.h>

#include <iostream>
#include <sstream>
#include <string>

namespace logger {

//------------------------------------------------------------------------------

// Initialize static members.

// Note: selected_ is reset by setLogging early in the execution.
bool  Logger::selected_[] = {
  false,  // Trace
  false,  // Debug
  false,  // Configs
  true,   // Info
  true,   // Warning
  true,   // Error
  true,   // Fatal
};

const std::string Logger::CategoryNames_[] = {
  "Trace",
  "Debug",
  "Configs",
  "Info",
  "Warning",
  "Error",
  "Fatal"
};

//------------------------------------------------------------------------------

Logger::~Logger() {
  os_ << std::endl;
  std::cout << os_.str();
  if (msgCategory_ <= Debug) { std::cout.flush(); }
}

//------------------------------------------------------------------------------

std::ostringstream & Logger::get(Category cat) {
  msgCategory_ = cat;
  os_.width(8);
  os_ << std::left << CategoryNames_[cat] << " : ";
  return os_;
}

//------------------------------------------------------------------------------

void Logger::disableAll() {
  for (int i = 0; i < static_cast<int>(MAX); ++i) {
    selected_[i]=false;
  }
}

//------------------------------------------------------------------------------

bool Logger::lookup(const std::string& str, Category& cat) {
  bool found = false;
  for (int i = 0; i < static_cast<int>(MAX); ++i) {
    if (str == CategoryNames_[i]) {
      cat = Category(i);
      found = true;
      break;
    }
  }
  return found;
}


//------------------------------------------------------------------------------

}  // namespace logger
