!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!>  Fortran module handling the interface to logger::Logger

module c_logger_mod

use ISO_C_BINDING, only : C_INT, C_CHAR
use ftn_c_string_conversion

implicit none

private

save

public c_logger,       &
     & c_logger_init,  &
     & logger_trace,   &
     & logger_debug,   &
     & logger_info,    &
     & logger_warning, &
     & logger_error

!> Fortran derived type to represent a logger::Category
type logger_category
  integer(kind=C_INT) :: category = -1
end type logger_category

type(logger_category) :: logger_trace
type(logger_category) :: logger_debug
type(logger_category) :: logger_info
type(logger_category) :: logger_warning
type(logger_category) :: logger_error

interface
  subroutine ftn_logger(c_str,c_cat) bind(C,NAME='ftn_logger')
    use ISO_C_BINDING
    implicit none
    character(kind=C_CHAR, len=1), intent(in) :: c_str(*)
    integer(kind=C_INT), intent(in) :: c_cat
  end subroutine ftn_logger
end interface

interface
  logical(kind=C_BOOL) function ftn_logger_lookup(c_str,c_cat) &
    & bind(C,NAME='ftn_logger_lookup')
    use ISO_C_BINDING
    implicit none
    character(kind=C_CHAR, len=1), intent(in) :: c_str(*)
    integer(kind=C_INT), intent(out) :: c_cat
  end function ftn_logger_lookup
end interface

contains

!-------------------------------------------------------------------------------

!>  Log a message

subroutine c_logger (category,string)

  implicit none

  type(logger_category), intent(in) :: category
  character(len=*), intent(in) :: string

  character(kind=C_CHAR,len=1), allocatable :: c_string(:)

  if (category%category < 0) then
    call abor1_ftn ("c_logger::logger called for an uninitialised category")
  endif

  allocate(c_string(len(string)+1))
  c_string = ftn_to_c_string(string)

  call ftn_logger(c_string,category%category)

  deallocate(c_string)

end subroutine c_logger

!-------------------------------------------------------------------------------

!> Map the fortran logging categories to their c++ equivalents

subroutine c_logger_init () BIND(C,NAME="c_logger_init_")
  implicit none

  call lookup_category("Info"   ,logger_info)
  call c_logger (logger_info,"Fortran initialised logging category Info")

  call lookup_category("Trace"  ,logger_trace)
  call c_logger (logger_info,"Fortran initialised logging category Trace")

  call lookup_category("Debug"  ,logger_debug)
  call c_logger (logger_info,"Fortran initialised logging category Debug")

  call lookup_category("Warning",logger_warning)
  call c_logger (logger_info,"Fortran initialised logging category Warning")

  call lookup_category("Error"  ,logger_error)
  call c_logger (logger_info,"Fortran initialised logging category Error")
end subroutine c_logger_init

!-------------------------------------------------------------------------------

subroutine lookup_category (name,category)
  use ISO_C_BINDING

  implicit none

  character(len=*), intent(in) :: name
  type(logger_category),intent(out) :: category

  logical(kind=C_BOOL) :: lfound
  integer(kind=C_INT) :: i
  character(kind=C_CHAR,len=1), allocatable :: c_string(:)

  allocate(c_string(len(name)+1))
  c_string = ftn_to_c_string(name)
 
  lfound = ftn_logger_lookup(c_string,i)
  if (.not.lfound) then
    write (0,*) 'Lookup failed for logging category "',name,'"'
    call abor1_ftn("Lookup failed while initializing c_logger")
  endif

  deallocate(c_string)

  category%category = i
  
end subroutine lookup_category

!-------------------------------------------------------------------------------

end module c_logger_mod
