/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "logger/setLogging.h"

#include<string>

#include<boost/tokenizer.hpp>

#include "logger/Logger.h"
#include "util/Config.h"
#include "util/abor1_cpp.h"

namespace logger {
  void setLogging(const util::Config & config) {
    // Get a comma/whitespace delimited list of message categories.
    std::string cats = config.getData("categories");

    // Disable all logging
    Logger::disableAll();

    // Tokenise and loop over the categories.
    boost::tokenizer<> tok(cats);
    for (boost::tokenizer<>::iterator it = tok.begin(); it != tok.end();++it) {
      Category cat;
      bool found = Logger::lookup(*it, cat);
      if (!found) {
        LOG_ENABLE_LOGGING_FOR(Error);
        LOG(Error) << "Invalid logging category in config file: \""
                  << *it << "\"";
        ABORT("Invalid logging category");
      }

      Logger::setLogging(cat, true);  // Enable logging for this category.
      LOG(Info) << "Message logging enabled for category " << *it;
    }
}

}  // namespace logger
