/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_BMATRIX_H_
#define OOPS_ASSIMILATION_BMATRIX_H_

#include <boost/noncopyable.hpp>

#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/CostFunction.h"

namespace oops {

  /// The \f$ B \f$ matrix.
  /*!
   *  The solvers represent matrices as objects that implement a "multiply"
   *  method. This class defines objects that apply the \f$ B \f$ matrix.
   */

template<typename TRAIT> class BMatrix : private boost::noncopyable {
  typedef ControlIncrement<TRAIT>  CtrlInc;
  typedef CostFunction<TRAIT>      CostFct;

 public:
  explicit BMatrix(const CostFct & j): j_(j) {}
  void multiply(const CtrlInc & x, CtrlInc & bx) const {
    j_.jb().Bmult(x, bx);
  }

 private:
  CostFct const & j_;
};

}  // namespace oops

#endif  // OOPS_ASSIMILATION_BMATRIX_H_
