/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include <string>

#include "logger/Logger.h"
#include "oops/base/ModelIncrement.h"
#include "oops/base/ModelState.h"
#include "util/Config.h"
#include "util/DateTime.h"
#include "util/Duration.h"
#include "util/asser1.h"

namespace oops {

// -----------------------------------------------------------------------------

template<typename TRAIT>
CostFct3DVar<TRAIT>::CostFct3DVar(const util::Config & config)
  : CostFunction<TRAIT>::CostFunction(config), winHalf_(), winLen_()
{
  winLen_ = util::Duration(config.getData("window_length"));
  util::Duration half(winLen_/2);
  winHalf_ = util::DateTime(config.getData("window_begin"));
  winHalf_ += half;
  LOG(Trace) << "CostFct3DVar constructed";
}

// -----------------------------------------------------------------------------

template <typename TRAIT>
CostJb<TRAIT> * CostFct3DVar<TRAIT>::getJb(const Geometry & resol, const CtrlVar & xb,
                                           const util::Config & jbConf) const {
  ASSERT(xb.nsubwin() == 1);
  return new JbTerm(xb, resol, jbConf, util::Duration(0));
}

// -----------------------------------------------------------------------------

template <typename TRAIT>
CostJo<TRAIT> * CostFct3DVar<TRAIT>::getJo(const Observ & yobs,
                                           const util::Config & joConf) const {
  return new JoTerm(yobs, joConf, winLen_);
}

// -----------------------------------------------------------------------------

template <typename TRAIT>
CostJcBase<TRAIT> * CostFct3DVar<TRAIT>::getJc(const Geometry &,
                                               const util::Config &) const {
  JcTerm * pjc = 0;
  return pjc;
}

// -----------------------------------------------------------------------------

template <typename TRAIT>
void CostFct3DVar<TRAIT>::runNL(const CtrlVar & xx,
                                PostProcessor<State> & post) const {
  ASSERT(xx.nsubwin() == 1);
  ASSERT(xx[0].validTime() == winHalf_);
  ModelState<TRAIT> zz(xx[0], CFbase::getModel());
  zz.forecast(xx.modVar(), zz.validTime(), post);
}

// -----------------------------------------------------------------------------

template <typename TRAIT>
void CostFct3DVar<TRAIT>::runTLM(CtrlInc & dx,
                                 CostProcessor<Increment> & cost,
                                 PostProcessor<Increment> & post) const {
  ASSERT(dx[0].validTime() == winHalf_);
  TLM M(CFbase::getTLM());
  M.runTL(dx[0], dx.modVar(), util::Duration(0), cost, post);
}

// -----------------------------------------------------------------------------

template <typename TRAIT>
void CostFct3DVar<TRAIT>::runADJ(CtrlInc & dx,
                                 CostProcessor<Increment> & cost,
                                 PostProcessor<Increment> & post) const {
  TLM M(CFbase::getTLM());
  M.runAD(dx[0], dx.modVar(), util::Duration(0), cost, post);
  ASSERT(dx[0].validTime() == winHalf_);
}

// -----------------------------------------------------------------------------

template<typename TRAIT>
void CostFct3DVar<TRAIT>::addIncr(CtrlVar & xx, const CtrlInc & dx,
                                  PostProcessor<Increment> &) const {
  ASSERT(xx.nsubwin() == 1);
  ASSERT(xx[0].validTime() == winHalf_);
  ASSERT(dx[0].validTime() == winHalf_);
  dx.updateControl(xx);
}

// -----------------------------------------------------------------------------

}  // namespace oops
