/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_COSTFCT4DENSVAR_H_
#define OOPS_ASSIMILATION_COSTFCT4DENSVAR_H_

#include <map>

#include <boost/ptr_container/ptr_vector.hpp>

#include "logger/Logger.h"
#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/CostJb4D.h"
#include "oops/assimilation/CostJcBase.h"
#include "oops/assimilation/CostJo.h"
#include "oops/base/CostProcessor.h"
#include "oops/base/Departure.h"
#include "oops/base/Geometry.h"
#include "oops/base/Observation.h"
#include "oops/base/Observer.h"
#include "oops/base/ObsTrajectory.h"
#include "oops/base/PostProcessor.h"
#include "oops/base/StateInfo.h"
#include "oops/base/TrajSaver.h"
#include "util/DateTime.h"
#include "util/Duration.h"

namespace util {
  class Config;
}

namespace oops {

/// 4D-Ens-Var Cost Function
/*!
 *  Although so far only used for 4D-Ens-Var this cost function can
 *  be interpreted more generally as a four dimensional 3D-Var in the
 *  sense that the control variable is 4D (like weak-constraint 4D-Var)
 *  but the observation operator is 3D (does not involve the forecast
 *  model).
 */

// -----------------------------------------------------------------------------

template<typename TRAIT> class CostFct4DEnsVar : public CostFunction<TRAIT> {
  typedef typename TRAIT::StateBase     State;
  typedef typename TRAIT::IncrementBase Increment;
  typedef typename TRAIT::ModelData     Model;
  typedef typename TRAIT::ModelTraj     ModelTraj;
  typedef typename TRAIT::ModelAuxCtrl  ModelAuxCtrl;
  typedef typename TRAIT::ModelAuxIncr  ModelAuxIncr;
  typedef typename TRAIT::ObsAuxCtrl    ObsAuxCtrl;
  typedef typename TRAIT::ObsAuxIncr    ObsAuxIncr;
  typedef typename TRAIT::ObsTraj       ObsTraject;
  typedef ControlVariable<TRAIT>        CtrlVar;
  typedef ControlIncrement<TRAIT>       CtrlInc;
  typedef CostJb4D<TRAIT>               JbTerm;
  typedef CostJcBase<TRAIT>             JcTerm;
  typedef CostJo<TRAIT>                 JoTerm;
  typedef Observation<TRAIT>            Observ;
  typedef Departure<TRAIT>              Depart;
  typedef ObsErrorCovariance<TRAIT>     ObsError;
  typedef ObsTrajectory<ObsTraject>     ObsTraj;
  typedef CostFunction<TRAIT>           CFbase;
  typedef LinearModel<TRAIT>            TLM;

 public:
  explicit CostFct4DEnsVar(const util::Config &);
  ~CostFct4DEnsVar() {}

  void runTLM(CtrlInc &,
              CostProcessor<Increment>&, PostProcessor<Increment>&) const;
  void runADJ(CtrlInc &,
              CostProcessor<Increment>&, PostProcessor<Increment>&) const;

 private:
  void runNL(const CtrlVar &, PostProcessor<State>&) const;
  void addIncr(CtrlVar &, const CtrlInc &, PostProcessor<Increment>&) const;

  JbTerm * getJb(const Geometry &, const CtrlVar &, const util::Config &) const;
  JoTerm * getJo(const Observ &, const util::Config &) const;
  JcTerm * getJc(const Geometry &, const util::Config &) const;

  util::DateTime windowBegin_;
  util::Duration windowSub_;
  util::Duration windowLength_;
  int ncontrol_;
};

// -----------------------------------------------------------------------------
}  // namespace oops

#include "oops/assimilation/CostFct4DEnsVar.cc"

#endif  // OOPS_ASSIMILATION_COSTFCT4DENSVAR_H_
