/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_COSTFCT4DVAR_H_
#define OOPS_ASSIMILATION_COSTFCT4DVAR_H_

#include <map>

#include <boost/ptr_container/ptr_vector.hpp>

#include "logger/Logger.h"
#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/CostJb.h"
#include "oops/assimilation/CostJcBase.h"
#include "oops/assimilation/CostJo.h"
#include "oops/base/CostProcessor.h"
#include "oops/base/Departure.h"
#include "oops/base/Geometry.h"
#include "oops/base/Observation.h"
#include "oops/base/Observer.h"
#include "oops/base/ObsTrajectory.h"
#include "oops/base/PostProcessor.h"
#include "oops/base/StateInfo.h"
#include "oops/base/TrajSaver.h"
#include "util/DateTime.h"
#include "util/Duration.h"

namespace util {
  class Config;
}

namespace oops {

/// Strong Constraint 4D-Var Cost Function
/*!
 * This class is not really necessary since it is only a special
 * case of the more general weak constraint 4D-Var cost function
 * with one sub-window. It is provided for readability.
 */

// -----------------------------------------------------------------------------

template<typename TRAIT> class CostFct4DVar : public CostFunction<TRAIT> {
  typedef typename TRAIT::StateBase     State;
  typedef typename TRAIT::IncrementBase Increment;
  typedef typename TRAIT::ModelData     Model;
  typedef typename TRAIT::ModelTraj     ModelTraj;
  typedef typename TRAIT::ModelAuxCtrl  ModelAuxCtrl;
  typedef typename TRAIT::ModelAuxIncr  ModelAuxIncr;
  typedef typename TRAIT::ObsAuxCtrl    ObsAuxCtrl;
  typedef typename TRAIT::ObsAuxIncr    ObsAuxIncr;
  typedef typename TRAIT::ObsTraj       ObsTraject;
  typedef ControlVariable<TRAIT>        CtrlVar;
  typedef ControlIncrement<TRAIT>       CtrlInc;
  typedef CostJb<TRAIT>                 JbTerm;
  typedef CostJcBase<TRAIT>             JcTerm;
  typedef CostJo<TRAIT>                 JoTerm;
  typedef Observation<TRAIT>            Observ;
  typedef Departure<TRAIT>              Depart;
  typedef ObsErrorCovariance<TRAIT>     ObsError;
  typedef ObsTrajectory<ObsTraject>     ObsTraj;
  typedef CostFunction<TRAIT>           CFbase;
  typedef LinearModel<TRAIT>            TLM;

 public:
  explicit CostFct4DVar(const util::Config &);
  ~CostFct4DVar() {}

  void runTLM(CtrlInc &,
              CostProcessor<Increment>&, PostProcessor<Increment>&) const;
  void runADJ(CtrlInc &,
              CostProcessor<Increment>&, PostProcessor<Increment>&) const;

 private:
  void runNL(const CtrlVar &, PostProcessor<State>&) const;
  void addIncr(CtrlVar &, const CtrlInc &, PostProcessor<Increment>&) const;

  JbTerm * getJb(const Geometry &, const CtrlVar &, const util::Config &) const;
  JoTerm * getJo(const Observ &, const util::Config &) const;
  JcTerm * getJc(const Geometry &, const util::Config &) const;

  util::Duration windowLength_;
  util::DateTime windowBegin_;
  util::DateTime windowEnd_;
};

// -----------------------------------------------------------------------------
}  // namespace oops

#include "oops/assimilation/CostFct4DVar.cc"

#endif  // OOPS_ASSIMILATION_COSTFCT4DVAR_H_
