/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_COSTJBBASE_H_
#define OOPS_ASSIMILATION_COSTJBBASE_H_

#include <boost/noncopyable.hpp>

#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/ControlVariable.h"
#include "oops/base/ProcessorBase.h"
#include "oops/base/CostBase.h"

namespace util {
  class Config;
}

namespace oops {
  class Geometry;

// -----------------------------------------------------------------------------

/// Jb Cost Function Base Class
/*!
 * The CostJbBase is the base class for Jb terms of the cost function.
 */

template<typename TRAIT> class CostJbBase : private boost::noncopyable {
  typedef typename TRAIT::StateBase State;
  typedef ControlVariable<TRAIT>    CtrlVar;
  typedef ControlIncrement<TRAIT>   CtrlInc;

 public:
/// Constructor
  CostJbBase() {}

/// Destructor
  virtual ~CostJbBase() {}

/// Initialize before the nonlinear model integration.
  virtual ProcessorBase<State> * initialize(const CtrlVar &, const Geometry &,
                                            const util::Config &,
                                            const bool ltraj = false) =0;

/// Finalize after the nonlinear model integration.
  virtual double finalize() =0;

/// Finalize \f$ J_b\f$ after the TL run.
  virtual CtrlInc * finalTL(const CtrlInc &, const CtrlInc &) const =0;

/// Initialize \f$ J_b\f$ before the AD run.
  virtual CtrlInc * setupAD(const CtrlInc &) const =0;

/// Multiply by \f$ B\f$ and \f$ B^{-1}\f$.
  virtual void Bmult(const CtrlInc &, CtrlInc &) const =0;
  virtual void Bminv(const CtrlInc &, CtrlInc &) const =0;

/// Create new increment (set to 0).
  virtual CtrlInc * newIncrement() const =0;

/// Return Jb gradient at first guess.
  virtual const CtrlInc & getGradientFG() const =0;

/// Return first guess \f$ x_0-x_b\f$.
  virtual const CtrlInc & getFirstGuess() const =0;

/// Update gradient without using \f$ B^{-1}\f$.
  virtual void gradientUpdate(const CtrlInc &, const bool add = true) const =0;
};

// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // OOPS_ASSIMILATION_COSTJBBASE_H_
