/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_COSTJCBASE_H_
#define OOPS_ASSIMILATION_COSTJCBASE_H_

#include <boost/shared_ptr.hpp>

#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/ControlVariable.h"
#include "oops/assimilation/CostTermBase.h"
#include "oops/base/ProcessorBase.h"
#include "oops/base/WeightedDiffTLAD.h"

namespace util {
  class Config;
}

namespace oops {

// -----------------------------------------------------------------------------

/// Jc Cost Function
/*!
 * Abstract base class for Jc term of the cost function.
 */

template<typename TRAIT> class CostJcBase : public CostTermBase<TRAIT> {
  typedef typename TRAIT::StateBase      State;
  typedef typename TRAIT::IncrementBase  Increment;
  typedef ControlVariable<TRAIT>         CtrlVar;
  typedef ControlIncrement<TRAIT>        CtrlInc;

 public:
/// Destructor
  virtual ~CostJcBase() {}

/// Initialize before nonlinear model integration.
  virtual ProcessorBase<State> * initialize(const CtrlVar &,
                                            const util::Config &,
                                            const bool ltraj = false) =0;

/// Finalize computation after nonlinear model integration.
  virtual double finalize() =0;

/// Initialize \f$ J_c\f$ before starting the TL run.
  virtual WeightedDiffTLAD<Increment> * setupTL(const CtrlInc &) const =0;

/// Initialize \f$ J_c\f$ before starting the AD run.
  virtual WeightedDiffTLAD<Increment> *
    setupAD(boost::shared_ptr<const GeneralizedDeparture>) const =0;

/// Multiply by \f$ C\f$ and \f$ C^{-1}\f$.
  virtual Increment * CovarMult(const GeneralizedDeparture &) const =0;
  virtual Increment * CovarMinv(const GeneralizedDeparture &) const =0;

/// Provide new increment.
  virtual Increment * newDualVector() const =0;

/// Gradient of \f$ J_c\f$ at first guess.
  virtual Increment * getGradientFG() const =0;

/// Reset trajectory.
  virtual void reset() =0;
};

// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // OOPS_ASSIMILATION_COSTJCBASE_H_
