/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_COSTTERMBASE_H_
#define OOPS_ASSIMILATION_COSTTERMBASE_H_

#include <boost/shared_ptr.hpp>

#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/ControlVariable.h"
#include "oops/base/ProcessorBase.h"
#include "oops/base/CostBase.h"

namespace util {
  class Config;
}

namespace oops {

// -----------------------------------------------------------------------------

/// Base Class for Cost Function Terms
/*!
 * Abstract base class for the terms of the cost function.
 */

template<typename TRAIT> class CostTermBase {
  typedef typename TRAIT::StateBase     State;
  typedef typename TRAIT::IncrementBase Increment;
  typedef ControlVariable<TRAIT>         CtrlVar;
  typedef ControlIncrement<TRAIT>        CtrlInc;

 public:
/// Destructor
  virtual ~CostTermBase() {}

/// Initialize before nonlinear model integration.
  virtual ProcessorBase<State> * initialize(const CtrlVar &,
                                            const util::Config &,
                                            const bool ltraj = false) =0;

/// Finalize computation after nonlinear model integration.
  virtual double finalize() =0;

/// Initialize before starting the TL run.
  virtual CostBase<Increment> * setupTL(const CtrlInc &) const =0;

/// Initialize before starting the AD run.
  virtual CostBase<Increment> *
    setupAD(boost::shared_ptr<const GeneralizedDeparture>) const =0;

/// Multiply by covariance (or weight) matrix and its inverse.
  virtual GeneralizedDeparture * CovarMult(const GeneralizedDeparture &) const =0;
  virtual GeneralizedDeparture * CovarMinv(const GeneralizedDeparture &) const =0;

/// Provide new dual space vector (for example a Departure for Jo).
  virtual GeneralizedDeparture * newDualVector() const =0;

/// Gradient at first guess.
  virtual GeneralizedDeparture * getGradientFG() const =0;

/// Reset trajectory.
  virtual void reset() =0;
};

// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // OOPS_ASSIMILATION_COSTTERMBASE_H_
