/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "logger/Logger.h"
#include "oops/assimilation/BMatrix.h"
#include "oops/assimilation/DPCG.h"
#include "oops/assimilation/HtRinvHMatrix.h"
#include "util/asser1.h"
#include "util/dot_product.h"
#include "util/Config.h"
#include "util/IdentityMatrix.h"

namespace oops {

// -----------------------------------------------------------------------------

template<typename TRAIT>
void DPCGMinimizer<TRAIT>::minimize(const CostFct & J, CtrlInc & dx,
                                     const util::Config & config) {
  int ninner = config.getInt("ninner");
  double gnreduc = config.getReal("gradient_norm_reduction");

  LOG(Trace) << "DPCGMinimizer: max iter = " << ninner <<
                ", requested norm reduction = " << gnreduc;

// Define the matrices
  IdentityMatrix<CtrlInc> I;
  BMatrix<TRAIT> B(J);
  HtRinvHMatrix<TRAIT> HtRinvH(J);

// Define minimisation starting point
  dx.zero();
  CtrlInc dh(dx);
  dx.prtNorms("DPCGMinimizer in");

// Compute RHS
  CtrlInc rhs = J.getGradientFG();
  rhs *= -1.0;
  rhs.prtNorms("DPCGMinimizer rhs");

// Solve the linear system
  double reduc = DPCG(dx, dh, rhs, B, HtRinvH, I, ninner, gnreduc);

  LOG(Info) << "DPCGMinimizer reduction in residual norm = " << reduc;

  J.jb().gradientUpdate(dh);
  dx.prtNorms("DPCGMinimizer out");
}

// -----------------------------------------------------------------------------

}  // namespace oops
