/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_DPCGMINIMIZER_H_
#define OOPS_ASSIMILATION_DPCGMINIMIZER_H_

#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/Minimizer.h"

namespace util {
  class Config;
}

namespace oops {

/// DPCG Minimizer
/*!
 * Following Derber and Rosati (1989), an alternative to the change of
 * variable based on the \f$ B^{1/2}\f$ to precondition the variational
 * minimisation problem is to use the auxiliary variable \f$ \hat{x}=B^{-1}x\f$
 * and to update it in parallel to \f$ x\f$. \f$ J_b\f$ is then computed as
 * \f$ x^T\hat{x}\f$ eliminating the need for \f$ B^{-1}\f$ or \f$ B^{1/2}\f$.
 */

// -----------------------------------------------------------------------------

template<typename TRAIT> class DPCGMinimizer : public Minimizer<TRAIT> {
  typedef CostFunction<TRAIT>      CostFct;
  typedef ControlIncrement<TRAIT>  CtrlInc;

 public:
  DPCGMinimizer() {}
  ~DPCGMinimizer() {}
  void minimize(const CostFct &, CtrlInc &, const util::Config &);
};

// -----------------------------------------------------------------------------
}  // namespace oops

#include "oops/assimilation/DPCGMinimizer.cc"

#endif  // OOPS_ASSIMILATION_DPCGMINIMIZER_H_
