/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include <vector>

#include "logger/Logger.h"
#include "oops/assimilation/BMatrix.h"
#include "oops/assimilation/DPLanczos.h"
#include "oops/assimilation/HtRinvHMatrix.h"
#include "util/asser1.h"
#include "util/dot_product.h"
#include "util/Config.h"

namespace oops {

// -----------------------------------------------------------------------------

template<typename TRAIT>
void DPLanczosMinimizer<TRAIT>::minimize(const CostFct & J, CtrlInc & dx,
                                          const util::Config & config) {
  int ninner = config.getInt("ninner");
  double gnreduc = config.getReal("gradient_norm_reduction");

  LOG(Trace) << "DPLanczosMinimizer: max iter = " << ninner <<
                ", requested norm reduction = " << gnreduc;

// Define the matrices
  BMatrix<TRAIT> B(J);
  HtRinvHMatrix<TRAIT> HtRinvH(J);

// Define minimisation starting point
  dx.zero();
  CtrlInc dh(dx);
  dx.prtNorms("DPLanczosMinimizer in");

// Compute RHS
  CtrlInc rhs = J.getGradientFG();
  rhs *= -1.0;
  rhs.prtNorms("DPLanczosMinimizer rhs");

// Generate the preconditioner F
  F_.setup(Zhl_, Zl_, alphas_, betas_);

// Solve the linear system
  double reduc = DPLanczos(dx, dh, rhs, B, HtRinvH, F_, ninner, gnreduc,
                           Zhl_, Zl_, alphas_, betas_);

  LOG(Info) << "DPLanczosMinimizer reduction in residual norm = " << reduc;

  J.jb().gradientUpdate(dh);
  dx.prtNorms("DPLanczosMinimizer out");
}

// -----------------------------------------------------------------------------

}  // namespace oops
