/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_DPLANCZOSMINIMIZER_H_
#define OOPS_ASSIMILATION_DPLANCZOSMINIMIZER_H_

#include <vector>

#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/Minimizer.h"
#include "oops/assimilation/FMatrix.h"

namespace util {
  class Config;
}

namespace oops {

/// DPLanczos Minimizer
/*!
 * Lanczos version of DPCG Minimizer
 */

// -----------------------------------------------------------------------------

template<typename TRAIT> class DPLanczosMinimizer : public Minimizer<TRAIT> {
  typedef CostFunction<TRAIT>      CostFct;
  typedef ControlIncrement<TRAIT>  CtrlInc;

 public:
  DPLanczosMinimizer() {}
  ~DPLanczosMinimizer() {}
  void minimize(const CostFct &, CtrlInc &, const util::Config &);

 private:
  FMatrix<CtrlInc> F_;
  std::vector<CtrlInc> Zl_;
  std::vector<CtrlInc> Zhl_;
  std::vector<double> alphas_;
  std::vector<double> betas_;
};

// -----------------------------------------------------------------------------
}  // namespace oops

#include "oops/assimilation/DPLanczosMinimizer.cc"

#endif  // OOPS_ASSIMILATION_DPLANCZOSMINIMIZER_H_
