/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "logger/Logger.h"
#include "oops/assimilation/BMatrix.h"
#include "oops/assimilation/GMRESR.h"
#include "oops/assimilation/HessianMatrix.h"
#include "util/Config.h"
#include "util/asser1.h"

namespace oops {

// -----------------------------------------------------------------------------

template<typename TRAIT>
void GMRESRMinimizer<TRAIT>::minimize(const CostFct & J, CtrlInc & dx,
                                      const util::Config & config) {
  int ninner = config.getInt("ninner");
  double gnreduc = config.getReal("gradient_norm_reduction");

  LOG(Trace) << "GMRESRMinimizer: max iter = " << ninner <<
                ", requested norm reduction = " << gnreduc;

  dx.zero();
  dx.prtNorms("GMRESRMinimizer in");

// Compute RHS
  CtrlInc rhs = J.getGradientFG();
  rhs *= -1.0;
  rhs.prtNorms("GMRESRMinimizer rhs");

// Define the matrices
  HessianMatrix<TRAIT> hessian(J);
  BMatrix<TRAIT> B(J);

// Solve the linear system
  double reduc = GMRESR(dx, rhs, hessian, B, ninner, gnreduc);

  LOG(Info) << "GMRESRMinimizer reduction in residual norm = " << reduc;

  CtrlInc di(dx);
  J.jb().Bminv(dx, di);
  J.jb().gradientUpdate(di);
  dx.prtNorms("GMRESRMinimizer out");
}

// -----------------------------------------------------------------------------

}  // namespace oops
