/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_INCREMENTALASSIMILATION_H_
#define OOPS_ASSIMILATION_INCREMENTALASSIMILATION_H_

#include <sstream>
#include <string>

#include <boost/scoped_ptr.hpp>

#include "logger/Logger.h"
#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/Minimizer.h"
#include "oops/assimilation/ControlVariable.h"
#include "oops/base/PostProcessor.h"
#include "oops/base/StateInfo.h"
#include "util/Config.h"

namespace oops {

template<typename TRAIT>
void IncrementalAssimilation(ControlVariable<TRAIT> & xx, CostFunction<TRAIT> & J,
                             Minimizer<TRAIT> & minim, const util::Config & config) {
  typedef typename TRAIT::StateBase State;

  const util::Config outerConfig(config, "var4d");
  unsigned nouter = outerConfig.getElementSize("iteration");
  LOG(Configs) << "var4d has "<< nouter << " outer iterations.";
  for (unsigned jouter = 0; jouter < nouter; ++jouter) {
    LOG(Info) << "OUTER LOOP " << jouter;

//  Get configuration for inner loop
    const util::Config innerConfig(outerConfig, "iteration", jouter);
    LOG(Configs) << "IncrementalAssimilation: "
                 << "Inner loop configuration is:" << "\n" << innerConfig;

//  Setup for the inner loop
    PostProcessor<State> post;
    if (innerConfig.elementExists("prints")) {
      const util::Config prtConfig(innerConfig, "prints");
      post.enroll(new StateInfo<State>("traj", prtConfig));
    }
    J.linearize(xx, innerConfig, post);
    boost::scoped_ptr< ControlIncrement<TRAIT> > dx(J.jb().newIncrement());

//  Minimization
    minim.minimize(J, *dx, innerConfig);

//  Compute analysis in physical space
    J.addIncrement(xx, *dx);

//  Clean-up trajectory, etc...
    J.reset();
  }
}

}  // namespace oops
#endif  // OOPS_ASSIMILATION_INCREMENTALASSIMILATION_H_
