/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "logger/Logger.h"
#include "util/abor1_cpp.h"
#include "util/Config.h"

namespace oops {

// -----------------------------------------------------------------------------

template <typename TRAIT>
MinFactory<TRAIT>::MinFactory(const std::string & name) {
  if (getMakers().find(name) != getMakers().end()) {
    LOG(Error) << name << " already registered in minimizer factory.";
    ABORT("Element already registered in MinFactory.");
  }
  getMakers()[name] = this;
}

// -----------------------------------------------------------------------------

template <typename TRAIT>
Minimizer<TRAIT>* MinFactory<TRAIT>::create(const util::Config & config) {
  std::string id = config.getData("algorithm");
  LOG(Trace) << "Minimizer algorithm=" << id;
  typename std::map<string, MinFactory<TRAIT>*>::iterator j = getMakers().find(id);
  if (j == getMakers().end()) {
    LOG(Error) << id << " does not exist in minimizer factory.";
    ABORT("Element does not exist in MinFactory.");
  }
  return (*j).second->make();
}

// -----------------------------------------------------------------------------

}  // namespace oops
