/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_MINIMIZER_H_
#define OOPS_ASSIMILATION_MINIMIZER_H_

#include <map>
#include <string>

#include <boost/noncopyable.hpp>

#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/ControlIncrement.h"

namespace util {
  class Config;
}

namespace oops {

// -----------------------------------------------------------------------------

/// A Minimizer knows how to minimize a cost function
template<typename TRAIT> class Minimizer : private boost::noncopyable {
  typedef CostFunction<TRAIT>      CostFct;
  typedef ControlIncrement<TRAIT>  CtrlInc;

 public:
  virtual ~Minimizer() {}
  virtual void minimize(const CostFct &, CtrlInc &, const util::Config &) =0;
};

// -----------------------------------------------------------------------------

/// Minimizer Factory
template <typename TRAIT>
class MinFactory {
 public:
  static Minimizer<TRAIT> * create(const util::Config &);
  virtual ~MinFactory() { getMakers().clear(); }
 protected:
  explicit MinFactory(const string &);
 private:
  virtual Minimizer<TRAIT> * make() =0;
  static std::map < std::string, MinFactory<TRAIT> * > & getMakers() {
    static std::map < std::string, MinFactory<TRAIT> * > makers_;
    return makers_;
  }
};

template<class TRAIT, class FCT>
class MinMaker : public MinFactory<TRAIT> {
  virtual Minimizer<TRAIT> * make() { return new FCT(); }
 public:
  explicit MinMaker(const string & name) : MinFactory<TRAIT>(name) {}
};

// -----------------------------------------------------------------------------
}  // namespace oops

#include "oops/assimilation/Minimizer.cc"

#endif  // OOPS_ASSIMILATION_MINIMIZER_H_
