/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_PROCESSORJQ_H_
#define OOPS_ASSIMILATION_PROCESSORJQ_H_

#include <boost/noncopyable.hpp>

#include "logger/Logger.h"
#include "util/Duration.h"

namespace oops {

// -----------------------------------------------------------------------------

/// Computes model error from current state.
/*!
 *  ProcessorJq computes and stores model error.
 */

template <typename STATE, typename CTRLVAR, typename CTRLINC>
class ProcessorJq : public ProcessorBase<STATE> {
 public:
  ProcessorJq(const CTRLVAR & fg, CTRLINC & dx, const util::Duration & sublen)
    : ProcessorBase<STATE>(sublen), fg_(fg), dx_(dx), end_(), isub_(0) {}
  ~ProcessorJq() {}

 private:
  void doInitialize(const util::DateTime &, const util::DateTime & end,
                    const util::Duration &) {
    end_ = end;
  }

  void doProcessing(const STATE & xx) {
    LOG(Debug) << "ProcessorJq::doProcessing called " << xx.validTime();
    if (xx.validTime() == end_) {
      ++isub_;
      ASSERT(fg_.nsubwin() >= isub_);
      if (fg_.nsubwin() > isub_) {
        ASSERT(dx_[isub_].validTime() == xx.validTime());
        ASSERT(fg_[isub_].validTime() == xx.validTime());
        dx_[isub_].diff(fg_[isub_], xx);
      }
    }
  }

  const CTRLVAR & fg_;
  CTRLINC & dx_;
  util::DateTime end_;
  unsigned isub_;
};

// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // OOPS_ASSIMILATION_PROCESSORJQ_H_
