/*
* Copyright 2011 ECMWF
*
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
*
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_RPCGMINIMIZER_H_
#define OOPS_ASSIMILATION_RPCGMINIMIZER_H_

#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/DualVector.h"
#include "oops/assimilation/Minimizer.h"

namespace util {
  class Config;
}

namespace oops {

/// RPCG Minimizer
/*!
 * Observation space minimisation algorithm from
 * Gratton and Tshimanga, QJRMS, 135: 1573-1585 (2009).
 */

// -----------------------------------------------------------------------------

template<typename TRAIT> class RPCGMinimizer : public Minimizer<TRAIT> {
  typedef typename TRAIT::IncrementBase Increment;
  typedef CostFunction<TRAIT>           CostFct;
  typedef ControlIncrement<TRAIT>       CtrlInc;
  typedef DualVector<TRAIT>             Dual;

 public:
  RPCGMinimizer() {}
  ~RPCGMinimizer() {}
  void minimize(const CostFct &, CtrlInc &, const util::Config &);
};

// -----------------------------------------------------------------------------
}  // namespace oops

#include "oops/assimilation/RPCGMinimizer.cc"

#endif  // OOPS_ASSIMILATION_RPCGMINIMIZER_H_
