/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_RINVMATRIX_H_
#define OOPS_ASSIMILATION_RINVMATRIX_H_

#include <boost/noncopyable.hpp>

#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/DualVector.h"

namespace oops {

/// The \f$ R^{-1} \f$ matrix.
/*!
 *  The solvers represent matrices as objects that implement a "multiply"
 *  method. This class defines objects that apply a generalized \f$ R^{-1} \f$
 *  matrix which includes \f$ R^{-1} \f$ itself and the equivalent operators
 *  for the other terms of the cost function.
 */

template<typename TRAIT> class RinvMatrix : private boost::noncopyable {
  typedef CostFunction<TRAIT> CostFct;
  typedef DualVector<TRAIT>   Dual;

 public:
  explicit RinvMatrix(const CostFct & j): j_(j) {}

  void multiply(const Dual & dx, Dual & dy) const {
    dy.clear();
    for (unsigned jj = 0; jj < j_.nterms(); ++jj) {
      dy.append(j_.jterm(jj).CovarMinv(*dx.getv(jj)));
    }
  }

 private:
  CostFct const & j_;
};

}  // namespace oops

#endif  // OOPS_ASSIMILATION_RINVMATRIX_H_
