/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include <boost/scoped_ptr.hpp>

#include "oops/assimilation/DualVector.h"

namespace oops {

template<typename TRAIT>
void SaddlePointMatrix<TRAIT>::multiply(const SPVector & x,
                                        SPVector & z) const {
  typedef DualVector<TRAIT> Multipliers;

  PostProcessor<Increment> post;

// The three blocks below could be done in parallel

// ADJ block
  CostProcessor<Increment> costad;
  z.dx(j_.jb().setupAD(x.lambda().dx()));
  for (unsigned jj = 0; jj < j_.nterms(); ++jj) {
    costad.enroll(j_.jterm(jj).setupAD(x.lambda().getv(jj)));
  }
  j_.runADJ(z.dx(), costad, post);
  z.dx() += x.lambda().dx();

// TLM block
  CtrlInc ww(x.dx());
  CostProcessor<Increment> costtl;
  for (unsigned jj = 0; jj < j_.nterms(); ++jj) {
    costtl.enroll(j_.jterm(jj).setupTL(ww));
  }
  j_.runTLM(ww, costtl, post);
  z.lambda().clear();
  z.lambda().dx(j_.jb().finalTL(ww, x.dx()));
  for (unsigned jj = 0; jj < j_.nterms(); ++jj) {
    z.lambda().append(costtl.releaseTL(jj));
  }

// Diagonal block
  Multipliers diag;
  diag.dx(j_.jb().newIncrement());
  j_.jb().Bmult(x.lambda().dx(), diag.dx());
  for (unsigned jj = 0; jj < j_.nterms(); ++jj) {
    diag.append(j_.jterm(jj).CovarMult(*x.lambda().getv(jj)));
  }

// The three blocks above could be done in parallel

  z.lambda() += diag;
}

}  // namespace oops
