/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_SADDLEPOINTMATRIX_H_
#define OOPS_ASSIMILATION_SADDLEPOINTMATRIX_H_

#include <boost/noncopyable.hpp>

#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/SaddlePointVector.h"

namespace oops {

/// The Saddle-point matrix.
/*!
 *  The solvers represent matrices as objects that implement a "multiply"
 *  method. This class defines objects that apply the saddle-point matrix.
 */

template<typename TRAIT>
class SaddlePointMatrix : private boost::noncopyable {
  typedef typename TRAIT::IncrementBase Increment;
  typedef SaddlePointVector<TRAIT>      SPVector;
  typedef CostFunction<TRAIT>           CostFct;
  typedef ControlIncrement<TRAIT>       CtrlInc;

 public:
  explicit SaddlePointMatrix(const CostFct & j): j_(j) {}
  void multiply(const SPVector &, SPVector &) const;

 private:
  CostFct const & j_;
};

}  // namespace oops

#include "oops/assimilation/SaddlePointMatrix.cc"

#endif  // OOPS_ASSIMILATION_SADDLEPOINTMATRIX_H_
