/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_SADDLEPOINTMINIMIZER_H_
#define OOPS_ASSIMILATION_SADDLEPOINTMINIMIZER_H_

#include <boost/scoped_ptr.hpp>

#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/DualVector.h"
#include "oops/assimilation/Minimizer.h"

namespace util {
  class Config;
}

namespace oops {

/// SaddlePoint Minimizer
/*!
 * Implements the SaddlePoint algorithm.
 */

// -----------------------------------------------------------------------------

template<typename TRAIT> class SaddlePointMinimizer : public Minimizer<TRAIT> {
  typedef CostFunction<TRAIT>      CostFct;
  typedef ControlIncrement<TRAIT>  CtrlInc;
  typedef DualVector<TRAIT>        Multipliers;

 public:
  SaddlePointMinimizer() {}
  ~SaddlePointMinimizer() {}
  void minimize(const CostFct &, CtrlInc &, const util::Config &);
};

// -----------------------------------------------------------------------------

}  // namespace oops

#include "oops/assimilation/SaddlePointMinimizer.cc"

#endif  // OOPS_ASSIMILATION_SADDLEPOINTMINIMIZER_H_
