/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include <boost/scoped_ptr.hpp>

#include "oops/base/CostProcessor.h"
#include "oops/base/PostProcessor.h"

namespace oops {
// -----------------------------------------------------------------------------

template<typename TRAIT>
void SaddlePointPrecondMatrix<TRAIT>::multiply(const SPVector & x,
                                               SPVector & z) const {
  z.lambda().clear();
  int norder = 0;  // truncation order for Lhatinv and Lhatinvt

  z.lambda().dx(Lhatinvt(x.dx(), norder));

  CtrlInc l(z.lambda().dx());
  j_.jb().Bmult(z.lambda().dx(), l);
  l *= -1.0;
  l += x.lambda().dx();
  z.dx(Lhatinv(l, norder));

  for (unsigned jj = 0; jj < j_.nterms(); ++jj) {
    z.lambda().append(j_.jterm(jj).CovarMinv(*x.lambda().getv(jj)));
  }
}

// -----------------------------------------------------------------------------

template<typename TRAIT>
ControlIncrement<TRAIT> * SaddlePointPrecondMatrix<TRAIT>::Lhatinv(
                              const CtrlInc & xx, const int norder) const {
// Approximate L=I + (I-L) + (I-L)^2 + ... + (I-L)^norder
  PostProcessor<Increment> post;
  CostProcessor<Increment> cost;

  CtrlInc * zz = new CtrlInc(xx);
  CtrlInc ww(xx);

  for (int i = 0; i < norder; ++i) {
    j_.runTLM(ww, cost, post);
    ww.shift_front();
    *zz += ww;
  }

  return zz;
}

// -----------------------------------------------------------------------------

template<typename TRAIT>
ControlIncrement<TRAIT> * SaddlePointPrecondMatrix<TRAIT>::Lhatinvt(
                               const CtrlInc & xx, const int norder) const {
// Approximate L'=I + (I-L') + (I-L')^2 + ... + (I-L')^norder
  PostProcessor<Increment> post;
  CostProcessor<Increment> cost;

  CtrlInc * zz = new CtrlInc(xx);
  CtrlInc ww(xx);

  for (int i = 0; i < norder; ++i) {
    ww.shift_back();
    j_.runADJ(ww, cost, post);
    *zz += ww;
  }

  return zz;
}

// -----------------------------------------------------------------------------
}  // namespace oops
