/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_SADDLEPOINTPRECONDMATRIX_H_
#define OOPS_ASSIMILATION_SADDLEPOINTPRECONDMATRIX_H_

#include <boost/noncopyable.hpp>

#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/CostFunction.h"
#include "oops/assimilation/DualVector.h"
#include "oops/assimilation/SaddlePointVector.h"

namespace oops {

  /// The preconditioner for the saddle-point minimizer.
  /*!
   *  The solvers represent matrices as objects that implement a "multiply"
   *  method. This class defines objects that apply the saddle-point matrix.
   */

// -----------------------------------------------------------------------------
template<typename TRAIT>
class SaddlePointPrecondMatrix : private boost::noncopyable {
  typedef typename TRAIT::IncrementBase Increment;
  typedef SaddlePointVector<TRAIT>      SPVector;
  typedef CostFunction<TRAIT>           CostFct;
  typedef ControlIncrement<TRAIT>       CtrlInc;
  typedef DualVector<TRAIT>             Multipliers;

 public:
  explicit SaddlePointPrecondMatrix(const CostFct & j): j_(j) {}
  void multiply(const SPVector &, SPVector &) const;

 private:
  CtrlInc * Lhatinv(const CtrlInc &, const int) const;
  CtrlInc * Lhatinvt(const CtrlInc &, const int) const;

  CostFct const & j_;
};
// -----------------------------------------------------------------------------

}  // namespace oops

#include "oops/assimilation/SaddlePointPrecondMatrix.cc"

#endif  // OOPS_ASSIMILATION_SADDLEPOINTPRECONDMATRIX_H_
