/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "util/dot_product.h"

namespace oops {

// -----------------------------------------------------------------------------

template<typename TRAIT>
SaddlePointVector<TRAIT>::SaddlePointVector(CtrlInc * dx,
                                            Multipliers * lambda)
  : lambda_(lambda), dx_(dx)
{}

// -----------------------------------------------------------------------------

template<typename TRAIT>
SaddlePointVector<TRAIT>::SaddlePointVector(const SaddlePointVector & other)
  : lambda_(new Multipliers(*other.lambda_)), dx_(new CtrlInc(*other.dx_))
{}

// -----------------------------------------------------------------------------

template<typename TRAIT> SaddlePointVector<TRAIT> &
        SaddlePointVector<TRAIT>::operator=(const SaddlePointVector & rhs) {
  *lambda_ = *rhs.lambda_;
  *dx_     = *rhs.dx_;
  return *this;
}

// -----------------------------------------------------------------------------

template<typename TRAIT> SaddlePointVector<TRAIT> &
        SaddlePointVector<TRAIT>::operator+=(const SaddlePointVector & rhs) {
  *lambda_ += *rhs.lambda_;
  *dx_     += *rhs.dx_;
  return *this;
}

// -----------------------------------------------------------------------------

template<typename TRAIT> SaddlePointVector<TRAIT> &
        SaddlePointVector<TRAIT>::operator-=(const SaddlePointVector & rhs) {
  *lambda_ -= *rhs.lambda_;
  *dx_     -= *rhs.dx_;
  return *this;
}

// -----------------------------------------------------------------------------

template<typename TRAIT> SaddlePointVector<TRAIT> &
        SaddlePointVector<TRAIT>::operator*=(const double rhs) {
  *lambda_ *= rhs;
  *dx_     *= rhs;
  return *this;
}

// -----------------------------------------------------------------------------

template<typename TRAIT> void SaddlePointVector<TRAIT>::zero() {
  lambda_->zero();
  dx_->zero();
}

// -----------------------------------------------------------------------------

template<typename TRAIT> void SaddlePointVector<TRAIT>::axpy(const double zz,
                                               const SaddlePointVector & rhs) {
  lambda_->axpy(zz, *rhs.lambda_);
  dx_->axpy(zz, *rhs.dx_);
}

// -----------------------------------------------------------------------------

template<typename TRAIT> double SaddlePointVector<TRAIT>::dot_product_with(
                                    const SaddlePointVector & x2) const {
return dot_product(*lambda_, *x2.lambda_)
      +dot_product(*dx_, *x2.dx_);
}

// -----------------------------------------------------------------------------

}  // namespace oops
