/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_SADDLEPOINTVECTOR_H_
#define OOPS_ASSIMILATION_SADDLEPOINTVECTOR_H_

#include <boost/scoped_ptr.hpp>

#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/DualVector.h"

namespace oops {

/*!
 * Control vector for the saddle point formulation.
 * The vector contains two ControlIncrements and one Departure,
 * and knows how to do basic linear algebra.
 */
/// Control vector for the saddle point formulation.

// -----------------------------------------------------------------------------
template<typename TRAIT> class SaddlePointVector {
  typedef ControlIncrement<TRAIT>  CtrlInc;
  typedef DualVector<TRAIT>        Multipliers;

 public:
  SaddlePointVector(CtrlInc *, Multipliers *);
  SaddlePointVector(const SaddlePointVector &);

/// Accessor method to get the lambda_ component
  const Multipliers & lambda() const {return *lambda_;}
  Multipliers & lambda() {return *lambda_;}

/// Accessor method to set the lambda_ component
  void lambda(Multipliers * lambda) {lambda_.reset(lambda);}

/// Accessor method to get the dx_ component
  const CtrlInc & dx() const {return *dx_;}
  CtrlInc & dx() {return *dx_;}

/// Accessor method to set the dx_ component
  void dx(CtrlInc * dx) {dx_.reset(dx);}

  SaddlePointVector & operator=(const SaddlePointVector &);
  SaddlePointVector & operator+=(const SaddlePointVector &);
  SaddlePointVector & operator-=(const SaddlePointVector &);
  SaddlePointVector & operator*=(const double);
  void zero();
  void axpy(const double, const SaddlePointVector &);
  double dot_product_with(const SaddlePointVector &) const;

 private:
  boost::scoped_ptr<Multipliers> lambda_;
  boost::scoped_ptr<CtrlInc> dx_;
};
// -----------------------------------------------------------------------------
}  // namespace oops

#include "oops/assimilation/SaddlePointVector.cc"

#endif  // OOPS_ASSIMILATION_SADDLEPOINTVECTOR_H_
