/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_TRIDIAGSOLVE_H_
#define OOPS_ASSIMILATION_TRIDIAGSOLVE_H_

#include <vector>

namespace oops {

void TriDiagSolve(const std::vector<double> & diag, const std::vector<double> & sub,
                  std::vector<double> & d, std::vector<double> & sol) {
  const double n = d.size();
  ASSERT(sub.size() == n-1);
  ASSERT(diag.size() == n); 
  sol.resize(n);
  std::vector<double> c(sub);

  // Forward substitution
  c[0] = c[0] / diag[0];
  d[0] = d[0] / diag[0];
  if (n > 2) {
    for (int iiter = 1; iiter <= n-2; ++iiter) {
      double denom = diag[iiter]-sub[iiter-1]*c[iiter-1];
      c[iiter] = c[iiter]/denom;
      d[iiter] = (d[iiter]-sub[iiter-1]*d[iiter-1])/denom;
    }
  }
  d[n-1] = (d[n-1]-sub[n-2]*d[n-2])/(diag[n-1]-sub[n-2]*c[n-2]);
  // Backward substitution
  sol[n-1] = d[n-1];
  for (int iiter = n-2; iiter >= 0; --iiter) {
    sol[iiter] = (d[iiter]-(c[iiter]*sol[iiter+1]));
  }
}
}  // namespace oops

#endif  // OOPS_ASSIMILATION_TRIDIAGSOLVE_H_
