/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_TRIDIAGSPECTRUM_H_
#define OOPS_ASSIMILATION_TRIDIAGSPECTRUM_H_

#include <vector>

#include "logger/Logger.h"
#include "util/asser1.h"

extern "C" {
  void FtnTriDiagSpectrum(const int &, const double *, const double *,
                          double *, double *);
}

namespace oops {

void TriDiagSpectrum(const std::vector<double> & diag, const std::vector<double> & sub,
                     std::vector<double> & evals,
                     std::vector< std::vector<double> > & evecs) {
  const unsigned nn = diag.size();
  ASSERT(sub.size() == nn);
  evals.resize(nn);
  evecs.resize(nn);

  if (nn > 0) {
    std::vector<double> ftnev(nn*nn);

    FtnTriDiagSpectrum(nn, &diag[0], &sub[0], &evals[0], &ftnev[0]);

    unsigned ii = 0;
    for (unsigned jv = 0; jv < nn; ++jv) {
      evecs[jv].resize(nn);
      for (unsigned jj = 0; jj < nn; ++jj) {
        evecs[jv][jj] = ftnev[ii];
        ++ii;
      }
    }
  }
}

}  // namespace oops

#endif  // OOPS_ASSIMILATION_TRIDIAGSPECTRUM_H_
