/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_ASSIMILATION_INSTANTIATEMINFACTORY_H_
#define OOPS_ASSIMILATION_INSTANTIATEMINFACTORY_H_

#include "oops/assimilation/DGMRESRMinimizer.h"
#include "oops/assimilation/DIPCGMinimizer.h"
#include "oops/assimilation/GMRESRMinimizer.h"
#include "oops/assimilation/IPCGMinimizer.h"
#include "oops/assimilation/RPCGMinimizer.h"
#include "oops/assimilation/SaddlePointMinimizer.h"
#include "oops/assimilation/Minimizer.h"
#include "oops/assimilation/DPCGMinimizer.h"
#include "oops/assimilation/DPLanczosMinimizer.h"

namespace oops {

template <typename TRAIT> void instantiateMinFactory() {
  static MinMaker<TRAIT, DGMRESRMinimizer<TRAIT> >      makerDGMRESR_("DGMRESR");
  static MinMaker<TRAIT, DIPCGMinimizer<TRAIT> >        makerDIPCG_("DIPCG");
  static MinMaker<TRAIT, GMRESRMinimizer<TRAIT> >       makerGMRESR_("GMRESR");
  static MinMaker<TRAIT, IPCGMinimizer<TRAIT> >         makerIPCG_("IPCG");
  static MinMaker<TRAIT, SaddlePointMinimizer<TRAIT> >  makerSADDLE_("SaddlePoint");
  static MinMaker<TRAIT, RPCGMinimizer<TRAIT> >         makerRPCG_("RPCG");
  static MinMaker<TRAIT, DPCGMinimizer<TRAIT> >         makerDPCG_("DPCG");
  static MinMaker<TRAIT, DPLanczosMinimizer<TRAIT> >    makerDPLanczos_("DPLanczos");
}

}  // namespace oops

#endif  // OOPS_ASSIMILATION_INSTANTIATEMINFACTORY_H_
