/*
 * Copyright 2011 ECMWF
 *
 * This software was developed at ECMWF for evaluation
 * and may be used for academic and research purposes only.
 * The software is provided as is without any warranty.
 *
 * This software can be used, copied and modified but not
 * redistributed or sold. This notice must be reproduced
 * on each copy made.
 */

#ifndef OOPS_ENSEMBLE_H_
#define OOPS_ENSEMBLE_H_

#include <boost/ptr_container/ptr_vector.hpp>
#include <boost/scoped_ptr.hpp>

#include "oops/assimilation/ControlIncrement.h"
#include "oops/assimilation/ControlVariable.h"
#include "oops/base/Geometry.h"

#include "util/Duration.h"

namespace util {
class Config;
class DateTime;
}

namespace oops {

// -----------------------------------------------------------------------------

/// Ensemble

template<typename TRAIT> class Ensemble {
	typedef typename TRAIT::StateBase State;
	typedef typename TRAIT::IncrementBase Increment;

public:
/// Constructor
	Ensemble(const util::DateTime&, const util::Config&);

/// Destructor
	virtual ~Ensemble() {
		int debug=0;
	}

	/// Accessors
	unsigned size() const {
		return rank_;
	}
	Increment & operator[](const int ii) {
		return (*ensemblePerturbs_)[ii];
	}
	const Increment & operator[](const int ii) const {
		return (*ensemblePerturbs_)[ii];
	}

	void linearize(const State&, const util::Config&, const Geometry &);

private:
	State* mean(const State& xb, const util::Config& confTLM,
			const Geometry & resol);

	const util::DateTime validTime_;
	const util::DateTime referenceTime_;
	const util::Duration term_;
	string name_;
	string type_;
	int rank_;
	boost::scoped_ptr<boost::ptr_vector<Increment> > ensemblePerturbs_;

};

// -----------------------------------------------------------------------------

}// namespace oops

#include "oops/base/Ensemble.cc"

#endif  // OOPS_ENSEMBLE_H_
