/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_ENSEMBLECOVARIANCE_H_
#define OOPS_BASE_ENSEMBLECOVARIANCE_H_

#include <boost/shared_ptr.hpp>
#include <boost/ptr_container/ptr_vector.hpp>

#include "oops/base/ModelSpaceCovariance.h"
#include "oops/base/EnsemblesCollection.h"
#include "oops/base/Ensemble.h"
#include "util/DateTime.h"
#include "util/Config.h"

namespace oops {

/// Generic ensemble based model space error covariance.

// -----------------------------------------------------------------------------
template <typename TRAIT>
class EnsembleCovariance : public ModelSpaceCovariance<TRAIT> {
  typedef typename TRAIT::StateBase            State;
  typedef typename TRAIT::IncrementBase        Increment;
  typedef Ensemble<TRAIT>                      Ensemble_;
  typedef boost::shared_ptr<Ensemble<TRAIT> >  EnsemblePtr_;
  typedef EnsemblesCollection<TRAIT>	       EnsemblesCollection_;

 public:
  EnsembleCovariance(const Geometry &, const util::Config &);

  ~EnsembleCovariance();

  void linearize(const State &, const Geometry &, const util::Config &);

  void mult(const Increment &, Increment &) const;
  void inv_mult(const Increment &, Increment &) const;

  Increment * getIncrement() const;
  void randomize(Increment &) const;

 private:
  const util::Config config_;
  boost::scoped_ptr<Geometry> resol_;

};
// -----------------------------------------------------------------------------

}  // namespace oops

#include "oops/base/EnsembleCovariance.cc"

#endif  // OOPS_BASE_ENSEMBLECOVARIANCE_H_
