/*
 * Copyright 2011 ECMWF
 *
 * This software was developed at ECMWF for evaluation
 * and may be used for academic and research purposes only.
 * The software is provided as is without any warranty.
 *
 * This software can be used, copied and modified but not
 * redistributed or sold. This notice must be reproduced
 * on each copy made.
 */

#ifndef OOPS_ENSEMBLES_COLLECTION_H_
#define OOPS_ENSEMBLES_COLLECTION_H_

#include <boost/shared_ptr.hpp>
#include <boost/unordered_map.hpp>
#include <boost/foreach.hpp>

#include "oops/base/Ensemble.h"
#include "oops/base/Geometry.h"

#include "util/Duration.h"

namespace util {
class Config;
class DateTime;
}

namespace oops {

// -----------------------------------------------------------------------------

/// This class handles access to the ensembles of perturbation with a
/// DateTime

template<typename TRAIT> class EnsemblesCollection {
	typedef boost::unordered_map<util::DateTime,
			boost::shared_ptr<Ensemble<TRAIT> > > EnsemblesMap;

public:
/// Constructor

/// Destructor
	virtual ~EnsemblesCollection() {
	}

/// Accessors
	boost::shared_ptr<Ensemble<TRAIT> > operator[](
			const util::DateTime& d) const {
		return ensMap_.at(d);
	}

	void put(const util::DateTime& d, boost::shared_ptr<Ensemble<TRAIT> > e) {
		ensMap_[d] = e;
	}

	static EnsemblesCollection<TRAIT>& getInstance() {
		static boost::shared_ptr<EnsemblesCollection<TRAIT> > instance_;
		if ( instance_.use_count() == 0 )
			instance_.reset(new EnsemblesCollection<TRAIT>());
		return *instance_;
	}

private:
	EnsemblesCollection() {
	}

	EnsemblesMap ensMap_;
};

// -----------------------------------------------------------------------------

}// namespace oops

#endif  // OOPS_ENSEMBLES_COLLECTION_H_
