/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_HYBRIDCOVARIANCE_H_
#define OOPS_BASE_HYBRIDCOVARIANCE_H_

#include <boost/scoped_ptr.hpp>

#include "oops/base/EnsembleCovariance.h"
#include "oops/base/Geometry.h"
#include "oops/base/ModelSpaceCovariance.h"
#include "oops/base/StaticCovariance.h"
#include "util/DateTime.h"
#include "util/Config.h"

namespace oops {

/// Generic hybrid static-ensemble model space error covariance.

// -----------------------------------------------------------------------------
template <typename TRAIT>
class HybridCovariance : public ModelSpaceCovariance<TRAIT> {
  typedef typename TRAIT::StateBase     State;
  typedef typename TRAIT::IncrementBase Increment;
  typedef StaticCovariance<TRAIT>       StaticCovariance_;
  typedef EnsembleCovariance<TRAIT>     EnsembleCovariance_;

 public:
  HybridCovariance(const Geometry &, const util::Config &);
  ~HybridCovariance();

  void linearize(const State &, const Geometry &, const util::Config &);

  void mult(const Increment &, Increment &) const;
  void inv_mult(const Increment &, Increment &) const;

  Increment * getIncrement() const;
  void randomize(Increment &) const;

 private:
  boost::scoped_ptr<StaticCovariance_> static_;
  boost::scoped_ptr<EnsembleCovariance_>  ensemble_;
  double ensWeight_;
};
// -----------------------------------------------------------------------------

}  // namespace oops

#include "oops/base/HybridCovariance.cc"

#endif  // OOPS_BASE_HYBRIDCOVARIANCE_H_
