/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "util/DateTime.h"
#include "util/Duration.h"

namespace oops {

// -----------------------------------------------------------------------------

template<typename TRAIT>
void LinearModel<TRAIT>::runTL(Increment & dx, const ModelAuxIncr & dmerr,
                               const util::Duration & len,
                               CostProcessor<Increment> & cost,
                               PostProcessor<Increment> & post) const {
  util::DateTime end(dx.validTime()+len);
  ModelIncrement<TRAIT> zz(dx, model_);
  zz.forecastTL(dmerr, end, mtraj_, post, cost);
  dx = zz;
}

// -----------------------------------------------------------------------------
template<typename TRAIT>
void LinearModel<TRAIT>::runAD(Increment & dx, ModelAuxIncr & dmerr,
                               const util::Duration & len,
                               CostProcessor<Increment> & cost,
                               PostProcessor<Increment> & post) const {
  util::DateTime bgn(dx.validTime()-len);
  ModelIncrement<TRAIT> zz(dx, model_);
  zz.forecastAD(dmerr, bgn, mtraj_, post, cost);
  dx = zz;
}

// -----------------------------------------------------------------------------

}  // namespace oops
