/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_LINEARMODEL_H_
#define OOPS_BASE_LINEARMODEL_H_

#include "oops/base/CostProcessor.h"
#include "oops/base/PostProcessor.h"
#include "oops/base/ModelIncrement.h"

namespace util {
  class Duration;
}

namespace oops {

/// LinearModel Class
/*!
 *  Holds linaer model configuration and trajectory.
 */

// -----------------------------------------------------------------------------

template <typename TRAIT> class LinearModel {
  typedef typename TRAIT::ModelData     ModelBase;
  typedef typename TRAIT::ModelTraj     ModelTraj;
  typedef typename TRAIT::IncrementBase Increment;
  typedef typename TRAIT::ModelAuxIncr  ModelAuxIncr;

 public:
  LinearModel(const ModelBase & model, const ModelTraj & traj)
    : model_(model), mtraj_(traj) {}

  ~LinearModel() {}

  const ModelBase & modelParam() const {return model_;}
  const ModelTraj & trajectory() const {return mtraj_;}

  void runTL(Increment &, const ModelAuxIncr &, const util::Duration &,
             CostProcessor<Increment> &, PostProcessor<Increment> &) const;
  void runAD(Increment &, ModelAuxIncr &, const util::Duration &,
             CostProcessor<Increment> &, PostProcessor<Increment> &) const;

 private:
  const ModelBase & model_;
  const ModelTraj & mtraj_;
};

// -----------------------------------------------------------------------------
}  // namespace oops

#include "oops/base/LinearModel.cc"

#endif  // OOPS_BASE_LINEARMODEL_H_
