/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "logger/Logger.h"
#include "util/asser1.h"

namespace oops {

// -----------------------------------------------------------------------------
/// Constructor, destructor
// -----------------------------------------------------------------------------
template<typename TRAIT>
ModelIncrement<TRAIT>::ModelIncrement(const Increment & dx, const Model & tlm)
  : TRAIT::IncrementBase(tlm.resolution(), tlm.variablesTL(), dx.validTime()), tlm_(tlm)
{
  Increment::operator=(dx);
  LOG(Trace) << "ModelIncrement created.";
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
ModelIncrement<TRAIT>::ModelIncrement(const Model & tlm, const util::DateTime & tt)
  : TRAIT::IncrementBase(tlm.resolution(), tlm.variablesTL(), tt), tlm_(tlm)
{
  LOG(Trace) << "ModelIncrement created.";
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
ModelIncrement<TRAIT>::~ModelIncrement() {
  LOG(Trace) << "ModelIncrement destructed";
}
// -----------------------------------------------------------------------------
/// Run forecast TL and AD
// -----------------------------------------------------------------------------
template<typename TRAIT>
void ModelIncrement<TRAIT>::forecastTL(const ModelAuxIncr & mctl,
                                       const util::DateTime & end,
                                       const ModelTraj & traj,
                                       PostProcessor<Increment> & post,
                                       CostProcessor<Increment> & cost) {
  LOG(Trace) << "ModelIncrement:forecastTL: Starting TL, time is " << this->validTime();
  this->initTL(tlm_, traj);
  post.initialize(this->validTime(), end, tlm_.timestep());
  cost.initializeTL(this->validTime(), end, tlm_.timestep());
  this->prtNorms("Start TL");
  post.process(*this);
  cost.processTL(*this);
  while (this->validTime() < end) {
    this->stepTL(tlm_, traj, mctl);
    post.process(*this);
    cost.processTL(*this);
  }
  ASSERT(this->validTime() == end);
  post.finalize();
  cost.finalizeTL();
  LOG(Trace) << "ModelIncrement:forecastTL: Finished TL, time is " << this->validTime();
  this->prtNorms("End TL");
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void ModelIncrement<TRAIT>::forecastAD(ModelAuxIncr & mctl,
                                       const util::DateTime & bgn,
                                       const ModelTraj & traj,
                                       PostProcessor<Increment> & post,
                                       CostProcessor<Increment> & cost) {
  LOG(Trace) << "ModelIncrement:forecastAD: Starting AD, time is " << this->validTime();
  this->prtNorms("Start AD");
  post.initialize(bgn, this->validTime(), tlm_.timestep());
  cost.initializeAD(bgn, this->validTime(), tlm_.timestep());
  while (this->validTime() > bgn) {
    post.process(*this);
    cost.processAD(*this);
    this->stepAD(tlm_, traj, mctl);
  }
  post.process(*this);
  cost.processAD(*this);
  this->initAD(tlm_, traj);
  ASSERT(this->validTime() == bgn);
  post.finalize();
  cost.finalizeAD();
  LOG(Trace) << "ModelIncrement:forecastAD: Finished AD, time is " << this->validTime();
  this->prtNorms("End AD");
}
// -----------------------------------------------------------------------------

}  // namespace oops
