/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "logger/Logger.h"
#include "util/abor1_cpp.h"
#include "util/Config.h"

namespace oops {

// -----------------------------------------------------------------------------

template <typename TRAIT>
std::map < std::string, CovarianceFactory<TRAIT> * > CovarianceFactory<TRAIT>::makers_;

// -----------------------------------------------------------------------------

template <typename TRAIT>
CovarianceFactory<TRAIT>::CovarianceFactory(const std::string & name) {
  if (makers_.find(name) != makers_.end()) {
    LOG(Error) << name << " already registered in covariance factory.";
    ABORT("Element already registered in CovarianceFactory.");
  }
  makers_[name] = this;
}

// -----------------------------------------------------------------------------

template <typename TRAIT>
ModelSpaceCovariance<TRAIT>* CovarianceFactory<TRAIT>::create(
                                                         const util::Config & conf,
                                                         const Geometry & resol) {
  const std::string id = conf.getData("covariance_type");
  LOG(Trace) << "ModelSpaceCovariance type = " << id;
  typename std::map<std::string, CovarianceFactory<TRAIT>*>::iterator
    j = makers_.find(id);
  if (j == makers_.end()) {
    LOG(Error) << id << " does not exist in covariance factory.";
    ABORT("Element does not exist in CovarianceFactory.");
  }
  return (*j).second->make(conf, resol);
}

// -----------------------------------------------------------------------------

}  // namespace oops
