/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "oops/base/Geometry.h"

#include "logger/Logger.h"
#include "util/Config.h"
#include "util/DateTime.h"
#include "util/asser1.h"

namespace oops {

// -----------------------------------------------------------------------------
template<typename TRAIT>
ModelState<TRAIT>::ModelState(const Model & model, const util::Config & file)
  : StateBase(model.resolution(), model.variables(), util::DateTime()), model_(model)
{
  this->read(file);
  LOG(Trace) << "ModelState:ModelState created and read in.";
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
ModelState<TRAIT>::ModelState(const StateBase & xx, const Model & model)
  : StateBase(model.resolution(), model.variables(), xx.validTime()), model_(model)
{
  StateBase::operator=(xx);
  LOG(Trace) << "ModelState:ModelState created from StateBase.";
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
ModelState<TRAIT>::~ModelState() {
  LOG(Trace) << "ModelState:ModelState destructed.";
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void ModelState<TRAIT>::forecast(const ModelAuxCtrl & mctl, const util::DateTime & end,
                                 PostProcessor<StateBase> & post) {
  LOG(Info) << "ModelState:forecast: Starting forecast, time is " << this->validTime();
  this->prtNorms("Start NL");
  post.initialize(this->validTime(), end, model_.timestep());
  this->init(model_);
  post.process(*this);
  while (this->validTime() < end) {
    this->step(model_, mctl);
    post.process(*this);
  }
  ASSERT(this->validTime() == end);
  post.finalize();
  LOG(Info) << "ModelState:forecast: Finished forecast, time is " << this->validTime();
  this->prtNorms("End NL");
}
// -----------------------------------------------------------------------------

}  // namespace oops
