/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_OBSERRORCOVARIANCE_H_
#define OOPS_BASE_OBSERRORCOVARIANCE_H_

#include <boost/noncopyable.hpp>
#include <boost/ptr_container/ptr_vector.hpp>

#include "oops/base/Departure.h"
#include "oops/base/Observation.h"
#include "oops/observations/ObsErrorBase.h"

namespace oops {

// -----------------------------------------------------------------------------
/// Observation error covariance matrix
/*!
 *  This class provides the operations associated with the observation
 *  error covariance matrix. It contains a list of observation error
 *  covariances specific for each observation type.
 */

template <typename TRAIT>
class ObsErrorCovariance : private boost::noncopyable {
  typedef Observation<TRAIT>        Observ;
  typedef Departure<TRAIT>          Depart;
  typedef ObsErrorBase<TRAIT>       ObsErr;

 public:
  ObsErrorCovariance(const Observ &, const util::Config &);
  ~ObsErrorCovariance();

/// Linearize and reset for inner loop if needed
  void linearize(const Observ &);

/// Multiply a Departure by \f$R\f$
  void mult(const Depart &, Depart &) const;
  void mult_ad(const Depart &, Depart &) const;

/// Multiply a Departure by \f$R^{-1}\f$
  void inv_mult(const Depart &, Depart &) const;
  void inv_mult_ad(const Depart &, Depart &) const;

/// Compute and print Jo table
  double calcJo(const Depart &, Depart &) const;

/// Generate random perturbation
  void randomize(Depart &) const;

 private:
  boost::ptr_vector<ObsErr> obserr_;
};

}  // namespace oops

#include "oops/base/ObsErrorCovariance.cc"

#endif  // OOPS_BASE_OBSERRORCOVARIANCE_H_
