/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "logger/Logger.h"
#include "util/asser1.h"

namespace oops {

// -----------------------------------------------------------------------------
template <typename STATE, typename OBS, typename BIAS, typename TRAJ>
Observer<STATE, OBS, BIAS, TRAJ>::Observer(const util::Config & conf,
               OBS & yobs, const BIAS & ybias, const bool subwin):
  ProcessorBase<STATE>(conf),
  yy_(yobs), ybias_(ybias), traj_(0), tslots_(0), subwindows_(subwin)
{}
// -----------------------------------------------------------------------------
template <typename STATE, typename OBS, typename BIAS, typename TRAJ>
Observer<STATE, OBS, BIAS, TRAJ>::Observer(OBS & yobs, const BIAS & ybias,
               const util::Duration & tstep, const bool subwin):
  ProcessorBase<STATE>(),
  yy_(yobs), ybias_(ybias), traj_(0), tslots_(tstep), subwindows_(subwin)
{}
// -----------------------------------------------------------------------------
template <typename STATE, typename OBS, typename BIAS, typename TRAJ>
void Observer<STATE, OBS, BIAS, TRAJ>::setTraj() {
  traj_ = new TRAJ();
}
// -----------------------------------------------------------------------------
template <typename STATE, typename OBS, typename BIAS, typename TRAJ>
TRAJ * Observer<STATE, OBS, BIAS, TRAJ>::releaseTraj() {
  ASSERT(traj_ != 0);
  return traj_;
}
// -----------------------------------------------------------------------------
template <typename STATE, typename OBS, typename BIAS, typename TRAJ>
void Observer<STATE, OBS, BIAS, TRAJ>::doInitialize(const util::DateTime & bgn,
                        const util::DateTime & end, const util::Duration & tstep) {
  if (tslots_ == util::Duration(0)) tslots_ = tstep;
  if (subwindows_) yy_.subwindow(bgn, end);
  if (traj_ == 0) {
    yy_.obsOperatorSetup(tslots_);
  } else {
    yy_.obsOperatorSetup(tslots_, *traj_);
  }
}
// -----------------------------------------------------------------------------
template <typename STATE, typename OBS, typename BIAS, typename TRAJ>
void Observer<STATE, OBS, BIAS, TRAJ>::doProcessing(const STATE & xx) {
  if (traj_ == 0) {
    yy_.obsOperator(xx, ybias_);
  } else {
    yy_.obsOperator(xx, ybias_, *traj_);
  }
}
// -----------------------------------------------------------------------------
template <typename STATE, typename OBS, typename BIAS, typename TRAJ>
void Observer<STATE, OBS, BIAS, TRAJ>::doFinalize() {
  if (traj_ == 0) {
    yy_.obsOperatorFinal(ybias_);
  } else {
    yy_.obsOperatorFinal(ybias_, *traj_);
  }
}
// -----------------------------------------------------------------------------

}  // namespace oops
