/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_OBSERVER_H_
#define OOPS_BASE_OBSERVER_H_

#include "oops/base/ProcessorBase.h"
#include "util/Config.h"
#include "util/DateTime.h"
#include "util/Duration.h"

namespace oops {

/// Computes observation equivalent computations from forecast fields.

// Observer should take ownership of OBS and release it when done. YT

// Sub-windows knowledge could be removed if vector of obs was used in
// weak constraint 4D-Var. YT

template <typename STATE, typename OBS, typename BIAS, typename TRAJ>
class Observer : public ProcessorBase<STATE> {
 public:

  Observer(const util::Config &, OBS &, const BIAS &, const bool subwin = false);
  Observer(OBS &, const BIAS &, const util::Duration &, const bool subwin = false);

  ~Observer() {}

  void setTraj();
  TRAJ * releaseTraj();

 private:
  OBS & yy_;
  const BIAS & ybias_;
  TRAJ * traj_;

  util::Duration tslots_;
  const bool subwindows_;

  void doProcessing(const STATE &);
  void doInitialize(const util::DateTime &, const util::DateTime &,
                    const util::Duration &);
  void doFinalize();
};

}  // namespace oops

#include "oops/base/Observer.cc"

#endif  // OOPS_BASE_OBSERVER_H_
