/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "util/asser1.h"

namespace oops {

// -----------------------------------------------------------------------------
template <typename INCR, typename TRAIT>
void ObserverTLAD<INCR, TRAIT>::setupTL(Depart * dytl, const ObsAuxIncr & ybtl) {
  dytl_ = dytl;
  ybtl_ = &ybtl;
}
// -----------------------------------------------------------------------------
template <typename INCR, typename TRAIT>
void ObserverTLAD<INCR, TRAIT>::doFirstTL(const util::DateTime & bgn,
                           const util::DateTime & end, const util::Duration & tstep) {
  if (tslots_ == util::Duration(0)) tslots_ = tstep;
  if (subwindows_) dytl_->subwindow(bgn, end);
  dytl_->obsOperatorTLSetup(tslots_);
}
// -----------------------------------------------------------------------------
template <typename INCR, typename TRAIT>
void ObserverTLAD<INCR, TRAIT>::doProcessingTL(const INCR & dx) {
  dytl_->obsOperatorTL(dx, *ybtl_, traj_);
}
// -----------------------------------------------------------------------------
template <typename INCR, typename TRAIT>
void ObserverTLAD<INCR, TRAIT>::doLastTL() {
  dytl_->obsOperatorTLFinal(*ybtl_, traj_);
  tldone_ = true;
}
// -----------------------------------------------------------------------------
template <typename INCR, typename TRAIT>
Departure<TRAIT> * ObserverTLAD<INCR, TRAIT>::finalTL() {
  ASSERT(tldone_);
  tldone_ = false;
  ybtl_ = 0;
  return dytl_;
}
// -----------------------------------------------------------------------------
template <typename INCR, typename TRAIT>
void ObserverTLAD<INCR, TRAIT>::setupAD(boost::shared_ptr<const Depart> dyad,
                                        ObsAuxIncr * ybad) {
  dyad_ = dyad;
  ybad_ = ybad;
}
// -----------------------------------------------------------------------------
template <typename INCR, typename TRAIT>
void ObserverTLAD<INCR, TRAIT>::doFirstAD(const util::DateTime & bgn,
                           const util::DateTime & end, const util::Duration & tstep) {
  if (tslots_ == util::Duration(0)) tslots_ = tstep;
  if (subwindows_) dyad_->subwindow(bgn, end);
  dyad_->obsOperatorADFinal(tslots_, *ybad_, traj_);
}
// -----------------------------------------------------------------------------
template <typename INCR, typename TRAIT>
void ObserverTLAD<INCR, TRAIT>::doProcessingAD(INCR & dx) {
  dyad_->obsOperatorAD(dx, *ybad_, traj_);
}
// -----------------------------------------------------------------------------
template <typename INCR, typename TRAIT>
void ObserverTLAD<INCR, TRAIT>::doLastAD() {
  dyad_->obsOperatorADSetup();
  addone_ = true;
}
// -----------------------------------------------------------------------------
template <typename INCR, typename TRAIT>
typename TRAIT::ObsAuxIncr * ObserverTLAD<INCR, TRAIT>::finalAD() {
  ASSERT(addone_);
  addone_ = false;
  dyad_.reset();
  return ybad_;
}
// -----------------------------------------------------------------------------

}  // namespace oops
