/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_OBSERVERTLAD_H_
#define OOPS_BASE_OBSERVERTLAD_H_

#include <boost/shared_ptr.hpp>

#include "oops/base/CostBase.h"
#include "util/DateTime.h"
#include "util/Duration.h"

namespace oops {

/// Computes observation equivalent TL and AD to/from increments.
/*!
 *  ObserverTLAD should but cannot take ownership of its trajectory
 *  because the trecjtory encompasses all subwindows while the rest
 *  of the setup does not. One solution would be to use a vector of
 *  Observation in the cost function, with one Observation for each
 *  subwindow.
 */

template <typename INCR, typename TRAIT>
class ObserverTLAD : public CostBase<INCR> {
  typedef typename TRAIT::ObsAuxIncr ObsAuxIncr;
  typedef typename TRAIT::ObsTraj    ObsTraject;
  typedef Departure<TRAIT>           Depart;
  typedef ObsTrajectory<ObsTraject>  ObsTraj;

 public:

  ObserverTLAD(const ObsTraj & traj, const util::Duration & tslot,
               const bool subwin = false)
    : CostBase<INCR>(), traj_(traj), tslots_(tslot), subwindows_(subwin),
      dytl_(0), ybtl_(0), dyad_(), ybad_(0),
      tldone_(false), addone_(false) {}

  ~ObserverTLAD() {}

  void setupTL(Depart *, const ObsAuxIncr &);
  Depart * finalTL();
  void setupAD(boost::shared_ptr<const Depart>, ObsAuxIncr *);
  ObsAuxIncr * finalAD();

 private:
// TL
  void doFirstTL(const util::DateTime &, const util::DateTime &,
                 const util::Duration &);
  void doProcessingTL(const INCR &);
  void doLastTL();

// AD
  void doFirstAD(const util::DateTime &, const util::DateTime &,
                 const util::Duration &);
  void doProcessingAD(INCR &);
  void doLastAD();

// Data
  const ObsTraj & traj_;
  util::Duration tslots_;
  const bool subwindows_;
  Depart * dytl_;
  const ObsAuxIncr * ybtl_;
  boost::shared_ptr<const Depart> dyad_;
  ObsAuxIncr * ybad_;
  bool tldone_;
  bool addone_;
};

}  // namespace oops

#include "oops/base/ObserverTLAD.cc"

#endif  // OOPS_BASE_OBSERVERTLAD_H_
