/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_POSTPROCESSOR_H_
#define OOPS_BASE_POSTPROCESSOR_H_

#include <vector>

#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>

#include "oops/base/ProcessorBase.h"

namespace oops {

/// Control model post processing
/*!
 *  This class controls model post processing in the most general sense,
 *  ie all diagnostics computations that do not affect the model integration.
 *  It just calls all the individual processors one by one.
 */

template<typename FLDS>
class PostProcessor {
  typedef ProcessorBase<FLDS>  Process;
 public:
  PostProcessor() {}
  PostProcessor(const PostProcessor & pp): processors_(pp.processors_) {}
  ~PostProcessor() {}

  void enroll(Process * pp) {
    if (pp != 0) {
      boost::shared_ptr<Process> sp(pp);
      processors_.push_back(sp);
    }
  }

  void enroll(boost::shared_ptr<Process> pp) {
    if (pp != 0) processors_.push_back(pp);
  }

  void initialize(const util::DateTime & bgn, const util::DateTime & end,
                  const util::Duration & step) {
    BOOST_FOREACH(boost::shared_ptr<Process> jp, processors_)
      jp->initialize(bgn, end, step);
  }

  void process(const FLDS & xx) {
    BOOST_FOREACH(boost::shared_ptr<Process> jp, processors_) jp->process(xx);
  }

  void finalize() {
    BOOST_FOREACH(boost::shared_ptr<Process> jp, processors_) jp->finalize();
  }

 private:
  std::vector< boost::shared_ptr<Process> > processors_;
  PostProcessor operator= (const PostProcessor &);
};

}  // namespace oops

#endif  // OOPS_BASE_POSTPROCESSOR_H_
