/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "logger/Logger.h"
#include "util/asser1.h"

namespace oops {

// -----------------------------------------------------------------------------
/// Constructor, destructor
// -----------------------------------------------------------------------------
template<typename TRAIT>
StaticCovariance<TRAIT>::StaticCovariance(const Geometry & resol,
                                          const util::Config & config):
  covar_(resol, config)
{
  LOG(Trace) << "StaticCovariance created.";
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
StaticCovariance<TRAIT>::~StaticCovariance() {
  LOG(Trace) << "StaticCovariance destructed";
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void StaticCovariance<TRAIT>::linearize(const State & xb,
                                        const Geometry & resol,
                                        const util::Config & tlConf) {
  ASSERT(xb.validTime() == covar_.validTime());
  covar_.linearize(xb, resol, tlConf);
  LOG(Trace) << "StaticCovariance linearized: time = " << covar_.validTime();
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void StaticCovariance<TRAIT>::mult(const Increment & dxi, Increment & dxo) const {
  ASSERT(dxi.validTime() == covar_.validTime());
  covar_.mult(dxi, dxo);
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void StaticCovariance<TRAIT>::inv_mult(const Increment & dxi, Increment & dxo) const {
  ASSERT(dxi.validTime() == covar_.validTime());
  covar_.inv_mult(dxi, dxo);
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void StaticCovariance<TRAIT>::randomize(Increment & dx) const {
  covar_.randomize(dx);
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
typename TRAIT::IncrementBase * StaticCovariance<TRAIT>::getIncrement() const {
  Increment * dx = new Increment(covar_.resolution(), covar_.ctrlvars(),
                                 covar_.validTime());
  return dx;
}
// -----------------------------------------------------------------------------
}  // namespace oops
