/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_STATICCOVARIANCE_H_
#define OOPS_BASE_STATICCOVARIANCE_H_

#include <boost/noncopyable.hpp>

#include "oops/base/Geometry.h"
#include "oops/base/ModelSpaceCovariance.h"
#include "util/DateTime.h"
#include "util/Config.h"

namespace oops {

/// Wraps the model space error covariance.

// -----------------------------------------------------------------------------
template <typename TRAIT>
class StaticCovariance : public ModelSpaceCovariance<TRAIT>, private boost::noncopyable {
  typedef typename TRAIT::ModelStateError ModelSpaceError;
  typedef typename TRAIT::StateBase       State;
  typedef typename TRAIT::IncrementBase   Increment;

 public:
  StaticCovariance(const Geometry &, const util::Config &);
  ~StaticCovariance();

  void linearize(const State &, const Geometry &, const util::Config &);

  void mult(const Increment &, Increment &) const;
  void inv_mult(const Increment &, Increment &) const;

  Increment * getIncrement() const;
  void randomize(Increment &) const;

  const util::DateTime & validTime() const {return covar_.validTime();}

 private:
  ModelSpaceError covar_;
};
// -----------------------------------------------------------------------------

}  // namespace oops

#include "oops/base/StaticCovariance.cc"

#endif  // OOPS_BASE_STATICCOVARIANCE_H_
