/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "util/asser1.h"

namespace oops {

// -----------------------------------------------------------------------------
template <typename TRAIT>
TrajSaver<TRAIT>::TrajSaver(const util::Config & conf, const Geometry & resol,
                            const Model & model):
  ProcessorBase<State>(conf),
  resol_(resol), model_(model), traj_(), current_(0)
{}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void TrajSaver<TRAIT>::doInitialize(const util::DateTime &,
                                    const util::DateTime &,
                                    const util::Duration &) {
  current_ = new Trajectory(resol_, model_);
  traj_.push_back(current_);
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void TrajSaver<TRAIT>::doProcessing(const State & xx) {
  ASSERT(current_ != 0);
  current_->set(xx);
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void TrajSaver<TRAIT>::doFinalize() {
  current_ = 0;
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
std::auto_ptr< boost::ptr_vector<typename TRAIT::ModelTraj> >
  TrajSaver<TRAIT>::releaseTraj()
{
  ASSERT(current_ == 0);
  return traj_.release();
}
// -----------------------------------------------------------------------------

}  // namespace oops
