/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_TRAJSAVER_H_
#define OOPS_BASE_TRAJSAVER_H_

#include <memory>

#include <boost/ptr_container/ptr_vector.hpp>

#include "oops/base/Geometry.h"
#include "oops/base/ProcessorBase.h"
#include "util/Config.h"
#include "util/DateTime.h"
#include "util/Duration.h"

namespace oops {

/// Save trajectory during forecast run.

template <typename TRAIT>
class TrajSaver : public ProcessorBase< typename TRAIT::StateBase > {
  typedef typename TRAIT::ModelData  Model;
  typedef typename TRAIT::ModelTraj  Trajectory;
  typedef typename TRAIT::StateBase  State;

 public:

  TrajSaver(const util::Config &, const Geometry &, const Model &);
  ~TrajSaver() {}

  std::auto_ptr< boost::ptr_vector<Trajectory> > releaseTraj();

 private:
  const Geometry resol_;
  const Model & model_;
  boost::ptr_vector<Trajectory> traj_;
  Trajectory * current_;

  void doProcessing(const State &);
  void doInitialize(const util::DateTime &, const util::DateTime &,
                    const util::Duration &);
  void doFinalize();
};

}  // namespace oops

#include "oops/base/TrajSaver.cc"

#endif  // OOPS_BASE_TRAJSAVER_H_
