/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_WEIGHTEDDIFF_H_
#define OOPS_BASE_WEIGHTEDDIFF_H_

#include <map>

#include <boost/scoped_ptr.hpp>

#include "oops/base/Geometry.h"
#include "oops/base/ProcessorBase.h"
#include "oops/base/Variables.h"
#include "util/Accumulator.h"
#include "util/DateTime.h"
#include "util/Duration.h"
#include "util/DolphChebyshev.h"
#include "util/asser1.h"

namespace oops {

// -----------------------------------------------------------------------------

/// Compute time average of states or increments during model run.
/*!
 *  Derived classes will compute different types of averages (plain
 *  mean, various types of digital filters) by overwriting the weights
 *  computation method.
 */

template <typename INCR, typename FLDS>
class WeightedDiff : public ProcessorBase<FLDS> {
 public:
  WeightedDiff(const util::DateTime &, const util::Duration &,
               const Geometry &, const util::Config &,
               const util::Duration &, WeightingFct &);
  virtual ~WeightedDiff() {}

  void setup();
  INCR * release();

 private:
  void doInitialize(const util::DateTime &, const util::DateTime &,
                    const util::Duration &);
  void doProcessing(const FLDS &);

  WeightingFct & wfct_;
  std::map< util::DateTime, double > weights_;
//  std::unique_ptr< Accumulator<INCR, FLDS> > avg_;
  Accumulator<INCR, FLDS> * avg_;
  double sum_;
  bool linit_;
  const util::DateTime vtime_;
  const util::DateTime bgn_;
  const util::DateTime end_;
  util::Duration tstep_;
  util::DateTime bgnleg_;
  util::DateTime endleg_;
  oops::Variables vars_;
  oops::Geometry resol_;
  bool lsetup_;
};

// -----------------------------------------------------------------------------

}  // namespace oops

#include "oops/base/WeightedDiff.cc"

#endif  // OOPS_BASE_WEIGHTEDDIFF_H_
