/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_BASE_WEIGHTEDMEAN_H_
#define OOPS_BASE_WEIGHTEDMEAN_H_

#include <map>

#include <boost/scoped_ptr.hpp>

#include "oops/base/ProcessorBase.h"
#include "util/Accumulator.h"
#include "util/DateTime.h"
#include "util/Duration.h"
#include "util/DolphChebyshev.h"
#include "util/asser1.h"

namespace oops {

// -----------------------------------------------------------------------------

/// Compute time average of states or increments during model run.
/*!
 *  Derived classes will compute different types of averages (plain
 *  mean, various types of digital filters) by overwriting the weights
 *  computation method.
 */

template <typename FLDS>
class WeightedMean : public ProcessorBase<FLDS> {
 public:
  WeightedMean(const util::DateTime &, const util::Duration &,
               const Geometry &, const util::Config &);
  virtual ~WeightedMean() {}

  FLDS * release();

 private:
  void doInitialize(const util::DateTime &, const util::DateTime &,
                    const util::Duration &);

  void doProcessing(const FLDS &);

  boost::scoped_ptr<WeightingFct> wfct_;
  std::map< util::DateTime, double > weights_;
//  std::unique_ptr< Accumulator<FLDS, FLDS> > avg_;
  Accumulator<FLDS, FLDS> * avg_;
  double sum_;
  bool linit_;
  const util::DateTime bgn_;
  const util::DateTime end_;
  util::DateTime endleg_;
};

// -----------------------------------------------------------------------------

}  // namespace oops

#include "oops/base/WeightedMean.cc"

#endif  // OOPS_BASE_WEIGHTEDMEAN_H_
