/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

namespace oops {
// -----------------------------------------------------------------------------
template <typename TRAIT>
ObsCumulated<TRAIT>::ObsCumulated(const ObsEqv & equiv)
  : gom_(0), gomtl_(0), gomad_(0), equiv_(&equiv)
{
  LOG(Trace) << "ObsCumulated created";
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
ObsCumulated<TRAIT>::~ObsCumulated() {}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::setup(const util::DateTime & t1,
                                const util::DateTime & t2) {
  gom_.reset(equiv_->getGOM(t1, t2));
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::obsOperator(
    const State & xx, ObsVector &, const ObsAuxCtrl &,
    const util::DateTime & t1, const util::DateTime & t2) {
// Get locations info for interpolator
  Locations * locs = equiv_->locations(t1, t2);

// Interpolate state variables to obs locations
  if (locs->nobs()>0) xx.interpolate(*locs, *gom_);

  delete locs;
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::obsOperator(
    const State & xx, ObsVector & yeqv, const ObsAuxCtrl & octl,
    const util::DateTime & t1, const util::DateTime & t2, ObsTraj &) {
  obsOperator(xx, yeqv, octl, t1, t2);
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::final(ObsVector & yeqv, const ObsAuxCtrl & octl) {
  equiv_->obsEquiv(*gom_, yeqv, octl);
  gom_.reset();
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::final(ObsVector & yeqv, const ObsAuxCtrl & octl,
                                ObsTraj & traj) {
  equiv_->obsEquiv(*gom_, yeqv, octl, traj);
  gom_.reset();
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::setupTL(const util::DateTime & t1,
                                  const util::DateTime & t2) {
  gomtl_.reset(equiv_->getGOM(t1, t2));
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::obsOperatorTL(
    const Increment & xx, ObsVector &, const ObsAuxIncr &,
    const util::DateTime & t1, const util::DateTime & t2, const ObsTraj &) {
// Get locations info for interpolator
  Locations * locs = equiv_->locations(t1, t2);

// Interpolate state variables to obs locations
  if (locs->nobs()>0) xx.interpolateTL(*locs, *gomtl_);

  delete locs;
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::finalTL(ObsVector & dytl, const ObsAuxIncr & octl,
                                  const ObsTraj & traj) {
  equiv_->obsEquivTL(*gomtl_, dytl, octl, traj);
  gomtl_.reset();
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::finalAD(
    const util::DateTime & t1, const util::DateTime & t2,
    const ObsVector & dyad, ObsAuxIncr & octl, const ObsTraj & traj) {
  gomad_.reset(equiv_->getGOM(t1, t2));
  LOG(Trace) << "ObsCumulated:finalAD got GOM";
  equiv_->obsEquivAD(*gomad_, dyad, octl, traj);
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::obsOperatorAD(
    Increment & xx, const ObsVector &, ObsAuxIncr &,
    const util::DateTime & t1, const util::DateTime & t2, const ObsTraj &) {
// Get locations info for interpolator
  Locations * locs = equiv_->locations(t1, t2);

// Interpolate state variables to obs locations
  if (locs->nobs() > 0) xx.interpolateAD(*locs, *gomad_);

  delete locs;
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsCumulated<TRAIT>::setupAD() {
  gomad_.reset();
  LOG(Trace) << "ObsCumulated:setupAD reset GOM";
}
// -----------------------------------------------------------------------------
}  // namespace oops
