/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include <map>
#include <string>

#include "logger/Logger.h"
#include "util/abor1_cpp.h"
#include "util/Config.h"

namespace oops {

// -----------------------------------------------------------------------------

template <typename TRAIT>
ObsErrorFactory<TRAIT>::ObsErrorFactory(const std::string & name) {
  if (getMakers().find(name) != getMakers().end()) {
    LOG(Error) << name << " already registered in observation error factory.";
    ABORT("Element already registered in ObsErrorFactory.");
  }
  getMakers()[name] = this;
}

// -----------------------------------------------------------------------------

template <typename TRAIT>
ObsErrorBase<TRAIT>* ObsErrorFactory<TRAIT>::create(const Observ & obs,
                                                    const util::Config & conf) {
  const std::string id = conf.getData("Covariance");
  typename std::map<string, ObsErrorFactory<TRAIT>*>::iterator j = getMakers().find(id);
  if (j == getMakers().end()) {
    LOG(Error) << id << " does not exist in observation error factory.";
    ABORT("Element does not exist in ObsErrorFactory.");
  }
  return (*j).second->make(obs, conf);
}

// -----------------------------------------------------------------------------
}  // namespace oops
