/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "logger/Logger.h"
#include "util/Config.h"

namespace oops {

// -----------------------------------------------------------------------------
template<typename TRAIT>
ObsErrorDiag<TRAIT>::ObsErrorDiag(const Observ & yobs, const util::Config & conf) {
// Get obs error
  Depart tmp(yobs);
  const util::Config dataConfig(conf, "ObsData");
  const string errname = dataConfig.getData("obserror");
  tmp.read(errname);

  meanerr_ = tmp.mean();

// Compute in inverse of variances
  tmp *= tmp;
  tmp.invert();

// Get diagonal matrix
  Rinv_.reset(new oops::DiagonalMatrix<Depart>(tmp));
  LOG(Trace) << "ObsErrorDiag:ObsErrorDiag constructed";
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
ObsErrorDiag<TRAIT>::~ObsErrorDiag() {
  LOG(Trace) << "ObsErrorDiag:~ObsErrorDiag destructed";
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void ObsErrorDiag<TRAIT>::linearize(const Observ &) {}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void ObsErrorDiag<TRAIT>::mult(const Depart & dyi, Depart & dyo) const {
  Rinv_->inv_mult(dyi, dyo);
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void ObsErrorDiag<TRAIT>::mult_ad(const Depart & dyi, Depart & dyo) const {
  Depart tmp(dyi);
  Rinv_->inv_mult(dyi, tmp);
  dyo += tmp;
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void ObsErrorDiag<TRAIT>::inv_mult(const Depart & dyi, Depart & dyo) const {
  Rinv_->mult(dyi, dyo);
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void ObsErrorDiag<TRAIT>::inv_mult_ad(const Depart & dyi, Depart & dyo) const {
  Depart tmp(dyi);
  Rinv_->mult(dyi, tmp);
  dyo += tmp;
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
double ObsErrorDiag<TRAIT>::calcJo(const Depart & dy, Depart & dz, int & nobs) const {
  Rinv_->mult(dy, dz);
  const double zjo = 0.5 * dot_product(dy, dz);

  nobs = dy.nobs();

// Print
  if (nobs > 0) {
    LOG(Info) << "Type " << dy.obsname();
    LOG(Info) << "    " << zjo << "    " << nobs
              << "    " << zjo/nobs << "    " << meanerr_;
  } else {
    LOG(Info) << "WARNING: *** No observations !!! ***";
  }
  return zjo;
}
// -----------------------------------------------------------------------------
template<typename TRAIT>
void ObsErrorDiag<TRAIT>::randomize(Depart & dy) const {
  Depart tmp(dy);
  tmp.random();
  Rinv_->inv_mult(tmp, dy);
}
// -----------------------------------------------------------------------------

}  // namespace oops
