/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_OBSERVATIONS_OBSERRORDIAG_H_
#define OOPS_OBSERVATIONS_OBSERRORDIAG_H_

#include <boost/scoped_ptr.hpp>

#include "oops/observations/DepType.h"
#include "oops/observations/ObsType.h"
#include "oops/observations/ObsErrorBase.h"
#include "util/DiagonalMatrix.h"

namespace util {
  class Config;
}

namespace oops {

// -----------------------------------------------------------------------------
/// Observation error covariance matrix for a given observation type.
/*!
 *  Observation error covariance matrix for a given observation type
 *  is diagonal for now, should be generalized.
 */

template<typename TRAIT>
class ObsErrorDiag : public ObsErrorBase<TRAIT> {
  typedef oops::ObsType<TRAIT> Observ;
  typedef oops::DepType<TRAIT> Depart;

 public:
  ObsErrorDiag(const Observ &, const util::Config &);
  ~ObsErrorDiag();

/// Linearize and reset for inner loop if needed
  void linearize(const Observ &);

/// Multiply a Departure by \f$R\f$
  void mult(const Depart &, Depart &) const;
  void mult_ad(const Depart &, Depart &) const;

/// Multiply a Departure by \f$R^{-1}\f$
  void inv_mult(const Depart &, Depart &) const;
  void inv_mult_ad(const Depart &, Depart &) const;

/// Print Jo table
  double calcJo(const Depart &, Depart &, int &) const;

/// Generate random perturbation
  void randomize(Depart &) const;

 private:
  double meanerr_;
  boost::scoped_ptr< oops::DiagonalMatrix<Depart> > Rinv_;
};
// -----------------------------------------------------------------------------
}  // namespace oops

#include "oops/observations/ObsErrorDiag.cc"

#endif  // OOPS_OBSERVATIONS_OBSERRORDIAG_H_
