/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

namespace oops {
// -----------------------------------------------------------------------------
template <typename TRAIT>
ObsOnTheFly<TRAIT>::ObsOnTheFly(const ObsEqv & equiv)
  : equiv_(&equiv)
{LOG(Trace) << "ObsOnTheFly created";}
// -----------------------------------------------------------------------------
template <typename TRAIT>
ObsOnTheFly<TRAIT>::~ObsOnTheFly() {}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::setup(const DateTime &, const DateTime &) {}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::obsOperator(
    const State & xx, ObsVector & yeqv, const ObsAuxCtrl & octl,
    const util::DateTime & t1, const util::DateTime & t2) {
// Get locations info for interpolator
  Locations * locs = equiv_->locations(t1, t2);

  if (locs->nobs()>0) {
    boost::scoped_ptr<LocalModelValues> gom(equiv_->getGOM(t1, t2));

//  Interpolate state variables to obs locations
    xx.interpolate(*locs, *gom);

//  Compute Obs Equivalent
    equiv_->obsEquiv(*gom, yeqv, octl);
  }
  delete locs;
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::obsOperator(
    const State & xx, ObsVector & yeqv, const ObsAuxCtrl & octl,
    const util::DateTime & t1, const util::DateTime & t2, ObsTraj & traj) {
// Get locations info for interpolator
  Locations * locs = equiv_->locations(t1, t2);

  if (locs->nobs() > 0) {
    boost::shared_ptr<LocalModelValues> gom(equiv_->getGOM(t1, t2));

// Interpolate state variables to obs locations
    xx.interpolate(*locs, *gom);

// Compute Obs Equivalent
    equiv_->obsEquiv(*gom, yeqv, octl, traj);
  }
  delete locs;
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::final(ObsVector &, const ObsAuxCtrl &) {}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::final(ObsVector &, const ObsAuxCtrl &, ObsTraj &) {}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::setupTL(const DateTime &, const DateTime &) {}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::obsOperatorTL(
    const Increment & xx, ObsVector & dytl, const ObsAuxIncr & octl,
    const util::DateTime & t1, const util::DateTime & t2, const ObsTraj & traj) {
// Get locations info for interpolator
  Locations * locs = equiv_->locations(t1, t2);

  if (locs->nobs()>0) {
    boost::shared_ptr<LocalModelValues> gom(equiv_->getGOM(t1, t2));

//  Interpolate state variables to obs locations
    xx.interpolateTL(*locs, *gom);

//  Compute Linear Departure
    equiv_->obsEquivTL(*gom, dytl, octl, traj);
  }
  delete locs;
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::finalTL(ObsVector &, const ObsAuxIncr &,
                          const ObsTraj &) {}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::finalAD(const DateTime &, const DateTime &,
                          const ObsVector &, ObsAuxIncr &,
                          const ObsTraj &) {}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::obsOperatorAD(
    Increment & xx, const ObsVector & dyad, ObsAuxIncr & octl,
    const util::DateTime & t1, const util::DateTime & t2, const ObsTraj & traj) {
// Get locations info for interpolator
  Locations * locs = equiv_->locations(t1, t2);

  if (locs->nobs() > 0) {
    boost::shared_ptr<LocalModelValues> gom(equiv_->getGOM(t1, t2));

// Compute Departure Adjoint
    equiv_->obsEquivAD(*gom, dyad, octl, traj);

// Interpolate state variables to obs locations
    xx.interpolateAD(*locs, *gom);
  }
  delete locs;
}
// -----------------------------------------------------------------------------
template <typename TRAIT>
void ObsOnTheFly<TRAIT>::setupAD() {}
// -----------------------------------------------------------------------------
}  // namespace oops
