/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_OBSERVATIONS_OBSONTHEFLY_H_
#define OOPS_OBSERVATIONS_OBSONTHEFLY_H_

#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include "logger/Logger.h"
#include "oops/observations/ObsOperStrategy.h"
#include "util/Config.h"
#include "util/DateTime.h"

using util::DateTime;

namespace oops {

/// ObsOperator for observations where ObsEquiv is called during the time loop.

// -----------------------------------------------------------------------------
template <typename TRAIT>
class ObsOnTheFly: public ObsOperStrategy<TRAIT>, private boost::noncopyable {
  typedef typename TRAIT::ObsOperator      ObsEqv;
  typedef typename TRAIT::ObsVector        ObsVector;
  typedef typename TRAIT::ObsAuxCtrl       ObsAuxCtrl;
  typedef typename TRAIT::ObsAuxIncr       ObsAuxIncr;
  typedef typename TRAIT::ObsTraj          ObsTraj;
  typedef typename TRAIT::Locations        Locations;
  typedef typename TRAIT::LocalModelValues LocalModelValues;
  typedef typename TRAIT::StateBase        State;
  typedef typename TRAIT::IncrementBase    Increment;

 public:
  explicit ObsOnTheFly(const ObsEqv &);
  ~ObsOnTheFly();

// NL
  void setup(const DateTime &, const DateTime &);
  void obsOperator(const State &, ObsVector &, const ObsAuxCtrl &,
                   const DateTime &, const DateTime &);
  void final(ObsVector &, const ObsAuxCtrl &);

// Trajectory
  void obsOperator(const State &, ObsVector &, const ObsAuxCtrl &,
                   const DateTime &, const DateTime &, ObsTraj &);
  void final(ObsVector &, const ObsAuxCtrl &, ObsTraj &);

// TL
  void setupTL(const DateTime &, const DateTime &);
  void obsOperatorTL(const Increment &, ObsVector &, const ObsAuxIncr &,
                     const DateTime &, const DateTime &, const ObsTraj &);
  void finalTL(ObsVector &, const ObsAuxIncr &, const ObsTraj &);

// AD
  void finalAD(const DateTime &, const DateTime &, const ObsVector &,
               ObsAuxIncr &, const ObsTraj &);
  void obsOperatorAD(Increment &, const ObsVector &, ObsAuxIncr &,
                     const DateTime &, const DateTime &, const ObsTraj &);
  void setupAD();

 private:
  const ObsEqv * equiv_;
};
// -----------------------------------------------------------------------------
}  // namespace oops

#include "oops/observations/ObsOnTheFly.cc"

#endif  // OOPS_OBSERVATIONS_OBSONTHEFLY_H_
