/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include <map>
#include <string>

#include "logger/Logger.h"
#include "util/abor1_cpp.h"
#include "util/Config.h"

namespace oops {

// -----------------------------------------------------------------------------

template <typename TRAIT>
ObsOpFactory<TRAIT>::ObsOpFactory(const std::string & name) {
  if (getMakers().find(name) != getMakers().end()) {
    LOG(Error) << name << " already registered in operator strategy factory.";
    ABORT("Element already registered in ObsOpFactory.");
  }
  getMakers()[name] = this;
}

// -----------------------------------------------------------------------------

template <typename TRAIT>
ObsOperStrategy<TRAIT>* ObsOpFactory<TRAIT>::create(const std::string & id,
                                                    ObsOper & oop) {
  typename std::map<string, ObsOpFactory<TRAIT>*>::iterator j = getMakers().find(id);
  if (j == getMakers().end()) {
    LOG(Error) << id << " does not exist in operator strategy factory.";
    ABORT("Element does not exist in ObsOpFactory.");
  }
  return (*j).second->make(oop);
}

// -----------------------------------------------------------------------------
}  // namespace oops
