/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_OBSERVATIONS_OBSTYPE_H_
#define OOPS_OBSERVATIONS_OBSTYPE_H_

#include <iostream>
#include <string>

#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include "oops/observations/ObsOperStrategy.h"
#include "util/Config.h"

// Forward declarations
namespace util {
  class DateTime;
}

namespace oops {

template<typename TRAIT> class ObsType;

template<typename TRAIT>
std::ostream & operator<< (std::ostream &, const ObsType<TRAIT> &);

/// Genric handling of ObsType.
/*!
 *  ObsType defines a generic observation operator.
 */

// -----------------------------------------------------------------------------
template<typename TRAIT> class ObsType {
  typedef typename TRAIT::StateBase    State;
  typedef typename TRAIT::ObsVector    ObsVect;
  typedef typename TRAIT::ObsAuxCtrl   ObsAuxCtrl;
  typedef typename TRAIT::ObsAuxIncr   ObsAuxIncr;
  typedef typename TRAIT::ObsTraj      ObsTraj;
  typedef typename TRAIT::ObsOperator  ObsOperator;  // Can be abstract

 public:
  static bool compatible(const ObsType &, const ObsType &);

  explicit ObsType(const util::Config &);
  ObsType(const ObsType &, const bool copyObs = true);
  ~ObsType();
  ObsType & operator=(const ObsType &);

/// NL operators
  void setup(const util::DateTime &, const util::DateTime &);
  void obsOperator(const State &, const ObsAuxCtrl &,
                   const util::DateTime &, const util::DateTime &);
  void finalize(const ObsAuxCtrl &);

/// NL operators (traj setup)
  void obsOperator(const State &, const ObsAuxCtrl &,
                   const util::DateTime &, const util::DateTime &, ObsTraj &);
  void finalize(const ObsAuxCtrl &, ObsTraj &);

/// Generate observation distribution
  void generate(const util::DateTime &, const util::DateTime &);

/// Add perturbation (Departure)
  void add_pert(const ObsVect &);

// Utilities
  void read(const std::string &);
  void write(const std::string &) const;
  friend std::ostream & operator<< <> (std::ostream &, const ObsType &);

  const std::string obsname() const {return equiv_->obsname();}
  ObsTraj * getTraj() const {return equiv_->getTraj();}
  boost::shared_ptr< ObsOperator > getEqv() const {return equiv_;}
  boost::shared_ptr< ObsOperStrategy<TRAIT> > getHop() const {return oper_;}
  const ObsVect & getVal() const {return *obsval_;}
//  const util::Config & config() const {return obsConfig_;}

 private:
  boost::shared_ptr< ObsOperator > equiv_;
  boost::shared_ptr< ObsOperStrategy<TRAIT> > oper_;
  boost::scoped_ptr< ObsVect > obsval_;
  const util::Config obsConfig_;
};
// -----------------------------------------------------------------------------

}  // namespace oops

#include "oops/observations/ObsType.cc"

#endif  // OOPS_OBSERVATIONS_OBSTYPE_H_
