/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_RUNS_FORECAST_H_
#define OOPS_RUNS_FORECAST_H_

#include <string>

#include "logger/Logger.h"
#include "oops/base/Geometry.h"
#include "oops/base/PostProcessor.h"
#include "oops/base/ModelState.h"
#include "oops/base/StateInfo.h"
#include "oops/base/StateWriter.h"
#include "oops/runs/Task.h"
#include "util/Config.h"
#include "util/DateTime.h"
#include "util/abor1_cpp.h"

namespace oops {

template <typename TRAIT> class Forecast: public Task {
  typedef typename TRAIT::ModelAuxCtrl  ModErr;
  typedef typename TRAIT::ModelData     Model;
  typedef typename TRAIT::StateBase     StateBase;

 public:
// -----------------------------------------------------------------------------
  Forecast(int argc, char ** argv): Task(argc, argv) {}
// -----------------------------------------------------------------------------
  ~Forecast() {}
// -----------------------------------------------------------------------------
  void run(const util::Config & fullConfig) {
//  Setup time
    util::Config fcConfig(fullConfig, "forecast");
    util::DateTime bgndate(fcConfig.getData("begin"));
    util::DateTime enddate(fcConfig.getData("end"));
    LOG(Info) << "Running forecast from " << bgndate << " to " << enddate;

//  Setup resolution
    const util::Config resolConfig(fullConfig, "resolution");
    const Geometry resol(resolConfig);

//  Setup Model
    const util::Config modelConfig(fullConfig, "model");
    const Model model(resol, modelConfig);

//  Setup initial state
    const util::Config initialConfig(fullConfig, "initial");
    LOG(Configs) << "Initial configuration is:" << "\n" << initialConfig;
    ModelState<TRAIT> xx(model, initialConfig);
    xx.prtNorms("Read");

//  Setup augmented state
    ModErr moderr(initialConfig);

//  Setup forecast outputs
    PostProcessor<StateBase> post;

    const util::Config prtConfig(fullConfig, "prints", true);
    post.enroll(new StateInfo<StateBase>("fc", prtConfig));

    const util::Config outConfig(fullConfig, "output");
    post.enroll(new StateWriter<StateBase>(bgndate, outConfig));

//  Run forecast
    xx.forecast(moderr, enddate, post);
  }
// -----------------------------------------------------------------------------
};

}  // namespace oops
#endif  // OOPS_RUNS_FORECAST_H_
