/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "oops/runs/Task.h"

#include <string>

#include <boost/scoped_ptr.hpp>

#include "logger/Logger.h"
#include "logger/setLogging.h"
#include "util/Config.h"
#include "util/XmlDom.h"
#include "util/abor1_cpp.h"

extern "C" {
  void c_logger_init_();
  void set_err_trap_();
}

namespace oops {

// -----------------------------------------------------------------------------

Task::Task(int argc, char ** argv) {
  set_err_trap_();

// Get configuration file from command line
  if (argc != 2) {
    ABORT("Expected argc==2. Too many/few command line arguments.");
  }
  std::string configfile = argv[1];

// Read configuration
  config_.reset(new util::Config(configfile, new util::XmlDom()));
  config_->setCurrentNode("config");

  const util::Config logConfig(*config_, "logging");
  logger::setLogging(logConfig);
  c_logger_init_();  // initialise Fortran interface to Logger
  LOG(Info) << "Configuration input file is: " << configfile;
  LOG(Trace) << "Task constructed";

  LOG(Info) << "Full configuration is:" << "\n" << *config_;
}

// -----------------------------------------------------------------------------

Task::~Task() {
  LOG(Trace) << "Destructing Task";
}

// -----------------------------------------------------------------------------

void Task::execute() {
  run(*config_);
}

// -----------------------------------------------------------------------------
}
