/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef OOPS_RUNS_TEST_H_
#define OOPS_RUNS_TEST_H_

#include<string>
#include<vector>

#if defined(_AIX) || defined(__AIX) || defined(__AIX__) || defined(__aix) || defined(__aix__)
# define PLATFORM_ID_AIX
#endif

#define BOOST_TEST_NO_MAIN
#define BOOST_TEST_ALTERNATIVE_INIT_API

#ifndef PLATFORM_ID_AIX
#define BOOST_TEST_DYN_LINK
#include <boost/test/unit_test.hpp>
#else
#include <boost/test/included/unit_test.hpp>
#endif

#include<boost/tokenizer.hpp>

#include "logger/Logger.h"
#include "oops/observations/instantiateObsErrorFactory.h"
#include "oops/observations/instantiateObsOpFactory.h"
#include "oops/runs/Task.h"
#include "test/TestEnvironment.h"
#include "test/base/ModelState.h"
#include "test/base/Increment.h"
#include "util/Config.h"

namespace oops {

template <typename TRAIT> class Test: public Task {
 public:
// -----------------------------------------------------------------------------
  Test(int argc, char ** argv): Task(argc, argv) {
//  Instantiate Factories
    instantiateObsErrorFactory<TRAIT>();
    instantiateObsOpFactory<TRAIT>();
  }
// -----------------------------------------------------------------------------
  ~Test() {}
// -----------------------------------------------------------------------------
  void run(const util::Config & config) {
// Setup test environment
    test::TestEnvironment<TRAIT>::getInstance().testSetup(config);

// Extract the runtime config for the tests from the config file.
    std::string args = config.getData("test_framework_runtime_config");

// Create a string version of argv
    std::vector<std::string> argvec;
    argvec.push_back(std::string("abcd"));

    typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
    boost::char_separator<char> sep(" \n\t");
    tokenizer tok(args, sep);
    for (tokenizer::iterator it = tok.begin(); it != tok.end(); ++it) {
      argvec.push_back(*it);
    }

// Generate the argc and argv arguments for unit_test_main(...)
    int argc = argvec.size();
    char * argv[argc];
    for (int i = 0; i < argc; ++i) {
      argv[i] = new char[argvec[i].size()+1];
      strcpy(argv[i], argvec[i].c_str());
    }

// Run the tests
    LOG(Trace) << "Running the unit tests";
    boost::unit_test::unit_test_main(&init_unit_test, argc, argv);
    LOG(Trace) << "Finished running the unit tests";

// Tidy up
    for (int i = 0; i < argc; ++i) {
      delete [] argv[i];
    }
  }
// -----------------------------------------------------------------------------
 private:
// -----------------------------------------------------------------------------
// Tests for templated objects (such as State) are manually registered here.
  static bool init_unit_test() {
    test::initTestSuiteState<TRAIT>();
    test::initTestSuiteIncrement<TRAIT>();
    return true;
  }
};

}  // namespace oops
#endif  // OOPS_RUNS_TEST_H_
