/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef TEST_TESTENVIRONMENT_H_
#define TEST_TESTENVIRONMENT_H_

#include <boost/noncopyable.hpp>

#include "util/Config.h"

namespace test {

/// TestEnvironment is a singleton that defines the unit testing enviroment
/*! TestEnvironment contains globally available information for the unit
 *  tests. It is needed because there is no easy method to pass configuration
 *  and other data to the tests. By defining a singleton, each test can
 *  use TestEnvironment::getInstance() to get access to the global data.
 */

template <typename TRAIT> class TestEnvironment : private boost::noncopyable {
 public:
  static TestEnvironment<TRAIT> & getInstance() {
    static TestEnvironment<TRAIT> theTestEnvironment;
    return theTestEnvironment;
  }

  void testSetup(const util::Config & conf) {
    config_ = new util::Config(conf);
  }

  const util::Config *     config() const       {return config_;}

 private:
  TestEnvironment() {}  // ctor hidden

  ~TestEnvironment() {
    delete config_;
  }  // dtor hidden

  const util::Config * config_;
};

}  // namespace test

#endif  // TEST_TESTENVIRONMENT_H_
