/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef TEST_BASE_MODELSTATE_H_
#define TEST_BASE_MODELSTATE_H_

#include <iostream>
#include <string>

#include <boost/test/unit_test.hpp>

#include "oops/base/Geometry.h"
#include "oops/base/ModelState.h"
#include "test/TestEnvironment.h"
#include "util/Config.h"
#include "util/DateTime.h"

namespace test {

// -----------------------------------------------------------------------------

template <typename TRAIT> void testStateConstructor() {
  const util::Config * config_ = TestEnvironment<TRAIT>::getInstance().config();
  const util::Config resolConfig(*config_, "resolution");
  const oops::Geometry resol(resolConfig);
  const util::Config modelConfig(*config_, "model");
  const typename TRAIT::ModelData model(resol, modelConfig);
  const util::Config bgConfig(*config_, "state");
  oops::ModelState<TRAIT> * xx = new oops::ModelState<TRAIT>(model, bgConfig);
  BOOST_CHECK(xx);
  delete xx;
}

// -----------------------------------------------------------------------------

template <typename TRAIT> void testStateCopyConstructor() {
  const util::Config * config_ = TestEnvironment<TRAIT>::getInstance().config();

// Setup a state
  const util::Config resolConfig(*config_, "resolution");
  const oops::Geometry resol(resolConfig);
  const util::Config modelConfig(*config_, "model");
  const typename TRAIT::ModelData model(resol, modelConfig);
  const util::Config bgConfig(*config_, "state");
  oops::ModelState<TRAIT> * xx = new oops::ModelState<TRAIT>(model, bgConfig);
  BOOST_CHECK(xx);

// Setup another state
  oops::ModelState<TRAIT> * zz = new oops::ModelState<TRAIT>(*xx);
  BOOST_CHECK(zz);

  delete xx;
  delete zz;
}

// -----------------------------------------------------------------------------

template <typename TRAIT> void initTestSuiteState() {
  boost::unit_test::test_suite * ts = BOOST_TEST_SUITE("test_state");
  ts->add(BOOST_TEST_CASE(&testStateConstructor<TRAIT>));
  ts->add(BOOST_TEST_CASE(&testStateCopyConstructor<TRAIT>));
  boost::unit_test::framework::master_test_suite().add(ts);
}

// -----------------------------------------------------------------------------

}  // namespace test

#endif  // TEST_BASE_MODELSTATE_H_
