/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "util/Config.h"

#include<string>

#include "util/ConfigImplBase.h"

namespace util {

std::ostream& operator<<(std::ostream& output, const Config& c) {
  c.print(output);
  return output;
}

// -----------------------------------------------------------------------------

Config::Config(const std::string & filename, ConfigImplBase * const p)
    : pimpl_(p) {
  pimpl_->readFile(filename);
}

// -----------------------------------------------------------------------------

Config::Config(const util::Config& d) : pimpl_(d.pimpl_->copy()) {
}

// -----------------------------------------------------------------------------

Config::Config(const util::Config& d, const std::string & s, const bool allowEmpty)
            : pimpl_(d.pimpl_->copySubTree(s, allowEmpty)) {
}

// -----------------------------------------------------------------------------

Config::Config(const util::Config& d, const std::string & s, const unsigned idx)
            : pimpl_(d.pimpl_->copySubTree(s, false, idx)) {
}

// -----------------------------------------------------------------------------

Config::Config(ConfigImplBase * const p)
    : pimpl_(p) {
  if (!pimpl_->empty()) {
    pimpl_.reset(pimpl_->makeEmpty());
  }
}

// -----------------------------------------------------------------------------

Config::Config(const util::Config& d, const util::ConfigEmpty)
    : pimpl_(d.pimpl_->makeEmpty()) {
}

// -----------------------------------------------------------------------------

void Config::setCurrentNode(const std::string & str) {
  pimpl_->setCurrentNode(str);
}

// -----------------------------------------------------------------------------

bool Config::empty() const {
  return pimpl_ ->empty();
}

// -----------------------------------------------------------------------------

bool Config::elementExists(const std::string & s) const {
  return pimpl_->elementExists(s);
}

// -----------------------------------------------------------------------------

std::string Config::getData(const std::string & s) const {
  return pimpl_->getData(s);
}

// -----------------------------------------------------------------------------

int Config::getInt(const std::string & s) const {
  return pimpl_->getInt(s);
}

// -----------------------------------------------------------------------------

double Config::getReal(const std::string & s) const {
  return pimpl_->getReal(s);
}

// -----------------------------------------------------------------------------

void Config::addElement(const std::string & path,
                        const std::string & element) {
  pimpl_->addElement(path, element);
}

// -----------------------------------------------------------------------------

void Config::setElement(const std::string & path,
                        const std::string & contents) {
  pimpl_->setElement(path, contents);
}

// -----------------------------------------------------------------------------

void Config::setElement(const std::string & path, const int i) {
  pimpl_->setElement(path, i);
}

// -----------------------------------------------------------------------------

void Config::setElement(const std::string & path, const double d) {
  pimpl_->setElement(path, d);
}

// -----------------------------------------------------------------------------

unsigned Config::getElementSize(const std::string & path) const {
  return pimpl_->getElementSize(path);
}

// -----------------------------------------------------------------------------

void Config::print(std::ostream& output) const {  // NOLINT(runtime/references)
  pimpl_->print(output);
}

}  // namespace util
