/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef UTIL_CONFIGIMPLBASE_H_
#define UTIL_CONFIGIMPLBASE_H_

#include<ostream>
#include<string>

namespace util {

  /// Configuration
  /*!
   *  A ConfigImplBase represents tree-structured configuration data
   *  that has an Xpath-like syntax for identifying sub-trees and
   *  single elements.
   */

  class ConfigImplBase {
    friend std::ostream& operator<<(std::ostream& output,
                                    const ConfigImplBase& d);

   public:
  // Destructor
    virtual ~ConfigImplBase() {}

  // Methods
    /// Make an empty ConfigImplBase
    virtual ConfigImplBase * makeEmpty() const =0;

    /// Copy method
    virtual ConfigImplBase * copy() const =0;

    /// Copy from sub-tree
    virtual ConfigImplBase * copySubTree(const std::string &, const bool,
                                         const unsigned idx = 0) const =0;

    /// Read configuration from a file (discarding former contents)
    virtual void readFile(const std::string & filename) =0;

    /// Set the current node for Xpath relative addressing
    virtual void setCurrentNode(const std::string &) =0;

    /// Return true if the ConfigImplBase is empty
    virtual bool empty() const =0;

    /// Return true if there is an element satisfying the query string.
    virtual bool elementExists(const std::string &) const =0;

    /// Get the data from the first element that satisfies the query string.
    virtual std::string getData(const std::string &) const =0;

    /// Get the data from the first element that satisfies the query string.
    virtual int getInt(const std::string &) const =0;

    /// Get the data from the first element that satisfies the query string.
    virtual double getReal(const std::string &) const =0;

    /// Add a new Element to the tree
    virtual void addElement(const std::string &, const std::string &) =0;

    /// Set/reset the contents of an Element
    virtual void setElement(const std::string &, const std::string &) =0;

    /// Set/reset the contents of an Element
    virtual void setElement(const std::string &, const int) =0;

    /// Set/reset the contents of an Element
    virtual void setElement(const std::string &, const double) =0;

    virtual unsigned getElementSize(const std::string &) const =0;
    /// Print
    virtual void print(std::ostream&) const =0;

   protected:
    /// Construct an empty ConfigImplBase
    ConfigImplBase() {}

   private:
    explicit ConfigImplBase(const util::ConfigImplBase &);  // no copy
    ConfigImplBase & operator=(const ConfigImplBase &);  // no assignment
  };

}  // namespace oops
#endif  // UTIL_CONFIGIMPLBASE_H_
