/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef UTIL_DIAGONALMATRIX_H_
#define UTIL_DIAGONALMATRIX_H_

#include <boost/noncopyable.hpp>

namespace oops {

// -----------------------------------------------------------------------------
/// Diagonal matrix.
template <typename VECTOR> class DiagonalMatrix : private boost::noncopyable {
 public:
  explicit DiagonalMatrix(const VECTOR & diag): diag_(diag) {}
  ~DiagonalMatrix() {}

  void mult(const VECTOR & v1, VECTOR & v2) const {
    v2 = v1;
    v2 *= diag_;
  }

  void inv_mult(const VECTOR & v1, VECTOR & v2) const {
    v2 = v1;
    v2 /= diag_;
  }

 private:
  const VECTOR diag_;
};
// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // UTIL_DIAGONALMATRIX_H_
