/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef UTIL_POSTTIMER_H_
#define UTIL_POSTTIMER_H_

#include <vector>

#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>

#include "util/DateTime.h"
#include "util/Duration.h"

// Forward declarations
namespace util {
  class Config;
}

namespace oops {

// -----------------------------------------------------------------------------

/// Handles timing of post-processing and similar actions
/*!
 *  By default processing is performed on every call.
 */

class PostTimer : private boost::noncopyable {
 public:
  PostTimer();
  explicit PostTimer(const util::Duration &);
  explicit PostTimer(const util::Config &);
  PostTimer(const util::DateTime &, const util::Config &);
  PostTimer(const util::DateTime &, const util::DateTime &, const util::Duration &);
  ~PostTimer() {}

  void initialize(const util::DateTime &, const util::DateTime &,
                  const util::Duration &);
  bool itIsTime(const util::DateTime &);

 private:
  const util::Config * conf_;
  util::Duration freq_;
  util::DateTime bgn_;
  util::DateTime end_;
  boost::scoped_ptr<util::DateTime> start_;
  boost::scoped_ptr<util::DateTime> finish_;
  std::vector<util::DateTime> pptimes_;
};

// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // UTIL_POSTTIMER_H_
