/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "util/WeightingFct.h"

#include <map>
#include <string>

#include "logger/Logger.h"
#include "util/abor1_cpp.h"
#include "util/Config.h"

namespace oops {

// -----------------------------------------------------------------------------
//  Factory
// -----------------------------------------------------------------------------

// We have to use a pointer here because we don't know the order in
// which objects are created and this causes problems with xlc.

std::map < std::string, WeightFactory * > * WeightFactory::makers_ = 0;

// -----------------------------------------------------------------------------

WeightFactory::WeightFactory(const std::string & name) {
  if (!makers_) makers_=new std::map < std::string, WeightFactory * >();

  if (makers_->find(name) != makers_->end()) {
    LOG(Error) << name << " already registered in weight function factory.";
    ABORT("Element already registered in WeightFactory.");
  }
  (*makers_)[name] = this;
}

// -----------------------------------------------------------------------------

WeightingFct * WeightFactory::create(const util::Config & config) {
  if (!makers_) makers_=new std::map < std::string, WeightFactory * >();

  std::string id = config.getData("type");
  std::map<std::string, WeightFactory *>::iterator j = makers_->find(id);
  if (j == makers_->end()) {
    LOG(Error) << id << " does not exist in weight function factory.";
    ABORT("Element does not exist in WeightFactory.");
  }
  return (*j).second->make(config);
}

// -----------------------------------------------------------------------------

}  // namespace oops
