/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef UTIL_WEIGHTINGFCT_H_
#define UTIL_WEIGHTINGFCT_H_

#include <map>
#include <string>

#include "util/DateTime.h"

namespace util {
  class Config;
  class Duration;
}

namespace oops {

// -----------------------------------------------------------------------------

/// Weighting Function
/*!
 * Abstract base class for weighting functions for various filters.
 */

class WeightingFct {
 public:
  virtual ~WeightingFct() {}

  virtual std::map< util::DateTime, double > setWeights(const util::DateTime &,
                                                        const util::DateTime &,
                                                        const util::Duration &) =0;
};

// -----------------------------------------------------------------------------

/// Factory
class WeightFactory {
 public:
  static WeightingFct * create(const util::Config &);
  virtual ~WeightFactory() { makers_->clear(); }

 protected:
  explicit WeightFactory(const std::string &);
 private:
  virtual WeightingFct * make(const util::Config &) =0;
  static std::map < std::string, WeightFactory * > * makers_;
};

template<class FCT>
class WeightMaker : public WeightFactory {
  virtual WeightingFct * make(const util::Config & config)
    {return new FCT(config);}
 public:
  explicit WeightMaker(const std::string & name) : WeightFactory(name) {}
};

// -----------------------------------------------------------------------------

}  // namespace oops

#endif  // UTIL_WEIGHTINGFCT_H_
