/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#ifndef UTIL_ASSER1_H_
#define UTIL_ASSER1_H_

#include<string>

namespace util {
/// ASSERT macro.
/*!
 * Usage:
 *          ASSERT(expression);
 *
 * If the cpp variable CHECK_ASSERTS is defined, ASSERT will abort if
 * "expression" evaluates to false. The abort prints the expression together
 * with the name of the file in which the abort occurred, and the line number.
 *
 * NB: the macro emplys a few tricks to ensure that it does what is expected:
 *  - Wrapping x in parentheses ensures ASSERT(true||false) succeeds.
 *  - do...while(0) makes ASSERT behave properly if it is followed by "else".
 *  - "sizeof" is a trick to ensure that no code is generated if CHECK_ASSERTS
 *    is not defined. (According to the standard, the argument of sizeof is not
 *    evaluated.)
 */

#ifdef CHECK_ASSERTS
  #define ASSERT(x) \
    do { if (!(x)) { util::asser1(#x, __FILE__, __LINE__); } } while (0)
#else
  #define ASSERT(x) \
    do { (void)sizeof(x); } while (0)
#endif

void asser1(const std::string & cderror,
            const std::string & file,
            const int line);

}  // namespace util
#endif  // UTIL_ASSER1_H_
