!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!>  Fortran interface to Config.

module c_config_mod

use ISO_C_BINDING, only : C_PTR, C_INT, C_CHAR
use ftn_c_string_conversion
use kinds

implicit none

private
public c_config, c_wrap_config, &
     & c_config_set_current_node, &
     & c_config_empty, c_config_element_exists, &
     & c_config_get_int, c_config_get_real, &
     & c_config_get_data_length, c_config_get_data

!>  c_config encapsulates an Config pointer.
type c_config
  private
  type(C_PTR) dom_ptr
end type c_config

!> Fortran generic to check if a utils::Config is empty
interface c_config_empty
  module procedure c_config_empty_f, c_config_empty_c
end interface c_config_empty

!> Fortran generic to check if an element exists from utils::Config or a c_config_mod::c_config
interface c_config_element_exists
  module procedure c_config_element_exists_f, c_config_element_exists_c
end interface c_config_element_exists

!> Fortran generic to read integers from a utils::Config or a c_config_mod::c_config
interface c_config_get_int
  module procedure c_config_get_int_f, c_config_get_int_c
end interface c_config_get_int

!> Fortran generic to read reals from a utils::Config or a c_config_mod::c_config
interface c_config_get_real
  module procedure c_config_get_real_f, c_config_get_real_c
end interface c_config_get_real

!> Fortran generic to get the length of the data in a utils::Config or a c_config_mod::c_config
interface c_config_get_data_length
  module procedure c_config_get_len_f, c_config_get_len_c
end interface c_config_get_data_length

!> Fortran generic to read a string from a utils::Config or a c_config_mod::c_config
interface c_config_get_data
  module procedure c_config_get_data_f, c_config_get_data_c
end interface c_config_get_data

  interface
    integer(kind=C_INT) function ftn_config_get_data_length (dom,str) &
                  & bind(C,name='ftn_config_get_data_length')
      use ISO_C_BINDING
      implicit none
      type(C_PTR), value :: dom
      character(kind=C_CHAR, len=1), intent(in) :: str(*)
    end function ftn_config_get_data_length
  end interface

!-------------------------------------------------------------------------------

contains

!>  Wrap a raw c pointer to a Config structure into a c_config derived type.

type(c_config) function c_wrap_config (c_dom)
  type(c_ptr), intent(in) :: c_dom
  c_wrap_config%dom_ptr = c_dom
end function c_wrap_config

!-------------------------------------------------------------------------------

!>  Set the current XPath node

subroutine c_config_set_current_node (dom,xpath)
  implicit none
  type(c_config), intent(in) :: dom
  character(len=*), intent(in) :: xpath

  character(kind=C_CHAR,len=1), allocatable :: c_xpath(:)

  interface
    subroutine ftn_config_set_current_node (dd,pth) bind(C,name='ftn_config_set_current_node')
      use ISO_C_BINDING
      implicit none
      type(C_PTR), value :: dd
      character(kind=C_CHAR, len=1), intent(in) :: pth(*)
    end subroutine ftn_config_set_current_node
  end interface

    !  Translate filename from Fortran string to C++ char[].
  call alloc_and_copy_ftn_to_c_string(xpath,c_xpath)

  call ftn_config_set_current_node (dom%dom_ptr,c_xpath)
end subroutine c_config_set_current_node

!-------------------------------------------------------------------------------

!> Check if empty

logical function c_config_empty_f (dom)
  implicit none
  type(c_config), intent(in) :: dom
  c_config_empty_f = c_config_empty_c(dom%dom_ptr)
end function c_config_empty_f

!-------------------------------------------------------------------------------

!> Check if empty

logical function c_config_empty_c(c_dom)
  implicit none
  type(c_ptr), intent(in) :: c_dom

  interface
    logical(kind=C_BOOL) function ftn_config_empty (dom) &
                  & bind(C,name='ftn_config_empty')
      use ISO_C_BINDING
      implicit none
      type(C_PTR), value :: dom
    end function ftn_config_empty
  end interface

  ! Call C++ to process the query
  c_config_empty_c = LOGICAL(ftn_config_empty(c_dom))
end function c_config_empty_c

!-------------------------------------------------------------------------------

!> Check if an element exists

logical function c_config_element_exists_f (dom,query)
  implicit none
  type(c_config), intent(in) :: dom
  character(len=*), intent(in) :: query

  c_config_element_exists_f = c_config_element_exists_c (dom%dom_ptr,query)
end function c_config_element_exists_f

!-------------------------------------------------------------------------------

!> Check if an element exists

logical function c_config_element_exists_c (c_dom,query)
  implicit none
  type(c_ptr), intent(in) :: c_dom
  character(len=*), intent(in) :: query

  character(kind=C_CHAR,len=1), allocatable :: c_query(:)

  interface
    logical(kind=C_BOOL) function ftn_config_element_exists (dom,str) &
                  & bind(C,name='ftn_config_element_exists')
      use ISO_C_BINDING
      implicit none
      type(C_PTR), value :: dom
      character(kind=C_CHAR, len=1), intent(in) :: str(*)
    end function ftn_config_element_exists
  end interface

  !  Translate query from Fortran string to C++ char[].
  call alloc_and_copy_ftn_to_c_string(query,c_query)

  ! Call C++ to process the query
  c_config_element_exists_c = LOGICAL(ftn_config_element_exists(c_dom,c_query))
  deallocate(c_query)
end function c_config_element_exists_c

!-------------------------------------------------------------------------------

!>  Return data from a Config element as a integer.

integer function c_config_get_int_f (dom,query)
  implicit none
  type(c_config), intent(in) :: dom
  character(len=*), intent(in) :: query

  c_config_get_int_f = c_config_get_int_c (dom%dom_ptr,query)
end function c_config_get_int_f

!-------------------------------------------------------------------------------

!>  Return data from a Config element as a integer.

integer function c_config_get_int_c (c_dom,query)
  implicit none
  type(c_ptr), intent(in) :: c_dom
  character(len=*), intent(in) :: query

  character(kind=C_CHAR,len=1), allocatable :: c_query(:)

  interface
    integer(kind=C_INT) function ftn_config_get_data_as_int (dom,str) &
                  & bind(C,name='ftn_config_get_data_as_int')
      use ISO_C_BINDING
      implicit none
      type(C_PTR), value :: dom
      character(kind=C_CHAR, len=1), intent(in) :: str(*)
    end function ftn_config_get_data_as_int
  end interface

  !  Translate query from Fortran string to C++ char[].
  call alloc_and_copy_ftn_to_c_string(query,c_query)

  ! Call C++ to process the query
  c_config_get_int_c= ftn_config_get_data_as_int(c_dom,c_query)
  deallocate(c_query)
end function c_config_get_int_c

!-------------------------------------------------------------------------------

!>  Return data from a Config element as a real.

real(kind=kind_real) function c_config_get_real_f (dom,query)
  implicit none
  type(c_config), intent(in) :: dom
  character(len=*), intent(in) :: query

  c_config_get_real_f = c_config_get_real_c (dom%dom_ptr,query)
end function c_config_get_real_f

!-------------------------------------------------------------------------------

!>  Return data from a Config element as a real.

real(kind=kind_real) function c_config_get_real_c (c_dom,query)
  implicit none
  type(c_ptr), intent(in) :: c_dom
  character(len=*), intent(in) :: query

  character(kind=C_CHAR,len=1), allocatable :: c_query(:)

  interface
    real(kind=C_DOUBLE) function ftn_config_get_data_as_double (dom,str) &
                  & bind(C,name='ftn_config_get_data_as_double')
      use ISO_C_BINDING
      implicit none
      type(C_PTR), value :: dom
      character(kind=C_CHAR, len=1), intent(in) :: str(*)
    end function ftn_config_get_data_as_double
  end interface

  !  Translate query from Fortran string to C++ char[].
  call alloc_and_copy_ftn_to_c_string(query,c_query)

  ! Call C++ to process the query
  c_config_get_real_c= ftn_config_get_data_as_double(c_dom,c_query)
  deallocate(c_query)
end function c_config_get_real_c

!-------------------------------------------------------------------------------

!>  Return the number of chars of data in a Config element

integer function c_config_get_len_f (dom,query)
  implicit none
  type(c_config), intent(in) :: dom
  character(len=*), intent(in) :: query

  c_config_get_len_f = c_config_get_len_c (dom%dom_ptr,query)
end function c_config_get_len_f

!-------------------------------------------------------------------------------

!>  Return the number of chars of data in a Config element

integer function c_config_get_len_c (c_dom,query)
  implicit none
  type(c_ptr), intent(in) :: c_dom
  character(len=*), intent(in) :: query

  character(kind=C_CHAR,len=1), allocatable :: c_query(:)

  !  Translate query from Fortran string to C++ char[].
  call alloc_and_copy_ftn_to_c_string(query,c_query)

  ! Call C++ to process the query
  c_config_get_len_c= ftn_config_get_data_length(c_dom,c_query)
  deallocate(c_query)
end function c_config_get_len_c

!-------------------------------------------------------------------------------

!>  Return the contents of a Config element as a string

function c_config_get_data_f (dom,length,query)
  implicit none
  type(c_config), intent(in) :: dom
  integer, intent(in) :: length
  character(len=*), intent(in) :: query
  character(len=length) ::  c_config_get_data_f

  c_config_get_data_f = c_config_get_data_c (dom%dom_ptr,length,query)
end function c_config_get_data_f

!-------------------------------------------------------------------------------

!>  Return the contents of a Config element as a string

function c_config_get_data_c (c_dom,length,query)
  implicit none
  type(c_ptr), intent(in) :: c_dom
  integer, intent(in) :: length
  character(len=*), intent(in) :: query
  character(len=length) ::  c_config_get_data_c

  character(kind=C_CHAR,len=1), allocatable :: c_query(:), c_data(:)
  integer nchars

  interface
    subroutine ftn_config_get_data (dom,str,output) bind(C,name='ftn_config_get_data')
      use ISO_C_BINDING
      implicit none
      type(C_PTR), value :: dom
      character(kind=C_CHAR, len=1), intent(in)  :: str(*)
      character(kind=C_CHAR, len=1), intent(out) :: output(*)
    end subroutine ftn_config_get_data
  end interface

  !  Translate query from Fortran string to C++ char[].
  call alloc_and_copy_ftn_to_c_string(query,c_query)

  ! Call C++ to process the query
  nchars = ftn_config_get_data_length(c_dom,c_query)
  if (nchars > length) &
    call abor1_ftn ('c_config_get_data: return argument too short')

  if (nchars>0) then
    allocate(c_data(length_convert_ftn_to_c_string(nchars)))
    call ftn_config_get_data (c_dom,c_query,c_data)
    c_config_get_data_c = c_to_ftn_string(c_data)
  else
    call abor1_ftn ('c_config_get_data: element not found')
    c_config_get_data_c = ""
  endif

  deallocate(c_query)
end function c_config_get_data_c

!-------------------------------------------------------------------------------

end module c_config_mod
