/*
* Copyright 2011 ECMWF
* 
* This software was developed at ECMWF for evaluation
* and may be used for academic and research purposes only.
* The software is provided as is without any warranty.
* 
* This software can be used, copied and modified but not
* redistributed or sold. This notice must be reproduced
* on each copy made.
*/

#include "util/datetime_f.h"

#include <string.h>

#include <string>

#include "util/Duration.h"

namespace oops {

// -----------------------------------------------------------------------------

util::DateTime * datetime_construct_f(const char str[]) {
  std::string s(str);
  return new util::DateTime(s);
}

// -----------------------------------------------------------------------------

util::DateTime * datetime_clone_f(const util::DateTime * d) {
  return new util::DateTime(*d);
}

// -----------------------------------------------------------------------------

void datetime_destruct_f(const util::DateTime * d) {
  delete d;
}

// -----------------------------------------------------------------------------

void datetime_string_f(const util::DateTime * d, char * cstr) {
  const std::string ss(d->toString());
// Can't think of any way to check cstr points to valid space in memory...
  strcpy(cstr,  ss.c_str());
}

// -----------------------------------------------------------------------------

// Using double to avoid passing bool through C/Fortran interface
void datetime_diff_f(const util::DateTime * dt1, const util::DateTime * dt2,
                       double & zd) {
  const util::Duration dd = *dt1 - *dt2;
  zd = dd.toSeconds();
}

// -----------------------------------------------------------------------------

// Using double to avoid passing bool through C/Fortran interface
void datetime_diff_str_f(const util::DateTime * dt1, const util::DateTime * dt2,
                         char * cstr) {
  const util::Duration dd = *dt1 - *dt2;
  const std::string ss(dd.toString());
// Can't think of any way to check cstr points to valid space in memory...
  strcpy(cstr,  ss.c_str());
}

// -----------------------------------------------------------------------------

void datetime_update_f(util::DateTime * tt, const util::Duration * dt) {
  *tt += *dt;
}

// -----------------------------------------------------------------------------

}  // namespace oops
