!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!>  Fortran module handling the interface to utils::Duration

module duration_mod

use iso_c_binding
use string_f_c_mod
use kinds

implicit none
private
public duration, duration_create, duration_delete, assignment(=), &
     & f_c_duration, c_f_duration, duration_seconds

!>  Derived type encapsulating a C++ Duration pointer.

type duration
  type(c_ptr) ptr
end type duration

interface assignment(=)
  module procedure duration_assign
end interface assignment(=)

#include "duration.intfb.h"

!-------------------------------------------------------------------------------
contains
!-------------------------------------------------------------------------------

!>  Create a Duration from an ISO8601 string.

subroutine duration_create(fstring, self)
implicit none
type(duration), intent(out)  :: self
character(len=*), intent(in) :: fstring
character(kind=C_CHAR,len=1) :: cstring(21)

call f_c_string(fstring, cstring)
self%ptr = duration_construct_f(cstring)

end subroutine duration_create

!-------------------------------------------------------------------------------

!> Delete a Duration

subroutine duration_delete(self)
implicit none
type(duration), intent(inout) :: self

call duration_destruct_f(self%ptr)

end subroutine duration_delete

!-------------------------------------------------------------------------------

!> Assign a Duration

subroutine duration_assign(self, other)
implicit none
type(duration), intent(out) :: self
type(duration), intent(in)  :: other
character(kind=C_CHAR,len=1), dimension(21) :: cstring

call duration_string_f(other%ptr,cstring)
self%ptr = duration_construct_f(cstring)

!call duration_copy_f(other%ptr, self%ptr)

end subroutine duration_assign

!-------------------------------------------------------------------------------

!> Convert a Fortran duration to a C++ Duration

subroutine f_c_duration(fdt, cdt)
implicit none
type(duration), intent(in) :: fdt
type(c_ptr), intent(out)   :: cdt
cdt = fdt%ptr
end subroutine f_c_duration

!-------------------------------------------------------------------------------

!> Convert a C++ Duration to a Fortran duration

subroutine c_f_duration(cdt, fdt)
implicit none
type(c_ptr), intent(in)     :: cdt
type(duration), intent(out) :: fdt
fdt%ptr = cdt
end subroutine c_f_duration

!-------------------------------------------------------------------------------

!> Get Duration in seconds as real

real(kind=kind_real) function duration_seconds(fdt)
implicit none
type(duration), intent(in) :: fdt
real(c_double) :: zdt

call duration_real_f(fdt%ptr, zdt)
duration_seconds = zdt

end function duration_seconds

!-------------------------------------------------------------------------------

!> Get Duration as string

subroutine duration_to_string(fdt, fstring)
implicit none
type(duration), intent(in) :: fdt
character(len=20), intent(out) :: fstring
character(kind=C_CHAR,len=1), dimension(21) :: cstring

call duration_string_f(fdt%ptr,cstring)
call c_f_string(cstring,fstring)

end subroutine duration_to_string

!-------------------------------------------------------------------------------

end module duration_mod
