!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!>  Fortran routines to convert Fortran character strings to/from c strings.


module ftn_c_string_conversion

use ISO_C_BINDING, only : C_CHAR, C_NULL_CHAR

implicit none

private

public ftn_to_c_string, &
     & alloc_and_copy_ftn_to_c_string, &
     & c_to_ftn_string, &
     & length_convert_ftn_to_c_string, &
     & length_convert_c_to_ftn_string

contains

!> Convert Fortran string to c string.

function ftn_to_c_string(ftnstring)
  implicit none
  character(len=*), intent(in)  :: ftnstring
  character(kind=C_CHAR, len=1) :: ftn_to_c_string(len(ftnstring)+1)
  integer i
  do i=1,len(ftnstring)
    ftn_to_c_string(i) = ftnstring(i:i)
  enddo
  ftn_to_c_string(length_convert_ftn_to_c_string(len(ftnstring))) = C_NULL_CHAR
end function ftn_to_c_string

!> Convert Fortran string to c string, allocating space for the latter.

subroutine alloc_and_copy_ftn_to_c_string(ftnstring,cstring)
  implicit none
  character(len=*), intent(in)  :: ftnstring
  character(kind=C_CHAR, len=1), intent(out), allocatable :: cstring(:)

  allocate(cstring(length_convert_ftn_to_c_string(len(ftnstring))))
  cstring = ftn_to_c_string(ftnstring)
end subroutine alloc_and_copy_ftn_to_c_string

!> Convert c string to Fortran string.

function c_to_ftn_string(cstring)
  implicit none
  character(kind=C_CHAR, len=1), intent(in)  :: cstring(:)
  character(len=size(cstring)-1):: c_to_ftn_string
  integer i
  do i=1,length_convert_c_to_ftn_string(size(cstring))
    c_to_ftn_string(i:i) = cstring(i)
  enddo
end function c_to_ftn_string

!> Convert length of fortran string to length of c string

integer function length_convert_ftn_to_c_string(len_of_ftn_string)
  implicit none
  integer :: len_of_ftn_string
  length_convert_ftn_to_c_string = len_of_ftn_string + 1
end function length_convert_ftn_to_c_string

!> Convert length of c string to length of fortran string

integer function length_convert_c_to_ftn_string(len_of_c_string)
  implicit none
  integer :: len_of_c_string
  length_convert_c_to_ftn_string = len_of_c_string - 1
end function length_convert_c_to_ftn_string

end module ftn_c_string_conversion
