!
! Copyright 2011 ECMWF
! 
! This software was developed at ECMWF for evaluation
! and may be used for academic and research purposes only.
! The software is provided as is without any warranty.
! 
! This software can be used, copied and modified but not
! redistributed or sold. This notice must be reproduced
! on each copy made.
!

!> Convert Fortran strings to/from C++

module string_f_c_mod

use iso_c_binding, only : C_CHAR, C_NULL_CHAR

implicit none
private
public f_c_string, c_f_string

contains

! ------------------------------------------------------------------------------
!> Convert Fortran string to C++ string.

subroutine f_c_string(fstring, cstring)
character(len=*), intent(in)               :: fstring
character(kind=C_CHAR, len=1), intent(out) :: cstring(:)
integer :: jj

if (size(cstring)<len(fstring)+1) call abor1_ftn('f_c_string: cstring too short')

do jj=1,len(fstring)
  cstring(jj) = fstring(jj:jj)
enddo
cstring(len(fstring)+1) = C_NULL_CHAR

end subroutine f_c_string

! ------------------------------------------------------------------------------
!> Convert C++ string to Fortran string.

subroutine c_f_string(cstring, fstring)
character(kind=C_CHAR, len=1), intent(in) :: cstring(:)
character(len=*), intent(inout)           :: fstring
integer :: jj

if (len(fstring)<size(cstring)-1) call abor1_ftn('c_f_string: fstring too short')

fstring=''
do jj=1,size(cstring)-1
  if (IACHAR(cstring(jj)) == 0) EXIT
  fstring(jj:jj) = cstring(jj)
enddo

end subroutine c_f_string

! ------------------------------------------------------------------------------

end module string_f_c_mod
